Package: CloneSeeker
Version: 1.0.16
Date: 2026-02-10
Title: Seeking and Finding Clones in Copy Number and Sequencing Data
Authors@R: c(person(given = "Mark", family = "Zucker", role = "aut"),
	    person(given = c("Kevin", "R."), family = "Coombes",
                   role = c("aut", "cre"), email = "krc@silicovore.com"))
Description: Defines the classes and functions used to simulate and
  to analyze data sets describing copy number variants and,
  optionally, sequencing mutations in order to detect clonal subsets.
  See Zucker et al. (2019) <doi:10.1093/bioinformatics/btz057>.
Depends: R (>= 3.0)
Imports: methods, graphics, combinat, mc2d, quantmod
License: Apache License (== 2.0)
URL: http://oompa.r-forge.r-project.org/
Author: Mark Zucker [aut],
  Kevin R. Coombes [aut, cre]
Maintainer: Kevin R. Coombes <krc@silicovore.com>
Repository: R-Forge
Repository/R-Forge/Project: clonefinder
Repository/R-Forge/Revision: 196
Repository/R-Forge/DateTimeStamp: 2026-07-06 13:08:50
Date/Publication: 2026-07-06 13:08:50
NeedsCompilation: no
Packaged: 2026-07-06 13:24:22 UTC; rforge

###
### NAMESPACE
###

### convert base functions or S3 methods into S4 methods
import(methods)
importFrom("graphics", "plot", "hist")
#exportMethods("plot", "hist") # come from 'graphics'
#exportMethods("summary")      # comes from 'base'

### other imported functions
importFrom("quantmod", "findPeaks")  # used in "seqSeg" in '03-utilities.R'
importFrom("mc2d", "rdirichlet", "ddirichlet") # used in '04-algorithms.R'
importFrom("graphics", "par", "segments")  # used in "plotTumorData",
importFrom("stats",
           "rbeta", "dbeta", "rbinom", "dbinom", "rnorm", "dnorm",
           "rmultinom", "runif", "dgeom",
           "density", "median", "sd", "na.omit")

### dealing with simplices
importFrom(combinat, "combn")    # used for combinations of psi-values
importFrom(combinat, "xsimplex") # used for generateSimplex
export("sampleSimplex", "generateSimplex")

### weight vectors
exportClasses("WeightVector")
export("WeightVector")

### tumors
exportClasses("Tumor")
export("Tumor", "getClone")
export("generateTumorData", "plotTumorData")
export("tumorGen", "dataGen", "runAlg") # deprecated

### algorithm.R
export("seekClones")

####################### IN PROGRESS ######################
### needed for simulations; currently internal
# exportClasses("Clone")
# export("Clone")

---
author: Kevin R. Coombes
date: 12 October 2018
title: Auxiliary Material for CloneFinfer
---

# Overview
This directory contains a combination of material that either
(1) constructs data sets that are part of the package or (2) performs
some of the analyses described in our manuscript.

## Package Material

1. `chlens.txt` is a simple table holding the lengths In base pairs)
   of human chromosomes. It is based on build [UNKNOWN} and was
   initialy obtained from [UNKNOWN].
2. `makeChlens.R` is a script that reads the previous file, converts
   it to a useful form, and saves it in the `sysdata.rda` file so it
   can be used by the package. In particular, it is used by functions
   that simulate `Tumor` objects to create  realistic segmentation
   data.
3. `makeTestExamples.R` is a script that simulates copy number and
   mutation data used for regression testing of the algorithms during
   package development. 

## Analyses

1. `simgen.R` is the script that created the original simulated data
   sets used in our paper to compare the CloneSeeker algorithm to other
   algorithms intended to detect clones.
2. `running.R` analyzes all th simulated data.
3. `asswssing.R` compares the infered results feomthe the algorithms
   to the true properties of the simulated adta.
4. `clinical.R` applies th CloneSeeker alorithm to CLL data.
5. `cnv_survival.R` tsts the associations of subclones with clinical
   variables in CLL.
6. ``figures.R` generates the figures in the manuscript.

chdata <- read.table("chlens.txt", sep="\t") # chromnumber, length
chnames <- paste("chr", c(1:22, "X", "Y"), sep="")
chdata$Chrom <- factor(chnames[chdata$V1], levels = chnames)
chlens <- as.vector(tapply(chdata$V2, list(chdata$Chrom), max))
names(chlens) <- chnames

save(chlens, file = "../../Data/sysdata.rda")

library(CloneFinder)
library(foreach)
library(doParallel)
#KRC: source('parOpt/clonefinder/functions.R')
#KRC: source('parOpt/clonefinder/algorithm.R')

convert.sc <- function(mutdata, cndata){
  segs <- mutdata$seg
  segs.unique <- unique(segs)
  sciclone.cn <- as.data.frame(t(sapply(1:nrow(cndata), function(k){
    start <- sum(cndata$markers[1:k])+1 - cndata$markers[k]
    end <- sum(cndata$markers[1:k])
    c('chr'=cndata$chr[k], 'start'=start, 'stop'=end, 'segment_mean'=cndata$X[k] + cndata$Y[k])
  })))
  #randomly generate loci for mutations on their respective segments:
  if(length(segs.unique)>0){
    mutcn <- round(t(sapply(1:nrow(mutdata), function(k){
      sort(c(cndata$X[segs[k]], cndata$Y[segs[k]]), decreasing=TRUE)
    })))
    colnames(mutcn) <- c('A', 'B')
    genotypes <- sapply(1:nrow(mutcn),function(k){
      paste(c(rep(colnames(mutcn)[1], mutcn[k,1]), rep(colnames(mutcn)[2], mutcn[k,2])), collapse='')
    })
    loci <- unlist(lapply(1:length(segs.unique),function(k){
      minim <- sciclone.cn$start[segs.unique[k]]
      maxim <- sciclone.cn$stop[segs.unique[k]]
      n <- length(which(segs==segs.unique[k]))
      sample(minim:maxim, n, replace=FALSE)
    }))
  }else{
    loci <- numeric(0)
  }
  sciclone.dat <- data.frame('chr'=mutdata$chr, 'start'=loci, 'refCounts'=mutdata$refCounts, 
                             'varCounts'=mutdata$varCounts, 'VAF'=mutdata$VAF, 'mutid'=mutdata$mut.id)
  list('sciclone.dat'=sciclone.dat, 'sciclone.cn'=sciclone.cn)
}

convert.exp <- function(mutdata, cndata){
  segs <- mutdata$seg
  segs.unique <- unique(segs)
  start <- sapply(1:nrow(cndata),function(k){sum(cndata$markers[1:k])+1 - cndata$markers[k]})
  end <- sapply(1:nrow(cndata),function(k){sum(cndata$markers[1:k])})
  CBS <- data.frame('chr'=cndata$chr, 'startpos'=start, 'endpos'=end, 'CN_Estimate'=cndata$X+cndata$Y)
  if(length(segs.unique)>0){
    loci <- unlist(lapply(1:length(segs.unique),function(k){
      minim <- CBS$startpos[segs.unique[k]]
      maxim <- CBS$endpos[segs.unique[k]]
      n <- length(which(segs==segs.unique[k]))
      sample(minim:maxim, n, replace=FALSE)
    }))
  }else{
    loci <- numeric(0)
  }
  mutids <- mutdata$mut.id
  SNV <- data.frame('chr'=mutdata$chr, 'startpos'=loci, 'AF_Tumor'=mutdata$VAF, 'PN_B'=rep(0, nrow(mutdata)), 'mutid'=mutids)
  list('SNV'=as.matrix(SNV), 'CBS'=as.matrix(CBS))
}

runAlgs <- function(i, set, minim.vaf, datapath, respath, mu, cf=TRUE, sc=FALSE, exp=TRUE, cn=TRUE, mix, real.dat, run, pars){
  datafiles <- list.files(datapath)
  dat <- get(load(paste(datapath, '/', datafiles[i], sep='')))
  number <- strsplit(strsplit(datafiles[i], split='-')[[1]][3], split='.rda')[[1]][1]
  if(mix | real.dat){
    number <- strsplit(strsplit(datafiles[i], split='-')[[1]][2], split='.rda')[[1]][1]
  }
  if(length(dat)==1){
    vardata <- dat$dat$seq.data
    cndata <- dat$dat$cn.data
  }else{
    vardata <- dat$seq.data
    cndata <- dat$cn.data
  }
  mutdata <- vardata[vardata$status=='somatic',]
  mut.filt <- filter.mut(mutdata, mu=mu, threshold=3)
  mutdata.filt <- mut.filt$mat
  mutdata.filt <- mutdata.filt[mutdata.filt$chr %in% 1:22,]
  mutdata.filt <- mutdata.filt[which(mutdata.filt$VAF > minim.vaf/2),]
  if(cn){
    if(cf){
      t1 <- Sys.time()
      res.cf <- runAlg(cndata, vardata, cnmodels, psisets[[2]], pars=pars, priorbank=NULL)
      t2 <- Sys.time()
      runtime.cf <- difftime(t2, t1, units='secs')
      res.cf <- list('res'=res.cf, 'runtime'=runtime.cf)
      if(!is.null(run)){
        filename <- paste(respath, '/res-cf-', set, run, '/res-cf-', set, '-',  number, '.rda', sep='')
      }else{
        filename <- paste(respath, '/res-cf-', set, '/res-cf-', set, '-',  number, '.rda', sep='')
      }
      save(res.cf, file=filename)
    }
    if(exp){
      inputs.exp <- convert.exp(mutdata.filt, cndata)
      t1 <- Sys.time()
      res.exp <- runExPANdS(inputs.exp$SNV,inputs.exp$CBS, precision=.04, min_CellFreq = minim.vaf)
      t2 <- Sys.time()
      runtime.exp <- difftime(t2, t1, units='secs')
      res.exp <- list('res'=res.exp, 'runtime'=runtime.exp)
      filename <- paste(respath, '\\res-exp-', set,'\\res-exp-', set, '-',  number, '.rda', sep='')
      save(res.exp, file=filename)
    }
    if(sc){
      inputs.sc <- convert.sc(mutdata.filt, cndata)
      t1 <- Sys.time()
      res.sc <- sciClone(inputs.sc$sciclone.dat, inputs.sc$sciclone.cn, sampleNames=paste('sample-',i, sep=''), maximumClusters = 10, minimumDepth = 50)
      t2 <- Sys.time()
      runtime.sc <- difftime(t2, t1, units='secs')
      res.sc <- list('res'=res.sc, 'runtime'=runtime.sc)
      filename <- paste(respath, '/res-sc-', set,'/res-sc-', set, '-', number, '.rda', sep='')
      save(res.sc, file=filename)
    }
  }else{
    if(cf){
      t1 <- Sys.time()
      res.cf <- runAlg(cndata, vardata, cnmodels, psisets[[2]], pars=pars, priorbanks=NULL)
      t2 <- Sys.time()
      runtime.cf <- difftime(t2, t1, units='secs')
      res.cf <- list('res'=res.cf, 'runtime'=runtime.cf, 'parameters'=c(list('mu'=mu,'minim.vaf'=minim.vaf),pars))
      filename <- paste(respath, '/res-cf-', set, '/res-cf-', set, '-',  number, '.rda', sep='')
      save(res.cf, file=filename)
    }
    if(exp){
      inputs.exp <- convert.exp(mutdata.filt, cndata)
      t1 <- Sys.time()
      res.exp <- res.exp <- runExPANdS(inputs.exp$SNV, precision=.04, min_CellFreq = .05)
      t2 <- Sys.time()
      runtime.exp <- difftime(t2, t1, units='secs')
      res.exp <- list('res'=res.exp, 'runtime'=runtime.exp)
      filename <- paste(respath, '\\res-exp-', set,'\\res-exp-', set, '-',  number, '.rda', sep='')
      save(res.exp, file=filename)
    }
    if(sc){
      inputs.sc <- convert.sc(mutdata.filt, cndata)
      t1 <- Sys.time()
      res.sc <- sciClone(inputs.sc$sciclone.dat, sampleNames=paste('sample-',i, sep=''), maximumClusters = 10, minimumDepth = 50)
      t2 <- Sys.time()
      runtime.sc <- difftime(t2, t1, units='secs')
      res.sc <- list('res'=res.sc, 'runtime'=runtime.sc)
      filename <- paste(respath, '/res-sc-', set,'/res-sc-', set, '-', number, '.rda', sep='')
      save(res.sc, file=filename)
    }
  }
}


###SciClone and Clonefinder, on the cluster:
#Loading and defining stuff:
kmax <- 5
load('parOpt/clonefinder/cnmodels.rda')
load('parOpt/clonefinder/psis.10.rda')
load('parOpt/clonefinder/psis.20.rda')
load('parOpt/clonefinder/psis.50.rda')
load('parOpt/clonefinder/psis.100.rda')
load('parOpt/clonefinder/etamat.10.rda')
load('parOpt/clonefinder/etamat.20.rda')
load('parOpt/clonefinder/etamat.50.rda')
psisets <- list(psis.10, psis.20,psis.50,psis.100)
etasets <- list(etamat.10, etamat.20,etamat.50)

setNumber <- 'CLL'
dpath <- paste('parOpt/dat/data-', setNumber, sep='')
rpath <- paste('parOpt/res/res-', setNumber, sep='')
#metadata <- get(load(paste('parOpt/sim/sims-',setNumber,'/metadata-',setNumber,'.rda', sep='')))
cf.var <- TRUE
#mix <- real.dat <- sc.var <- FALSE
cf.var <- real.dat <- TRUE
mix <- sc.var <- FALSE
run <- 5
X <- list(1:262)
nodes <- 28
parlist <- list(list('sigma0'=.4, 'theta'=.5, 'ktheta'=.3, 'mtheta'=.5, 'alpha'=.5, 'thresh'=0, 'cutoff'=40, 'Q'=100, 'iters'=4),
                list('sigma0'=.3, 'theta'=.5, 'ktheta'=.3, 'mtheta'=.5, 'alpha'=.5, 'thresh'=0, 'cutoff'=40, 'Q'=100, 'iters'=4),
                list('sigma0'=8, 'theta'=.5, 'ktheta'=.5, 'mtheta'=.5, 'alpha'=.5, 'thresh'=.04, 'cutoff'=500, 'Q'=100, 'iters'=4),
                list('sigma0'=6, 'theta'=.6, 'ktheta'=.6, 'mtheta'=.5, 'alpha'=.5, 'thresh'=.04, 'cutoff'=500, 'Q'=100, 'iters'=4),
                list('sigma0'=5, 'theta'=.9, 'ktheta'=.3, 'mtheta'=.9, 'alpha'=.5, 'thresh'=.04, 'cutoff'=100, 'Q'=100, 'iters'=4),
                list('sigma.lrr'=.15,'sigma.baf'=.03, 'sigmaM'=.14, 'theta'=.9, 'ktheta'=.9, 'mtheta'=.9, 'alpha'=.5, 'cutoff'=500, 'thresh'=.04, 'Q'=100, 'iters'=4))
pars <- parlist[[5]]

f1 <- function(setNumber, cf.var, sc.var, nodes, mix, real.dat, run, pars, X){
  dpath <- paste('parOpt/dat/data-', setNumber, sep='')
  rpath <- paste('parOpt/res/res-', setNumber, sep='')
  cl <- makeForkCluster(nodes)
  registerDoParallel(cl)
  foreach(x=unlist(X),.export=c('mix','real.dat', 'kmax', 'psisets','etasets','cnmodels', 'runAlgs', 'convert.exp','setNumber','parlist',
                                'psiOptim2','psiFilter','filter','dpath','rpath','filter.mut','runAlg','ddirichlet',
                                'rdirichlet','dbeta2','estBetaParams'), .packages=c('gtools')) %dopar% {
    #mu <- metadata$data.params$mu
    runAlgs(x, set=setNumber, cn=TRUE, minim.vaf=.04, datapath=dpath, respath=rpath, mu=70, cf=cf.var, 
            sc=sc.var, exp=FALSE, mix=mix, real.dat=real.dat, run=run, pars)                      
  }
  parLapply(cl,6:33,function(x){
    runAlgs(x, set=setNumber, cn=TRUE, minim.vaf=.04, datapath=dpath, respath=rpath, mu=70, cf=cf.var, 
            sc=sc.var, exp=FALSE, mix=mix, real.dat=real.dat, run=run, pars)
  })
}

f1('CLL', cf.var=TRUE, sc.var=FALSE, nodes=28, X=list(1:262), mix=FALSE, real.dat=TRUE, run=5, pars=parlist[[5]])
#Best pars so far:
#list('sigma0'=.4, 'theta'=.3, 'ktheta'=.5, 'ntheta'=.5, 'stheta'=.5, 'mtheta'=.5, 'alpha'=.5, 'thresh'=0)
#list('sigma0'=.4, 'theta'=.5, 'ktheta'=.5, 'ntheta'=.5, 'stheta'=.5, 'mtheta'=.5, 'alpha'=.5, 'thresh'=0)
#Maybe: list('sigma0'=.4, 'theta'=.7, 'ktheta'=.9, 'ntheta'=.5, 'stheta'=.5, 'mtheta'=.5, 'alpha'=.5, 'thresh'=0

#Mixtures:
f1('mix', cf.var=TRUE, sc.var=FALSE, nodes=28, X=list(1:300), mix=TRUE, real.dat=FALSE, run=NULL, pars=parlist[[5]])


###For parameter optimization of algorithm.
source('parOpt/clonefinder/algorithm.R')
sampleIndices <- t(sapply(1:10,function(i){
  seq(from=i,to=300,by=12)
}))
sigma.lrr.range <- c(.01,.2)
sigma.baf.range <- c(.01,.1)
theta.range <- c(.01,.99)
ktheta.range <- c(.01,.99)
ranges <- c(sigma.lrr.range[2]-sigma.lrr.range[1],sigma.baf.range[2]-sigma.baf.range[1],
            theta.range[2]-theta.range[1],ktheta.range[2]-ktheta.range[1])
pars1 <- list('sigma.lrr'=sigma.lrr.range[1],'sigma.baf'=sigma.baf.range[1],'theta'=theta.range[1],
           'ktheta'=ktheta.range[1],'mtheta'=.9, 'alpha'=.5, 'thresh'=.04, 'cutoff'=100, 'Q'=100, 'iters'=4)
iterations.1 <- 4
iterations.2 <- 3
dpath <- 'parOpt/dat/testdata/'

binOpt <- function(i){
  cl <- makeForkCluster(length(sampleIndices[[i]]))
  registerDoParallel(cl)
  output <- foreach(x=unlist(sampleIndices[i,]),.export=c('kmax', 'psisets','etasets','cnmodels', 'runAlgs',
      'psiOptim2','psiFilter','filter','dpath','filter.mut','runAlg','ddirichlet',
      'rdirichlet','dbeta2','estBetaParams','pars1','ranges','iterations.1','iterations.2'), .packages=c('gtools')) %dopar% {
        dat <- get(load(paste('parOpt/dat/testdata/mixdat-',x,'.rda', sep='')))
        sim <- get(load(paste('parOpt/sim/testmixtures/mixsim-',x,'.rda', sep='')))
        truePsi <- sim$psi
        trueK <- length(which(truePsi>0))
        parset1 <- pars1
        cndata <- dat$cn.data
        res.cf.base <- runAlg(cndata, vardata=NULL, cnmodels, psisets[[2]], pars=parset1, priorbank=NULL)
        kdist.base <- abs(ks-trueK)
        psidist.base <- sum((res.cf$psi-truePsi)^2)^.5
        for(z in 1:iterations.2){
          for(n in 1:4){
            interval <- ranges[n]
            for(j in 1:iterations.1){
              newpars <- parset1
              newpars[n] <- parset1[n] + interval
              res.cf <- runAlg(cndata, vardata=NULL, cnmodels, psisets[[2]], pars=newpars, priorbank=NULL)
              ks <- length(which(res.cf$psi>0))
              kdist <- abs(ks-trueK)
              psidist <- sum((res.cf$psi-truePsi)^2)^.5
              if(psidist<psidist.base){
                parset1 <- newpars
                sign <- -1*sign
              }
              interval <- sign*ranges[n]/j
            }
            parset1 <- newpars
          }
        }
        list('pars'=pars,'kdist'=kdist,'psidist'=psidist)
      }
  output
}

temp <- binOpt(1)
save(temp,file='parOpt/binoptim.rda')

###Uniform sampling:
constants <- c('mtheta'=.9, 'alpha'=.5, 'thresh'=.04, 'cutoff'=100, 'Q'=100, 'iters'=4)
g <- function(index.list,j){
  sigmas.lrr <- runif(j,sigma.lrr.range[1],sigma.lrr.range[2])
  sigmas.baf <- runif(j,sigma.baf.range[1],sigma.baf.range[2])
  thetas <- runif(j,theta.range[1],theta.range[2])
  kthetas <- runif(j,ktheta.range[1],ktheta.range[2])
  pardf <- data.frame('sigma.lrr'=sigmas.lrr,'sigma.baf'=sigmas.baf,'theta'=thetas,'ktheta'=kthetas)
  par.index <- rep(1:j,length(index.list))
  cl <- makeForkCluster(length(index.list))
  output <- parLapply(cl,1:(j*length(index.list)),function(k){
    x <- ceiling(k/length(i))
    y <- par.index[k]
    pars <- pardf[y,]
    pars <- as.list(c(pars,constants))
    dat <- get(load(paste('parOpt/dat/testdata/mixdat-',x,'.rda', sep='')))
    cndata <- dat$cn.data
    sim <- get(load(paste('parOpt/sim/testmixtures/mixsim-',x,'.rda', sep='')))
    truePsi <- sim$psi
    trueK <- length(which(truePsi>0))
    pars <- c(constants,pardf[y,])
    res.cf <- runAlg(cndata, vardata=NULL, cnmodels, psisets[[2]], pars=pars, priorbank=NULL)
    kdiff <- length(which(res.cf$psi>0))-trueK
    psi <- res.cf$psi
    if(length(psi)>length(truePsi)){
      truePsi <- c(truePsi,rep(0,length(psi)-length(truePsi)))
    }
    psidist <- sum((psi-truePsi)^2)^.5
    list('pars'=pars,'k'=length(which(res.cf$psi>0)),'kdiff'=kdiff,'psidist'=psidist,'x'=x,'y'=y)
  })
  output
}

t1 <- Sys.time()
run1 <- g(index.list=sampleIndices[1,],20)
t2 <- Sys.time()
rt <- difftime(t2,t1)
save(run1,file='parOpt/run1.rda')

library(survival)
library(prodlim)
library(quantmod)
cll.path <- 'C:/Users/Mark/OneDrive - The Ohio State University/clonetools/res/res-CLL'
cll.files <- list.files(cll.path)
samples <- sapply(1:length(cll.files),function(j){strsplit(strsplit(cll.files[j],split='-')[[1]][4],split='.rda')[[1]][1]})
cf.results <- lapply(1:length(cll.files), function(j){get(load(paste(cll.path, '/', cll.files[j], sep='')))$res})
cll.ids <- sapply(1:length(cll.files),function(j){strsplit(strsplit(cll.files[j],split='-')[[1]][4],split='.rda')[[1]][1]})
pars <- cf.results[[1]]$pars
rewind <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/SNP_array_data/rewind_both.rda'))
total.markers <- sum(rewind[rewind$SamID==samples[1],]$num.mark)
clinical <- get(load('C:/Users/Mark/Documents/Lab/clinok.rda'))
#clinical <- get(load('E:/CLL/ClinicalOK.rda'))
eligible <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clinical/eligible.rda'))
clinical$ttp.eligible <- eligible
cids <- rownames(clinical)
common.cids <- intersect(c(cids),samples)
clinical <- clinical[sapply(1:length(common.cids),function(j){which(rownames(clinical)==common.cids[j])}),]
cf.results <- lapply(1:length(common.cids),function(j){cf.results[[which(samples==common.cids[j])]]})
ttp.eligible <- which(clinical$ttp.eligible)

#Table of characteristics: age, ethnicity, sex, Rai stage
race <- table(clinical$Race)
names(race) <- c('Asian','Black','Hispanic','White')
sex <- table(clinical$Sex)
names(sex) <- c('Female','Male')
mutstatus <- table(clinical$mutation.status)
names(mutstatus) <- c('Mutated','Unmutates')
rai <- table(clinical$Rai.Stage)
zap70 <- table(clinical$Zap70Protein)
median.age.dx <- median(clinical$AgeAtDx)
dohner <- table(clinical$Dohner)
timeSam2Treat <- median(clinical$TimeDiagnosis2SigTreat)
df <- data.frame('NumberofPatients'=c(race,sex,mutstatus,rai,zap70,median.age.dx,dohner,timeSam2Treat))
namesvec <- c(names(race), names(sex), names(mutstatus), names(rai), names(zap70), "MedianAgeatDiagnosis", names(dohner), "TimefromSampletoTreatment")
#rownames(df) <- namesvec
#save(df, file='C:/Users/Mark/OneDrive - The Ohio State University/clonetools/df.tsv')

###Common alteration CNV fractions:
#Seperate genome into common segments according to all segment breaks
sams <- unique(rewind$SamID)
chrs <- unique(rewind$chrom)

###
ks <- sapply(1:length(cf.results),function(j){length(which(cf.results[[j]]$psi>0))})
columns <- colnames(clinical)

alt.frac <- sapply(1:length(cf.results),function(j){
  temp <- rewind[rewind$SamID==common.cids[j],]
  notna <- which(!is.na(temp$loc.end) & !is.na(temp$loc.start))
  all.lens <- sum(as.numeric(temp$loc.end[notna]) - as.numeric(temp$loc.start[notna]))
  alt <- which(sapply(1:nrow(cf.results[[j]]$A),function(k){
    unique(all.equal(unname(cf.results[[j]]$A[k,1:length(which(cf.results[[j]]$psi>0))]), rep(1, length(which(cf.results[[j]]$psi>0))))!=TRUE) & 
      unique(all.equal(unname(cf.results[[j]]$B[k,1:length(which(cf.results[[j]]$psi>0))]), rep(1, length(which(cf.results[[j]]$psi>0))))!=TRUE)
  }))
  lens <- unlist(sapply(1:length(alt), function(k){
    index <- which(rewind$seg.median==cf.results[[j]]$filtered.data$cndata.filt$LRR[alt[k]] &
                   rewind$AvgBAF==cf.results[[j]]$filtered.data$cndata.filt$BAF[alt[k]])
    rewind$loc.end[index] - rewind$loc.start[index]
  }))
  sum(as.numeric(lens))/all.lens
})

k.status <- af.status <- rep(0,length(ks))
names(ks) <- names(alt.frac) <- common.cids
k.status[ks>1] <- 1
af.status[alt.frac>median(alt.frac)] <- 1

merge <- function(vector){
  if(length(vector)>1){
    starts <- rep(NA, length(vector))
    ends <- starts
    diffs <- diff(vector)
    stop <- FALSE
    curr <- 1
    index <- 1
    start <- vector[1]
    end <- vector[1]
    while(stop==FALSE) {
      if(diffs[index]==1){
        end <- vector[index+1]
      }else{
        starts[curr] <- start
        ends[curr] <- end
        start <- vector[index+1]
        end <- start
        curr <- curr + 1
      }
      if(index+1>=length(vector)){
        stop <- TRUE
        if(diffs[index]==1){
          starts[curr] <- start
          ends[curr] <- end
        }
      }
      index <- index+1
    }
    if(vector[length(vector)] - vector[length(vector)-1] >1){
      starts[length(na.omit(starts))+1] <- vector[length(vector)]
      ends[length(na.omit(ends))+1] <- vector[length(vector)]
    }else{
      ends[length(na.omit(ends))] <- vector[length(vector)]
    }
    df <- data.frame('start'=na.omit(starts), 'end'=na.omit(ends))
  }else{
    df <- data.frame('start'=vector[1], 'end'=vector[1])
  }
  df
}

marker.thresh <- 3000
frac.thresh <- .2
complex <- sapply(1:length(cf.results),function(j){
  segs <- nrow(cf.results[[j]]$A)
  psi <- cf.results[[j]]$psi
  markers <- cf.results[[j]]$filtered.data$cndata.filt$markers
  cns <- cf.results[[j]]$A + cf.results[[j]]$B
  gainfrac <- sapply(1:segs,function(k){sum(psi[which(cns[k,]>2)])})
  lossfrac <- sapply(1:segs,function(k){sum(psi[which(cns[k,]<2)])})
  merged.gain <- na.omit(merge(which(gainfrac>frac.thresh)))
  merged.loss <- na.omit(merge(which(lossfrac>frac.thresh)))
  gain.sizes <- numeric(0)
  loss.sizes <- numeric(0)
  if(nrow(merged.gain)>0){
    gain.sizes <- sapply(1:nrow(merged.gain),function(k){sum(markers[merged.gain$start[k]:merged.gain$end[k]])})
  }
  if(nrow(merged.loss)>0){
    loss.sizes <- sapply(1:nrow(merged.loss),function(k){sum(markers[merged.loss$start[k]:merged.loss$end[k]])})
  }
  length(which(gain.sizes>marker.thresh)) + length(which(loss.sizes>marker.thresh)) >= 3
})

###Treatment (significant):
#vars; note: time variables are in months (365.25/12).
White.blood.count <- clinical[,which(columns=='White.blood.count')]
Rai.Stage <- clinical[,which(columns=='Rai.Stage')]
TimeSample2SigTreat <- clinical[,which(columns=='TimeSample2SigTreat')]
TimeDiag2SigTreat <- clinical$TimeDiagnosis2SigTreat
PFS.time <- clinical[,which(columns=='PFS.time')]
EFS.time <- clinical[,which(columns=='EFS.time')]
mutation.status <- clinical[,which(columns=='mutation.status')]
OSAD <- clinical[,which(columns=='OSAfterDiagnosis')]
OSAS <- clinical[,which(columns=='OSAfterSample')]
TTP <- clinical[,which(columns=='TTP.time')]
TxDSS <- clinical[,which(columns=='TxDSS.time')]

#status (numeric)
NumericSigTreatment <- clinical[,which(columns=='NumericSigTreatment')]
PFS.numericStatus <- clinical[,which(columns=='PFS.numericStatus')]
EFS.numericStatus <- clinical[,which(columns=='EFS.numericStatus')]
NumericVitalStatus <- clinical[,which(columns=='NumericVitalStatus')]
NumericTTPStatus <- clinical[,which(columns=='TTP.numericStatus')]
NumericTxDSSStatus <- clinical[,which(columns=='TxDSS.numericStatus')]

########Cox Proportional Hazards:
colors <- c('red', 'blue', 'purple', 'darkorange')
Multiclonal <- k.status
Complex <- rep(NA, nrow(clinical))
Complex[which(clinical$CatCyto=='Simple')] <- 0
Complex[which(clinical$CatCyto=='Complex')] <- 1
High.AF <- af.status
Clones <- ks
Clones <- as.character(ks)
Clones[Clones=='3' | Clones=='4'] <- '3+'
Clones <- as.factor(Clones)
K <- Clones

###Time to significant treatment
t2st.karyo <- prodlim(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ Multiclonal + Complex)
diff <- survdiff(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ Multiclonal + Complex)
p_t2st.karyo <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(t2st.karyo, logrank=TRUE)

t2st.AF <- prodlim(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ Multiclonal + High.AF)
diff <- survdiff(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ Multiclonal + High.AF)
p_t2st.AF <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(t2st.AF, logrank=TRUE)

t2st <- prodlim(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ Multiclonal)
diff <- survdiff(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ Multiclonal)
p_t2st <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(t2st, logrank=TRUE)

t2st.int <- prodlim(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ K)
diff <- survdiff(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ K)
p_t2st.int <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)

###OSAS:
osas.karyo <- prodlim(Surv(OSAS, NumericVitalStatus) ~ Multiclonal + Complex)
diff <- survdiff(Surv(OSAS, NumericVitalStatus) ~ Multiclonal + Complex)
p_osas.karyo <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(osas.karyo, logrank=TRUE)

osas.AF <- prodlim(Surv(OSAS, NumericVitalStatus) ~ Multiclonal + High.AF)
diff <- survdiff(Surv(OSAS, NumericVitalStatus) ~ Multiclonal + High.AF)
p_osas.AF <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(osas.AF, logrank=TRUE)

osas <- prodlim(Surv(OSAS, NumericVitalStatus) ~ Multiclonal)
diff <- survdiff(Surv(OSAS, NumericVitalStatus) ~ Multiclonal)
p_osas <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(osas, logrank=TRUE)

osas.int <- prodlim(Surv(OSAS, NumericVitalStatus) ~ K)
diff <- survdiff(Surv(OSAS, NumericVitalStatus) ~ K)
p_osas.int <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)

###Progression free survival
ttp.time <- TTP[ttp.eligible]
TTP.status <- NumericTTPStatus[ttp.eligible]
Multiclonal <- Multiclonal[ttp.eligible]
Complex <- Complex[ttp.eligible]
High.AF <- High.AF[ttp.eligible]
K <- K[ttp.eligible]

ttp.karyo <- prodlim(Surv(ttp.time, TTP.status) ~ Multiclonal + Complex)
diff <- survdiff(Surv(ttp.time, TTP.status) ~ Multiclonal + Complex)
p_ttp.karyo <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(ttp.karyo, logrank=TRUE)

ttp <- prodlim(Surv(ttp.time, TTP.status) ~ Multiclonal)
diff <- survdiff(Surv(pfs.time, TTP.status) ~ Multiclonal)
p_ttp <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(ttp, logrank=TRUE)

ttp.int <- prodlim(Surv(ttp.time, TTP.status) ~ K)
diff <- survdiff(Surv(ttp.time, TTP.status) ~ K)
p_ttp.int <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)

###OSAT:
osat.time <- TxDSS[ttp.eligible]
vitalStatus <- NumericVitalStatus[ttp.eligible]
osat.karyo <- prodlim(Surv(osat.time, vitalStatus) ~ Multiclonal + Complex)
diff <- survdiff(Surv(osat.time, vitalStatus) ~ Multiclonal + Complex)
p_osat.karyo <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(osat.karyo, logrank=TRUE)

osat.AF <- prodlim(Surv(osat.time, vitalStatus) ~ Multiclonal + High.AF)
diff <- survdiff(Surv(osat.time, vitalStatus) ~ Multiclonal + High.AF)
p_osat.AF <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(osat.AF, logrank=TRUE)

osat <- prodlim(Surv(osat.time, vitalStatus) ~ Multiclonal)
diff <- survdiff(Surv(osat.time, vitalStatus) ~ Multiclonal)
p_osat <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
plot(osat, logrank=TRUE)

osat.int <- prodlim(Surv(osat.time, vitalStatus) ~ K)
diff <- survdiff(Surv(osat.time, vitalStatus) ~ K)
p_osat.int <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)


###Combined plots
K <- Clones
png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/surv_int.png',height=700,width=2100,res=140)
par(mfrow=c(1,3))
plot(t2st.int, logrank=TRUE)
title("Time to Treatment",cex=1.5)
K <- K[ttp.eligible]
plot(pfs.int, logrank=TRUE)
title("Progression-Free Survival",cex=1.5)
plot(osat.int, logrank=TRUE)
title("Overall Survival after Treatment",cex=1.5)
dev.off()

#Among 2-cloners, does the size of the smaller subclone relate to clinical outcome?
indices <- which(ks==2)
psis <- t(sapply(1:length(cf.results),function(j){cf.results[[j]]$psi}))
minor.psis <- psis[,2]
minor.psi.status <- rep(0,length(minor.psis))
cutoff <- .2
minor.psi.status[which(minor.psis>cutoff)] <- 1
minor.psi.status2 <- minor.psi.status[indices]
survosas2 <- OSAS[indices]
survt2st2 <- TimeDiag2SigTreat[indices]
survttp2 <- TTP[intersect(indices,ttp.eligible)]
nvs <- NumericVitalStatus[indices]
nst <- NumericSigTreatment[indices]
nttps <- NumericTTPStatus[intersect(indices,ttp.eligible)]
minor.eligible <- minor.psis[intersect(indices,ttp.eligible)]
minor.eligible.status2 <- minor.psi.status[intersect(indices,ttp.eligible)]

osas.2 <- prodlim(Surv(survosas2, nvs)~minor.psi.status2)
t2st.2 <- prodlim(Surv(survt2st2,nst)~minor.psi.status2)
ttp.2 <- prodlim(Surv(survttp2,nttps)~minor.eligible.status2)
plot(osas.2,logrank =TRUE)
plot(t2st.2,logrank =TRUE)
plot(ttp.2,logrank =TRUE)

library(prodlim)
library(survival)
###Figure 1:
plotSetTraits <- function(setDir, seq=TRUE, cn=TRUE, merge=FALSE, kmax=5){
  simfiles <- list.files(setDir)
  metadata <- simfiles[1]
  simfiles <- simfiles[2:length(simfiles)]
  sims <- lapply(1:length(simfiles),function(j){get(load(paste(setDir, '\\', simfiles[j], sep='')))})
  if(length(sims[[1]])==1){
    sims <- lapply(1:length(sims),function(x){sims[[x]]$tumor})
  }
  ks <- sapply(1:length(sims),function(j){length(which(sims[[j]]$psi>0))})
  psis <- t(sapply(1:length(sims),function(j){
    psi <- sims[[j]]$psi
    if(length(psi)<kmax){
      psi <- c(psi, rep(0, kmax-length(psi)))
    }
    psi
  }))
  frac.altered <- sapply(1:length(sims),function(j){
    sim <- sims[[j]]
    lengths <- sim$clones[[1]]$cn$end + 1 - sim$clones[[1]]$cn$start
    sum(lengths[unique(unlist(lapply(1:length(sim$clones),function(x){
      which(sim$clones[[x]]$cn$A!=1 | sim$clones[[x]]$cn$B!=1)})))])/sum(lengths)
  })
  if(merge){
    sim <- sims[[j]]
    lengths <- sim$clones[[1]]$cn$end + 1 - sim$clones[[1]]$cn$start
    altered <- sapply(1:length(sim$clones),function(x){sim$clones[[x]]$cn$A!=1 | sim$clones[[x]]$cn$B!=1})
    altered.indices <- which(altered)
    diffs <- diff(altered.indices)
    #To be continued?
  }else{
    length.altered <- lapply(1:length(sims),function(j){
      sim <- sims[[j]]
      lengths <- sim$clones[[1]]$cn$end + 1 - sim$clones[[1]]$cn$start
      lengths[unique(unlist(lapply(1:length(sim$clones),function(x){
        which(sim$clones[[x]]$cn$A!=1 | sim$clones[[x]]$cn$B!=1)})))]
    })
    nCNVs <- sapply(1:length(length.altered),function(z){length(length.altered[[z]])}) 
  }
  length.altered <- unlist(length.altered)
  mutations <- sapply(1:length(sims),function(j){
    sim <- sims[[j]]
    if(length(sim)==1){
      sim <- sim$tumor
    }
    length(unique(unlist(lapply(1:length(sim$clones), function(x){sim$clones[[x]]$seq$mut.id}))))
  })
  if(cn==TRUE & seq==FALSE){
    plot(density(unlist(psis),bw=.05), xlab='Psi', xlim=c(0,1), main='F) Density of Psi Values across Simulations')
    plot(density(frac.altered,bw=.01), xlab='Genome Altered Fraction', xlim=c(0,1), main='G) Genome AF Density among Simulations')
  }else if(cn==FALSE & seq==TRUE){
    plot(density(unlist(psis),bw=.05), xlab='Psi', xlim=c(0,1), main='A) Density of Psi Values across Simulations')
    plot(density(mutations,bw=5), xlab='Mutations per Sample',xlim=c(0,max(mutations)), main='B) Density of Mutations per Simulation')
  }else if(cn & seq){
    plot(density(unlist(psis),bw=.05), xlab='Psi', xlim=c(0,1), main='C) Density of Psi Values across Simulations')
    plot(density(frac.altered,bw=.01), xlab='Genome Altered Fraction', xlim=c(0,1), main='D) Genome AF Density among Simulations')
    plot(density(mutations,bw=5), xlab='Mutations per Sample',xlim=c(0,max(mutations)), main='E) Density of Mutations per Simulation')
  }
}

png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/figure1.png',height=500,width=900,res=100)
layout(matrix(c(1,1,1,2,2,2,3,3,4,4,5,5,6,6,6,7,7,7), 3, 6, byrow = TRUE))
plotSetTraits('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-1', cn=FALSE)
plotSetTraits('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-2')
plotSetTraits('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-3', seq=FALSE)
dev.off()


#Figure S1 and S2:
metadata <- get(load('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-3\\metadata-3.rda'))
plotSNP <- function(chr.sim, chr.dat, j, sam, homFrac=.68, mu.0=0, sigma.0=.03){
  dat <- get(load(paste('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\dat\\data-3', '\\dat-3-', j,'.rda', sep='')))
  sim <- get(load(paste('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-3', '\\sim-3-', j,'.rda', sep='')))
  segdata <- dat$dat$cn.data
  #sigma0.lrr <- metadata$data.params$sigma0.lrr
  sigma0.lrr <- .12
  sigma0.baf <- metadata$data.params$sigma0.baf
  segs <- segdata[segdata$chr==chr.sim,]
  x.mu <- unlist(sapply(1:nrow(segs),function(k){rep(segs$X[k], segs$markers[k])}))
  y.mu <- unlist(sapply(1:nrow(segs),function(k){rep(segs$Y[k], segs$markers[k])}))
  index <- 1:length(x.mu)
  hom <- sample(1:length(index), round(length(index)*homFrac), replace=FALSE)
  inverted <- sample(1:length(index), round(length(index)*.5), replace=FALSE)
  x.mu[hom] <- x.mu[hom] + y.mu[hom]
  y.mu[hom] <- 0
  x.inverted <- x.mu[inverted]
  x.mu[inverted] <- y.mu[inverted]
  y.mu[inverted] <- x.inverted
  lrr.mu <- log10((x.mu + y.mu)/2)
  baf.mu <- x.mu/(x.mu + y.mu)
  lrr <- rnorm(length(lrr.mu), lrr.mu, sigma0.lrr)
  baf <- rnorm(length(baf.mu), baf.mu, sigma0.baf)
  baf[which(baf<0)] <- -baf[which(baf<0)]
  baf[which(baf>1)] <- 1/baf[which(baf>1)]
  path <- paste('E:\\SNP-HapMap\\MergeByChrom\\Full Data Table_Chr_', chr.dat, '.txt', sep='')
  cndata <- read.table(path, header=T)
  columns <- unique(sapply(4:ncol(cndata),function(x){strsplit(colnames(cndata)[x],split='[.]')[[1]][1]}))
  samn <- which(columns==sam)
  lrr.dat <- cndata[,samn*3+2]
  baf.dat <- cndata[,samn*3+3]
  par(mfrow=c(2,2))
  #maintab <- paste('Chr ', chr, sep='')
  plot(1:length(lrr), lrr, xlab='SNP Index', ylab='LRR', ylim=c(-1.5,1), pch='.', col='gray',
       main='A) SNP Array Data (simulated data)')
  plot(1:length(lrr.dat), lrr.dat, xlab='SNP Index', ylab='LRR', ylim=c(-1.5,1), pch='.', col='gray',
       main='B) SNP Array Data (real data)')
  abline(h=c(log10((1:4)/2)), col=c('gray','black','gray','gray'))
  abline(h=c(log10((1:4)/2)), col=c('gray','black','gray','gray'))
  plot(1:length(baf), baf, main='', xlab='SNP Index', ylab='BAF', ylim=c(0,1), pch='.', col='gray')
  abline(h=.5, col='black')
  plot(1:length(baf.dat), baf.dat, main='', xlab='SNP Index', ylab='BAF', ylim=c(0,1), pch='.', col='gray')
  abline(h=.5, col='black')
  #plot(density(baf), xlim=c(-1.5,1.5), ylim=c(0,5), main=paste('SD=',round(sd(baf),digits=3), sep=''))
}

#Figure S2
#Note: I only plot the first 225 SNVs in the function below because (unlike with the SNP array), the number of observations
#varies between samples, so I picked a number such that whatever samples or simulations I plot, they will both have at least
#This many SNVs. 
plotSEQ <- function(seqdata, seqsim){
  som.dat <- seqdata[seqdata$somatic_status=='SOMATIC',]
  vaf.dat <- na.omit(som.dat$alt_read_count/(som.dat$alt_read_count + som.dat$ref_read_count))
  tc.dat <- som.dat$alt_read_count + som.dat$ref_read_count
  seq.sim <- seqsim$dat$seq.data
  som.sim <- seq.sim[seq.sim$status=='somatic',]
  tc.sim <- som.sim$totalCounts
  vaf.sim <- som.sim$VAF
  par(mfrow=c(2,2))
  plot(tc.sim[1:225], ylim=c(0,250), xlab='Index', ylab='Total Read Counts', main='A) Somatic Variants (Simulation)', pch=16)
  plot(tc.dat[1:225], ylim=c(0, 250), xlab='Index', ylab='Total Read Counts', main='B) Somatic Variants (Data)', pch=16)
  plot(vaf.sim[1:225], ylim=c(0, 1), xlab='Index', ylab='VAF', main='', pch=16)
  plot(vaf.dat[1:225], ylim=c(0, 1), xlab='Index', ylab='VAF', main='', pch=16)
}

png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/figureS1.png',width=1000,height=500)
plotSNP(chr.sim=2, j=68, homFrac=.68, mu.0=0, sigma.0=.03, chr.dat=2, sam='NA18855')
dev.off()
png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/figureS2.png',width=1000,height=500)
plotSEQ(seqdata=get(load('C:\\Users\\Mark\\OneDrive - The Ohio State University\\deepcnv\\GS1115.rda')),
        seqsim=get(load('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\dat\\data-1\\dat-1-10.rda')))
dev.off()
#Figure S3:Karyotype plot:
###Not sure whether to include###

#Figure 2: Just mutations
assessments1 <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/assess/assess-1/assessments-1.rda'))
kdiff.cf1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[1]]$kdiff}))
psi.dist.cf1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[1]]$psi.dist}))
cn.dist.cf1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[1]]$cn.dist}))
mut.dist.cf1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[1]]$mut.dist}))
kdiff.sc1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[2]]$kdiff}))
psi.dist.sc1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[2]]$psi.dist}))
cn.dist.sc1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[2]]$cn.dist}))
mut.dist.sc1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[2]]$mut.dist}))
kdiff.exp1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[3]]$kdiff}))
psi.dist.exp1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[3]]$psi.dist}))
cn.dist.exp1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[3]]$cn.dist}))
mut.dist.exp1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[3]]$mut.dist}))
kdiff.exp1 <- na.omit(sapply(1:length(assessments1),function(j){assessments1[[j]][[3]]$kdiff}))
psi.cdf.cf1 <- list('x'=seq(from=min(psi.dist.cf1),to=max(psi.dist.cf1),length=150),'y'=sapply(seq(from=min(psi.dist.cf1),
   to=max(psi.dist.cf1),length=150),function(i){length(which(psi.dist.cf1<=i))/length(psi.dist.cf1)}))
cn.cdf.cf1 <- list('x'=seq(from=min(cn.dist.cf1),to=max(cn.dist.cf1),length=150),'y'=sapply(seq(from=min(cn.dist.cf1),
   to=max(cn.dist.cf1),length=150),function(i){length(which(cn.dist.cf1<=i))/length(cn.dist.cf1)}))
mut.cdf.cf1 <- list('x'=seq(from=min(mut.dist.cf1),to=max(mut.dist.cf1),length=150),'y'=sapply(seq(from=min(mut.dist.cf1),
   to=max(mut.dist.cf1),length=150),function(i){length(which(mut.dist.cf1<=i))/length(mut.dist.cf1)}))
psi.cdf.sc1 <- list('x'=seq(from=min(psi.dist.sc1),to=max(psi.dist.sc1),length=150),'y'=sapply(seq(from=min(psi.dist.sc1),
   to=max(psi.dist.sc1),length=150),function(i){length(which(psi.dist.sc1<=i))/length(psi.dist.sc1)}))
cn.cdf.sc1 <- list('x'=seq(from=min(cn.dist.sc1),to=max(cn.dist.sc1),length=150),'y'=sapply(seq(from=min(cn.dist.sc1),
   to=max(cn.dist.sc1),length=150),function(i){length(which(cn.dist.sc1<=i))/length(cn.dist.sc1)}))
mut.cdf.sc1 <- list('x'=seq(from=min(mut.dist.sc1),to=max(mut.dist.sc1),length=150),'y'=sapply(seq(from=min(mut.dist.sc1),
   to=max(mut.dist.sc1),length=150),function(i){length(which(mut.dist.sc1<=i))/length(mut.dist.sc1)}))
psi.cdf.exp1 <- list('x'=seq(from=min(psi.dist.exp1),to=max(psi.dist.exp1),length=150),'y'=sapply(seq(from=min(psi.dist.exp1),
   to=max(psi.dist.exp1),length=150),function(i){length(which(psi.dist.exp1<=i))/length(psi.dist.exp1)}))
cn.cdf.exp1 <- list('x'=seq(from=min(cn.dist.exp1),to=max(cn.dist.exp1),length=150),'y'=sapply(seq(from=min(cn.dist.exp1),
   to=max(cn.dist.exp1),length=150),function(i){length(which(cn.dist.exp1<=i))/length(cn.dist.exp1)}))
mut.cdf.exp1 <- list('x'=seq(from=min(mut.dist.exp1),to=max(mut.dist.exp1),length=150),'y'=sapply(seq(from=min(mut.dist.exp1),
   to=max(mut.dist.exp1),length=150),function(i){length(which(mut.dist.exp1<=i))/length(mut.dist.exp1)}))
xrange.psi1=c(min(psi.cdf.cf1$x,psi.cdf.sc1$x,psi.cdf.exp1$x),max(psi.cdf.cf1$x,psi.cdf.sc1$x,psi.cdf.exp1$x))
interval.psi1 <- max(xrange.psi1)-min(xrange.psi1)
xrange.cn1=c(min(cn.cdf.cf1$x,cn.cdf.sc1$x,cn.cdf.exp1$x),max(cn.cdf.cf1$x,cn.cdf.sc1$x,cn.cdf.exp1$x))
interval.cn1 <- max(xrange.cn1)-min(xrange.cn1)
xrange.mut1=c(min(mut.cdf.cf1$x,mut.cdf.sc1$x,mut.cdf.exp1$x),max(mut.cdf.cf1$x,mut.cdf.sc1$x,mut.cdf.exp1$x))
interval.mut1 <- max(xrange.mut1)-min(xrange.mut1)

png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/figure2_new.png',width=1400,height=2000,res=200)
layout(matrix(c(1,2,3,4,4,4,5,5,5,6,6,6),nrow=6,ncol=2))
hist(kdiff.cf1, breaks=seq(from=-5.5,to=5.5, by=1),xlab='Difference', main='A) Histogram of K difference (Clonefinder)')
hist(kdiff.sc1, breaks=seq(from=-5.5,to=5.5, by=1),xlab='Difference', main='Histogram of K difference (SciClone)')
hist(kdiff.exp1, breaks=seq(from=-5.5,to=5.5, by=1),xlab='Difference', main='Histogram of K difference (Expands)')
plot(psi.cdf.cf1$x,psi.cdf.cf1$y, ylim=c(0,1), main='B)  Psi Distance Cumulative Distribution',type='l',
     xlim=c(min(psi.cdf.cf1$x,psi.cdf.sc1$x,psi.cdf.exp1$x),max(psi.cdf.cf1$x,psi.cdf.sc1$x,psi.cdf.exp1$x)),
     xlab='Psi Distance',ylab='Cumulative Density')
lines(psi.cdf.sc1$x,psi.cdf.sc1$y,col='blue')
lines(psi.cdf.exp1$x,psi.cdf.exp1$y,col='red')
legend('bottomright', c('Clonefinder','Sciclone','Expands'),col=c('black','blue','red'),lty=c(1,1),cex=.7)
plot(mut.cdf.cf1$x,mut.cdf.cf1$y, ylim=c(0,1), main='C) Mutation Distance Cumulative Distribution',type='l',
     xlim=c(min(mut.cdf.cf1$x,mut.cdf.sc1$x,mut.cdf.exp1$x),max(mut.cdf.cf1$x,mut.cdf.sc1$x,mut.cdf.exp1$x)),
     xlab='Mutation Distance',ylab='Cumulative Density')
lines(mut.cdf.sc1$x,mut.cdf.sc1$y,col='blue')
lines(mut.cdf.exp1$x,mut.cdf.exp1$y,col='red')
legend('bottomright', c('Clonefinder','Sciclone','Expands'),col=c('black','blue','red'),lty=c(1,1),cex=1)
dev.off()


#Figure 3 - New (CNV-sim and mixtures)
assessments2 <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/assess/assess-2/assessments-2.rda'))
kdists.cf2 <- na.omit(sapply(1:length(assessments2),function(j){assessments2[[j]][[1]]$k.dist}))
psi.dist.cf2 <- na.omit(sapply(1:length(assessments2),function(j){assessments2[[j]][[1]]$psi.dist}))
cn.dist.cf2 <- na.omit(sapply(1:length(assessments2),function(j){assessments2[[j]][[1]]$cn.dist}))
kdiff2 <- na.omit(sapply(1:length(assessments2),function(j){assessments2[[j]][[1]]$kdiff}))
cndiff.avg2 <- na.omit(sapply(1:length(assessments2),function(j){assessments2[[j]][[1]]$avg.cndiff}))
psi.cdf.cf2 <- list('x'=seq(from=min(psi.dist.cf2),to=max(psi.dist.cf2),length=150),'y'=sapply(seq(from=min(psi.dist.cf2),
    to=max(psi.dist.cf2),length=150),function(i){length(which(psi.dist.cf2<=i))/length(psi.dist.cf2)}))
cn.cdf.cf2 <- list('x'=seq(from=min(cn.dist.cf2),to=max(cn.dist.cf2),length=150),'y'=sapply(seq(from=min(cn.dist.cf2),
    to=max(cn.dist.cf2),length=150),function(i){length(which(cn.dist.cf2<=i))/length(cn.dist.cf2)}))

assessmentsMix <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/assess/assess-mix/assessments-mix.rda'))
kdists.cfMix <- na.omit(sapply(1:length(assessmentsMix),function(j){assessmentsMix[[j]][[1]]$k.dist}))
psi.dist.cfMix <- na.omit(sapply(1:length(assessmentsMix),function(j){assessmentsMix[[j]][[1]]$psi.dist}))
cn.dist.cfMix <- na.omit(sapply(1:length(assessmentsMix),function(j){assessmentsMix[[j]][[1]]$cn.dist}))
kdiffMix <- na.omit(sapply(1:length(assessmentsMix),function(j){assessmentsMix[[j]][[1]]$kdiff}))
cndiff.avgMix <- na.omit(sapply(1:length(assessmentsMix),function(j){assessmentsMix[[j]][[1]]$avg.cndiff}))
psi.cdf.cfMix <- list('x'=seq(from=min(psi.dist.cfMix),to=max(psi.dist.cfMix),length=150),'y'=sapply(seq(from=min(psi.dist.cfMix),
       to=max(psi.dist.cfMix),length=150),function(i){length(which(psi.dist.cfMix<=i))/length(psi.dist.cfMix)}))
cn.cdf.cfMix <- list('x'=seq(from=min(cn.dist.cfMix),to=max(cn.dist.cfMix),length=150),'y'=sapply(seq(from=min(cn.dist.cfMix),
       to=max(cn.dist.cfMix),length=150),function(i){length(which(cn.dist.cfMix<=i))/length(cn.dist.cfMix)}))

png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/figure3_new.png',width=1500,height=2000,res=150)
par(mfrow=c(3,2))
hist(kdiff2, breaks=seq(from=-5.5,to=5.5, by=1),xlab='Difference', main='A) Histogram of K difference')
hist(kdiffMix, breaks=seq(from=-5.5,to=5.5, by=1),xlab='Difference', main='D) Histogram of K difference')
plot(psi.cdf.cf2$x,psi.cdf.cf2$y, ylim=c(0,1), main='B) Psi Distance Cumulative Distribution',type='l',
     xlim=c(min(psi.cdf.cf2$x),max(psi.cdf.cf2$x)),
     xlab='Psi Distance',ylab='Cumulative Density')
plot(psi.cdf.cfMix$x,psi.cdf.cfMix$y, ylim=c(0,1), main='E) Psi Distance Cumulative Distribution',type='l',
     xlim=c(min(psi.cdf.cfMix$x),max(psi.cdf.cfMix$x)),
     xlab='Psi Distance',ylab='Cumulative Density')
plot(cn.cdf.cf2$x,cn.cdf.cf2$y, ylim=c(0,1), main='C) Copy Number Distance Cumulative Distribution',type='l',
     xlim=c(min(cn.cdf.cf2$x),max(cn.cdf.cf2$x)),
     xlab='Copy Number Distance',ylab='Cumulative Density')
plot(cn.cdf.cfMix$x,cn.cdf.cfMix$y, ylim=c(0,1), main='F) Copy Number Distance Cumulative Distribution',type='l',
     xlim=c(min(cn.cdf.cfMix$x),max(cn.cdf.cfMix$x)),
     xlab='Copy Number Distance',ylab='Cumulative Density')
dev.off()


#Figure 4 New (sim-CNV+Mut, 2X2)
assessments3 <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/assess/assess-3/assessments-3.rda'))
cn.concordance.cf <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[1]]$concord.cn}))
mut.concordance.cf <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[1]]$concord.mut}))
psis.concordance.cf <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[1]]$concord.psi}))
kdists.cf <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[1]]$k.dist}))
psi.dist.cf <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[1]]$psi.dist}))
cn.dist.cf <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[1]]$cn.dist}))
mut.dist.cf <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[1]]$mut.dist}))
kdiff.cf <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[1]]$kdiff}))
psi.cdf.cf <- list('x'=seq(from=min(psi.dist.cf),to=max(psi.dist.cf),length=150),'y'=sapply(seq(from=min(psi.dist.cf),
                                                                                                to=max(psi.dist.cf),length=150),function(i){length(which(psi.dist.cf<=i))/length(psi.dist.cf)}))
cn.cdf.cf <- list('x'=seq(from=min(cn.dist.cf),to=max(cn.dist.cf),length=150),'y'=sapply(seq(from=min(cn.dist.cf),
                                                                                             to=max(cn.dist.cf),length=150),function(i){length(which(cn.dist.cf<=i))/length(cn.dist.cf)}))
mut.cdf.cf <- list('x'=seq(from=min(mut.dist.cf),to=max(mut.dist.cf),length=150),'y'=sapply(seq(from=min(mut.dist.cf),
                                                                                                to=max(mut.dist.cf),length=150),function(i){length(which(mut.dist.cf<=i))/length(mut.dist.cf)}))

psi.dist.sc <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[2]]$psi.dist}))
cn.dist.sc <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[2]]$cn.dist}))
mut.dist.sc <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[2]]$mut.dist}))
kdiff.sc <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[2]]$kdiff}))
psi.cdf.sc <- list('x'=seq(from=min(psi.dist.sc),to=max(psi.dist.sc),length=150),'y'=sapply(seq(from=min(psi.dist.sc),
                                                                                                to=max(psi.dist.sc),length=150),function(i){length(which(psi.dist.sc<=i))/length(psi.dist.sc)}))
cn.cdf.sc <- list('x'=seq(from=min(cn.dist.sc),to=max(cn.dist.sc),length=150),'y'=sapply(seq(from=min(cn.dist.sc),
                                                                                             to=max(cn.dist.sc),length=150),function(i){length(which(cn.dist.sc<=i))/length(cn.dist.sc)}))
mut.cdf.sc <- list('x'=seq(from=min(mut.dist.sc),to=max(mut.dist.sc),length=150),'y'=sapply(seq(from=min(mut.dist.sc),
                                                                                                to=max(mut.dist.sc),length=150),function(i){length(which(mut.dist.sc<=i))/length(mut.dist.sc)}))

psi.dist.exp <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[3]]$psi.dist}))
cn.dist.exp <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[3]]$cn.dist}))
mut.dist.exp <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[3]]$mut.dist}))
kdiff.exp <- na.omit(sapply(1:length(assessments3),function(j){assessments3[[j]][[3]]$kdiff}))
psi.cdf.exp <- list('x'=seq(from=min(psi.dist.exp),to=max(psi.dist.exp),length=150),'y'=sapply(seq(from=min(psi.dist.exp),
                                                                                                   to=max(psi.dist.exp),length=150),function(i){length(which(psi.dist.exp<=i))/length(psi.dist.exp)}))
cn.cdf.exp <- list('x'=seq(from=min(cn.dist.exp),to=max(cn.dist.exp),length=150),'y'=sapply(seq(from=min(cn.dist.exp),
                                                                                                to=max(cn.dist.exp),length=150),function(i){length(which(cn.dist.exp<=i))/length(cn.dist.exp)}))
mut.cdf.exp <- list('x'=seq(from=min(mut.dist.exp),to=max(mut.dist.exp),length=150),'y'=sapply(seq(from=min(mut.dist.exp),
                                                                                                   to=max(mut.dist.exp),length=150),function(i){length(which(mut.dist.exp<=i))/length(mut.dist.exp)}))
xrange.psi=c(min(psi.cdf.cf1$x,psi.cdf.sc1$x,psi.cdf.exp1$x),max(psi.cdf.cf1$x,psi.cdf.sc1$x,psi.cdf.exp1$x))
interval.psi <- max(xrange.psi)-min(xrange.psi)
xrange.cn=c(min(cn.cdf.cf$x,cn.cdf.sc$x,cn.cdf.exp$x),max(cn.cdf.cf$x,cn.cdf.sc$x,cn.cdf.exp$x))
interval.cn <- max(xrange.cn)-min(xrange.cn)
xrange.mut=c(min(mut.cdf.cf$x,mut.cdf.sc$x,mut.cdf.exp$x),max(mut.cdf.cf$x,mut.cdf.sc$x,mut.cdf.exp$x))
interval.mut <- max(xrange.mut)-min(xrange.mut)

png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/figure4_new.png',height=1900,width=2000,res=220)
layout(matrix(c(1,2,3,4,4,4,5,5,5,6,6,6),nrow=6,ncol=2))
hist(kdiff.cf, breaks=seq(from=-5.5,to=5.5, by=1),xlab='Difference', main='A) Histogram of K difference')
hist(kdiff.sc, breaks=seq(from=-5.5,to=5.5, by=1),xlab='Difference', main='Histogram of K difference')
hist(kdiff.exp, breaks=seq(from=-5.5,to=5.5, by=1),xlab='Difference', main='Histogram of K difference')
plot(psi.cdf.cf$x,psi.cdf.cf$y, ylim=c(0,1), main='B) Psi Distance Cumulative Distribution',type='l',
     xlim=c(min(psi.cdf.cf$x,psi.cdf.sc$x,psi.cdf.exp$x),max(psi.cdf.cf$x,psi.cdf.sc$x,psi.cdf.exp$x)),
     xlab='Psi Distance',ylab='Cumulative Density')
lines(psi.cdf.sc$x,psi.cdf.sc$y,col='blue')
lines(psi.cdf.exp$x,psi.cdf.exp$y,col='red')
legend('bottomright', c('Clonefinder','Sciclone','Expands'),col=c('black','blue','red'),lty=c(1,1),cex=1)
plot(cn.cdf.cf$x,cn.cdf.cf$y, ylim=c(0,1), main='C) Copy Number Distance Cumulative Distribution',type='l',
     xlim=c(min(cn.cdf.cf$x,cn.cdf.sc$x,cn.cdf.exp$x),max(cn.cdf.cf$x,cn.cdf.sc$x,cn.cdf.exp$x)),
     xlab='Copy Number Distance',ylab='Cumulative Density')
lines(cn.cdf.sc$x,cn.cdf.sc$y,col='blue')
lines(cn.cdf.exp$x,cn.cdf.exp$y,col='red')
legend('bottomright', c('Clonefinder','Sciclone','Expands'),col=c('black','blue','red'),lty=c(1,1),cex=1)
plot(mut.cdf.cf$x,mut.cdf.cf$y, ylim=c(0,1), main='D) Mutation Distance Cumulative Distribution',type='l',
     xlim=c(min(mut.cdf.cf$x,mut.cdf.sc$x,mut.cdf.exp$x),max(mut.cdf.cf$x,mut.cdf.sc$x,mut.cdf.exp$x)),
     xlab='Mutation Distance',ylab='Cumulative Density')
lines(mut.cdf.sc$x,mut.cdf.sc$y,col='blue')
lines(mut.cdf.exp$x,mut.cdf.exp$y,col='red')
legend('bottomright', c('Clonefinder','Sciclone','Expands'),col=c('black','blue','red'),lty=c(1,1),cex=1)
dev.off()


###Figure S4 (?):
res.path <- 'C:/Users/Mark/OneDrive - The Ohio State University/clonetools/res/res-CLL'
rewind <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/SNP_array_data/rewind_both.rda'))
samples <- unique(rewind$SamID)
resfiles <- list.files(res.path)
reslist <- lapply(1:length(resfiles),function(i){get(load(paste(res.path, '/', resfiles[i], sep='')))})
psis <- t(sapply(1:length(resfiles),function(i){reslist[[i]]$psi}))
ks <- sapply(1:nrow(psis),function(i){length(which(psis[i,]>0))})
alt.frac <- sapply(1:length(reslist),function(j){
  temp <- rewind[rewind$SamID==samples[j],]
  notna <- which(!is.na(temp$loc.end) & !is.na(temp$loc.start))
  all.lens <- sum(as.numeric(temp$loc.end[notna]) - as.numeric(temp$loc.start[notna]))
  alt <- which(sapply(1:nrow(reslist[[j]]$A),function(k){
    unique(all.equal(unname(reslist[[j]]$A[k,1:length(which(reslist[[j]]$psi>0))]), rep(1, length(which(reslist[[j]]$psi>0))))!=TRUE) & 
      unique(all.equal(unname(reslist[[j]]$B[k,1:length(which(reslist[[j]]$psi>0))]), rep(1, length(which(reslist[[j]]$psi>0))))!=TRUE)
  }))
  lens <- unlist(sapply(1:length(alt), function(k){
    index <- which(rewind$seg.median==reslist[[j]]$filtered.data$cndata.filt$LRR[alt[k]] &
                     rewind$AvgBAF==reslist[[j]]$filtered.data$cndata.filt$BAF[alt[k]])
    rewind$loc.end[index] - rewind$loc.start[index]
  }))
  #alt.marks <- sum(cf.results[[j]]$filtered.data$cndata.filt$markers[alt])
  #alt.marks/total.markers
  sum(as.numeric(lens))/all.lens
})
psivec <- as.vector(psis)

png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/figureS4.png',width=1000,height=700,res=100)
par(mfrow=c(2,1))
hist(ks,breaks=seq(from=.5,to=5.5,by=1), main='Frequency distribution of K',xlab='Number of subclones')
#plot(density(psivec[which(psivec>0)],from=0,to=1,bw=.01),main='Distribution of Psis',xlab='Psi',xlim=c(0,1))
plot(ks, alt.frac, main='', xlab='Number of subclones', ylab='Altered Genomic Fraction')
dev.off()


###Figure 5 new:
data.path <- 'C:/Users/Mark/OneDrive - The Ohio State University/clonetools/dat/data-CLL'
cll.files <- list.files(data.path)
cids.data <- sapply(1:length(cll.files),function(j){strsplit(strsplit(cll.files[j],split='dat-')[[1]][2],split='.rda')[[1]][1]})
clinical <- get(load('C:/Users/Mark/Documents/Lab/clinOk.rda'))
cids.clinical <- rownames(clinical)
common.cids <- intersect(cids.clinical,cids.data)
eligible <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clinical/eligible.rda'))
clinical$ttp.eligible <- eligible
clinical <- clinical[sapply(1:length(common.cids),function(j){which(rownames(clinical)==common.cids[j])}),]
ttp.eligible <- which(clinical$ttp.eligible)
res.files <- list.files(res.path)
cf.results <- lapply(1:length(res.files), function(j){get(load(paste(res.path, '/', res.files[j], sep='')))})
#cf.results <- lapply(1:length(common.cids),function(j){cf.results[[which(samples==common.cids[j])]]})
ks <- sapply(1:length(cf.results),function(j){length(which(cf.results[[j]]$psi>0))})
ok <- sapply(1:length(common.cids),function(i){which(cids.data==common.cids[i])})
ks <- ks[ok]
k.status <- rep(0,length(ks))
k.status[ks>1] <- 1
columns <- colnames(clinical)

TimeDiag2SigTreat <- clinical$TimeDiagnosis2SigTreat
TTP <- clinical[,which(columns=='TTP.time')]
TTP.numericStatus <- clinical[,which(columns=='TTP.numericStatus')]
osat.time <- clinical$TxDSS.time[ttp.eligible]
osas.time <- clinical$OSAfterSample
numericVitalStatus <- clinical$NumericVitalStatus
numericVitalStatus.elig <- numericVitalStatus[ttp.eligible]
NumericSigTreatment <- clinical$NumericSigTreatment

colors <- c('red', 'blue', 'purple', 'darkorange')
Multiclonal <- k.status
Multiclonal.elig <- k.status[ttp.eligible]
Complex <- rep(NA, nrow(clinical))
Complex[which(clinical$CatCyto=='Simple')] <- 0
Complex[which(clinical$CatCyto=='Complex')] <- 1
Complex.elig <- Complex[ttp.eligible]
df.main <- data.frame('osas.time'=osas.time,'numericVitalStatus'=numericVitalStatus,'Multiclonal'=Multiclonal,
    'Complex'=Complex,'TimeDiag2SigTreat'=TimeDiag2SigTreat,'NumericSigTreatment'=NumericSigTreatment)
df.elig <- data.frame('ttp.time'=TTP[ttp.eligible],'Multiclonal'=Multiclonal.elig,'Complex'=Complex.elig,
    'numericVitalStatus'=numericVitalStatus.elig,'osat.time'=osat.time,'TTP.status'=TTP.numericStatus[ttp.eligible])

osas <- prodlim(Surv(osas.time, numericVitalStatus) ~ Multiclonal,data=df.main)
diff <- survdiff(Surv(osas.time, numericVitalStatus) ~ Multiclonal,data=df.main)
p_osas <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
osas.karyo <- prodlim(Surv(osas.time, numericVitalStatus) ~ Multiclonal + Complex,data=df.main)
diff <- survdiff(Surv(osas.time, numericVitalStatus) ~ Multiclonal + Complex,data=df.main)
p_osas.karyo <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)

t2st <- prodlim(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ Multiclonal,data=df.main)
diff <- survdiff(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ Multiclonal,data=df.main)
p_t2st <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
t2st.karyo <- prodlim(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ Multiclonal + Complex,data=df.main)
diff <- survdiff(Surv(TimeDiag2SigTreat, NumericSigTreatment) ~ Multiclonal + Complex,data=df.main)
p_t2st.karyo <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)

osat <- prodlim(Surv(osat.time, numericVitalStatus) ~ Multiclonal,df.elig)
diff <- survdiff(Surv(osat.time, numericVitalStatus) ~ Multiclonal,df.elig)
p_osat <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
osat.karyo <- prodlim(Surv(osat.time, numericVitalStatus) ~ Multiclonal + Complex,df.elig)
diff <- survdiff(Surv(osat.time, numericVitalStatus) ~ Multiclonal + Complex,df.elig)
p_osat.karyo <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)

ttp <- prodlim(Surv(ttp.time, TTP.status) ~ Multiclonal,df.elig)
diff <- survdiff(Surv(ttp.time, TTP.status) ~ Multiclonal,df.elig)
p_ttp <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)
ttp.karyo <- prodlim(Surv(ttp.time, TTP.status) ~ Multiclonal + Complex,df.elig)
diff <- survdiff(Surv(ttp.time, TTP.status) ~ Multiclonal + Complex,df.elig)
p_ttp.karyo <- round(pchisq(diff$chisq, length(diff$n)-1, lower.tail = FALSE), digits=4)

png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/figure5_new.png',width=1600,height=2600,res=200)
par(mfrow=c(2,1))
plot(osas)
title(paste('A) Overall Survival after Sample; P = ',p_osas,sep=''))
plot(osat)
title(paste('B) Overall Survival after Treatment; P = ',p_osat,sep=''))
dev.off()

png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/figureS5.png',width=1200,height=1600,res=140)
par(mfrow=c(2,1))
plot(t2st)
title(paste('C) Time to Treatment; P = ',p_t2st,sep=''))
plot(ttp)
title(paste('D) Time to Progression; P = ',p_ttp,sep=''))
dev.off()

#Figure S6
png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/figureS6.png',width=1800,height=1800,res=140)
par(mfrow=c(2,2))
plot(osas.karyo)
title(paste('A) Overall Survival after Sample; P = ',p_osas.karyo,sep=''))
plot(osat.karyo)
title(paste('B) Overall Survival after Treatment; P = ',p_osat.karyo,sep=''))
plot(t2st.karyo)
title(paste('C) Time to Treatment; P = ',p_t2st.karyo,sep=''))
plot(ttp.karyo)
title(paste('D) Time to Progression; P = ',p_ttp.karyo,sep=''))
dev.off()

if (packageVersion("CloneSeeker") < "1.0.0") {
  stop("You need to update 'CloneSeeker'.")
}
library("CloneSeeker")

###Generate a sample simulation population:
#set: 1 (pcnv=0, nu=30, sigma.lrr=.15, sigma.baf=.03, sigma.reads=25); k = 1:5
#set: 2 (pcnv=1, nu=30, sigma.lrr=.15, sigma.baf=.03, sigma.reads=25); k = 1:5
#set: 3 (pcnv=1, nu=0, sigma.lrr=.15, sigma.baf=.03, sigma.reads=25); k = 1:5
gen.sims <- FALSE

integPath <- 'C:/Users/Mark/OneDrive - The Ohio State University/Integrated'
#path1 <- file.path(integPath, 'parOpt\\sim')
#path2 <- file.path(integPath, 'parOpt\\data')
path1 <- 'C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim2'
path2 <- 'C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\dat2'
if(gen.sims) {
  pars.default <- list('rounds'=400, 'nu'=0, 'pcnv'=1, 'norm.contam'=FALSE)
  dataPars <- list('snps.seq'=1000000, 'snps.cgh'=600000, 'mu'=70,
                   'sigma.reads'=25, 'sigma0.lrr'=.15, 
                   'sigma0.baf'=.03, 'density.sigma'=.1)
  threshold <- .04
  dirs <- list.dirs(path1)
  dirs <- dirs[-1]
  if(length(dirs) > 0) {
    setIDs <- as.numeric(sapply(1:length(dirs), function(j) {
      strsplit(dirs[j], split='sims-')[[1]][2]
    }))
    setID <- max(setIDs) + 1
  } else {
    setID <- 1
  }
  dir.create(paste(path1, '\\sims-', setID, sep=''))
  dir.create(paste(path2, '\\data-', setID, sep=''))
  simpath <- paste(path1, '\\sims-', setID, sep='')
  datapath <- paste(path2, '\\data-', setID, sep='')
  #psis <- genSimplex(20, 5, reps=1)
  psis <- get(load(file.path(integPath, 'psis.100.rda')))
  psis <- psis[which(sapply(1:nrow(psis), function(j) {
    length(which(psis[j, ]==0 | psis[j, ] > threshold))==ncol(psis)
  })), ]
  ks <- sapply(1:nrow(psis), function(j) {length(which(psis[j, ] > 0))})
  seed <- 123
  set.seed(seed)
  sampled <- unlist(lapply(1:5, function(x) {sample(which(ks==x), 60, replace=TRUE)}))
  psis <- psis[sampled, ]
  n <- nrow(psis)
  for(j in 1:nrow(psis)) {
    go <- FALSE
    while(go==FALSE) {
      tumor <- try(tumorGen(psis[j, ], pars.default$rounds,
                            pars.default$nu, pars.default$pcnv,
                            pars.default$norm.contam), silent=TRUE)
      if(class(tumor) != 'try-error') {
        data <- dataGen(tumor, dataPars$snps.seq, dataPars$snps.cgh,
                        dataPars$mu, dataPars$sigma.reads, 
                        dataPars$sigma0.lrr, dataPars$sigma0.baf,
                        dataPars$density.sigma)
      }
      if(class(tumor) != 'try-error' & class(data) != 'try-error') {
        go <- TRUE
      }
    }
    filename.tumor <- paste(simpath, '\\', 'sim-', setID, '-', j, '.rda', sep='')
    filename.data <- paste(datapath, '\\', 'dat-', setID, '-', j, '.rda', sep='')
    sim <- list('tumor'=tumor)
    save(sim, file=filename.tumor)
    dat <- list('dat'=data)
    save(dat, file=filename.data)
  }
  if(pars.default$nu > 0 & pars.default$pcnv > 0) {
    alt.types <- 'both'
  }else if(pars.default$nu > 0 & pars.default$pcnv == 0) {
    alt.types <- 'mut'
  }else if(pars.default$nu == 0 & pars.default$pcnv > 0) {
    alt.types <- 'cnv'
  } else {
    alt.types <- 'none'
  }
  metadata <- list('tumor.params'=pars.default,
                   'data.params'=dataPars,
                   'alt.types'=alt.types,
                   'date'=Sys.Date(), 
                  'setID'=setID, 'seed'=seed)
  save(metadata, file=paste(simpath, '\\metadata-', setID, '.rda', sep=''))
}


###Mixing real data:
mix <- function(datList, psi, datapath, simpath, index, version, alter=FALSE) {
  breaks <- lapply(1:22, function(j) {
    brks <- sort(unique(unlist(sapply(1:length(datList), function(k) {c(datList[[k]][datList[[k]]$chrom==j, ]$loc.start, 
      datList[[k]][datList[[k]]$chrom==j, ]$loc.end)}))))
    starts <- brks[1:(length(brks)-1)]
    ends <- c(starts[2:length(starts)]-1, brks[length(brks)])
    data.frame('start'=starts, 'end'=ends, 'chr'=rep(j, length(starts)))
  })
  coords <- Reduce(rbind, breaks)
  columns <- sample(1:2, length(psi), replace=TRUE)
  shifts <- sapply(1:length(datList), function(j) {
    lrrs <- unlist(sapply(1:nrow(datList[[j]]), function(k) {rep(datList[[j]]$seg.median[k], datList[[j]]$num.mark[k])}))
    dens <- density(na.omit(2*10^lrrs))
    dens$x[which.max(dens$y)] - 2
  })
  d2 <- lapply(1:length(datList), function(j) {
    temp <- datList[[j]]
    res <- t(sapply(1:nrow(coords), function(k) {
      chrdat <- temp[as.character(temp$chrom)==as.character(coords$chr[k]), ]
      seg <- chrdat[chrdat$loc.end>=coords$start[k] & chrdat$loc.start<=coords$end[k], ]
      #Afreq <- seg$AA*2 + seg$AB
      #Bfreq <- seg$BB*2 + seg$AB
      #baf <- Bfreq/(Afreq + Bfreq)
      baf <- seg$AvgBAF
      tcn <- 2*10^(seg$seg.median)
      tcn <- tcn - shifts[j]
      if(length(which(tcn<0)) > 0) {
        tcn[which(tcn<0)] <- -tcn[which(tcn<0)]
      }
      X <- tcn*(1-baf)
      Y <- tcn*baf
      if(nrow(seg) > 0) {
        output <- c(X, Y)
      } else {
        output <- rep(NA, 2)
      }
      output
    }))
    colnames(res) <- c('X', 'Y')
    res
  })
  data.mixed <- as.data.frame(Reduce('+', lapply(1:length(d2), function(k) {psi[k]*d2[[k]]})))
  notna <- which(sapply(1:nrow(data.mixed), function(x) {length(which(is.na(data.mixed[x, ])))==0}))
  for(j in 1:length(d2)) {
    d2[[j]] <- d2[[j]][notna, ]
  }
  data.mixed <- data.mixed[notna, ]
  starts <- coords$start[notna]
  ends <- coords$end[notna]
  chrs <- coords$chr[notna]
  data.mixed$chr <- chrs
  data.mixed$seg <- 1:length(notna)
  data.mixed$LRR <- log10((data.mixed$X + data.mixed$Y)/2)
  data.mixed$BAF <- data.mixed$X/(data.mixed$X + data.mixed$Y)
  data.mixed$markers <- unlist(sapply(1:22, function(i) {
    pos.chr <- pos[pos$Chr==i, ]
    data.chr <- data.mixed[data.mixed$chr==i, ]
    sapply(1:nrow(data.chr), function(j) {
      length(which(pos.chr  > = starts[which(data.mixed$chr==i)[j]] & pos.chr <= ends[which(data.mixed$chr==i)[j]]))
    })
  }))
  data.mixed$start <- starts
  data.mixed$end <- ends
  data.mixed <- data.mixed[, c(3, 4, 5, 6, 1, 2, 7, 8, 9)]
  clones <- lapply(1:length(d2), function(k) {
    #tcn <- d2[[k]][, 1] + d2[[k]][, 2]
    #BAF <- d2[[k]][, 1]/tcn
    #BAF[which(tcn==0)] <- 0
    #df <- as.data.frame(round(t(sapply(1:length(tcn), function(l) {tcn[l]*c(BAF[l], 1-BAF[l])}))))
    df <- data.frame('A'=rep(1, nrow(d2[[1]])), 'B'=rep(1, nrow(d2[[1]])))
    colnames(df) <- c('A', 'B')
    df$chr <- data.mixed$chr
    df$start <- starts
    df$end <- ends
    df$seg <- 1:length(starts)
    df$parent.index <- rep(0, length(starts))
    df$markers <- data.mixed$markers
    df <- df[, c(3, 4, 5, 1, 2, 6, 7, 8)]
    list('cn'=df, 'seq'=NULL)
  })
  for(k in 2:nrow(clones[[1]]$cn)) {
    bin1 <- data.mixed$X[k]==data.mixed$X[k-1] & data.mixed$Y[k]==data.mixed$Y[k-1]
    bin2 <- clones[[1]]$cn$chr[k]==clones[[1]]$cn$chr[k-1]
    if(is.na(bin1)) {
      bin1 <- FALSE
    }
    if(bin1==TRUE & bin2==TRUE) {
      for(l in 1:length(clones)) {
        clones[[l]]$cn$start[k] <- clones[[l]]$cn$start[k-1]
        clones[[l]]$cn$markers[k] <- clones[[l]]$cn$markers[k] + clones[[l]]$cn$markers[k-1]
        clones[[l]]$cn[k-1, ] <- rep(NA, ncol(clones[[1]]$cn))
      }
      data.mixed$start[k] <- data.mixed$start[k-1]
      data.mixed$markers[k] <- data.mixed$markers[k-1]
      data.mixed[k-1, ] <- rep(NA, ncol(data.mixed))
    }
  }
  notna <- unique(unlist(lapply(1:length(clones), function(j) {which(!is.na(clones[[j]]$cn$seg))})))
  for(k in 1:length(clones)) {
    clones[[k]]$cn <- clones[[k]]$cn[notna, ]
    clones[[k]]$cn$seg <- rownames(clones[[k]]$cn) <- 1:nrow(clones[[k]]$cn)
    clones[[k]]$cn
  }
  data.mixed <- data.mixed[notna, ]
  dat <- list('cn.data'=data.mixed, 'seq.data'=NULL)
  dat$cn.data <- na.omit(dat$cn.data)
  dat$cn.data$seg <- rownames(dat$cn.data) <- 1:nrow(dat$cn.data)
  if(alter) {
    alt.pools <- lapply(1:length(psi), function(j) {
      x <- dat$cn.data$X
      y <- dat$cn.data$Y
      markers <- dat$cn.data$markers
      which(markers > 1000 & x-shifts[j]/2 > .95 & x-shifts[j]/2<1.05 & y-shifts[j]/2 > .95 & y-shifts[j]/2<1.05)
    })
    altN <- sample(1:3, length(psi), prob=c(.6, .3, .1), replace=TRUE)
    altered <- lapply(1:length(psi), function(j) {sample(alt.pools[[j]], altN[j])})
    change <- lapply(1:length(psi), function(j) {sample(c(-1, 1), altN[j], replace=TRUE)})
    for(l in 1:length(psi)) {
      if(length(altered[[l]]) > 0) {
        for(m in 1:length(altered[[l]])) {
          allele <- sample(c(1, 2), 1)
          clones[[l]]$cn[altered[[l]][[m]], allele+3] <- 
            clones[[l]]$cn[altered[[l]][[m]], allele+3] + change[[l]][[m]]
          dat$cn.data[altered[[l]][[m]], allele+4] <- dat$cn.data[altered[[l]][[m]], allele+4] + 
            change[[l]][[m]]*psi[l]
          if(dat$cn.data[altered[[l]][[m]], allele+4] < 0) {
            dat$cn.data[altered[[l]][[m]], allele+4] <- -dat$cn.data[altered[[l]][[m]], allele+4]
          }
        }
      }
    }
  }
  tumor <- list('clones'=clones, 'psi'=psi, 'altered'=altered, 'change'=change)
  save(dat, file=paste(datapath, '/mixdat', version, '-', index, '.rda', sep=''))
  save(tumor, file=paste(simpath, '/mixsim', version, '-', index, '.rda', sep=''))
}

gen.mixtures <- FALSE
if(gen.mixtures) {
  version <- 1
  threshold <- .04
  if(version==1) {
    version <- ''
  }
  seed <- 12345
  set.seed(seed)
  path <- 'C:/Users/Mark/OneDrive - The Ohio State University/clonetools'
  rewind.hapmap <- get(load(paste(path, '\\SNP_array_data\\rewind.hapmap.rda', sep='')))
  rewind.cll <- get(load(paste(path, '\\', 'SNP_array_data\\rewind_both.rda', sep='')))
  rewind.cll <- rewind.cll[, c(1:6, 11)]
  rewind <- rbind(rewind.cll, rewind.hapmap)
  datapath <- paste('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/dat/data-mix2', version, sep='')
  simpath <- paste('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/sim/sims-mix2', version, sep='')
  psipool <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cloneseeker/psis.100.rda'))
  psipool <- psipool[which(sapply(1:nrow(psipool), function(j) {length(which(psipool[j, ]==0 | psipool[j, ] > threshold))==ncol(psipool)})), ]
  ks <- sapply(1:nrow(psipool), function(i) {length(which(psipool[i, ] > 0))})
  N <- 300 #Let's to 60 apiece for k=1, 2, 3, 4, and 5
  indices <- c(sample(which(ks==1), 60, replace=TRUE), sample(which(ks==2), 60, replace=TRUE), sample(which(ks==3), 60, replace=FALSE), 
               sample(which(ks==4), 60, replace=FALSE), sample(which(ks==5), 60, replace=FALSE))
  psis <- lapply(1:N, function(j) {psipool[indices[j], ][which(psipool[indices[j], ] > 0)]})
  #IDpool <- c('CL001', 'CL060', 'CL082', 'CL084', 'CL102', 'CL185', 'NA19193')
  IDpool <- c('NA07019', 'NA12234', 'NA12249', 'NA12753', 'NA12761', 'NA18545', 'NA18975', 'NA18999', 'NA18517')
  IDs <- lapply(1:N, function(j) {sample(IDpool, length(which(psis[[j]] > 0)), replace=FALSE)})
  metadata <- list('IDs'=IDs, 'psis'=psis, 'seed'=seed)
  save(metadata, file=paste(simpath, '\\metadata.rda', sep=''))
  pos <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/pos.rda'))
  sapply(1:length(psis), function(i) {mix(lapply(1:length(IDs[[i]]), function(j) {
    rewind[rewind$SamID==IDs[[i]][j], ]}), psis[[i]], datapath, simpath, i, version, alter=TRUE)})
}

check <- FALSE
if(check) {
  load('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-mix\\metadata-mix.rda')
  index <- 196
  sams <- metadata$IDs[[index]]
  first.two1 <- paste(strsplit(sams[1], split='')[[1]][1:2], collapse='')
  first.two2 <- paste(strsplit(sams[2], split='')[[1]][1:2], collapse='')
  folder1 <- folder2 <- 'data-CLL'
  if(first.two1=='NA') {
    folder1 <- 'data-hapmap'
  }
  if(first.two2=='NA') {
    folder2 <- 'data-hapmap'
  }
  dat1 <- get(load(paste('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\dat\\', 
                         folder1, '\\dat-', sams[1], '.rda', sep='')))
  dat2 <- get(load(paste('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\dat\\', 
                         folder2, '\\dat-', sams[2], '.rda', sep='')))
  mix.dat <- get(load(paste('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\dat\\data-mix\\mixdat-', index, '.rda', sep='')))
  dat1 <- dat1$dat$cn.data
  dat2 <- dat2$dat$cn.data
  mix.dat <- mix.dat$cn.data
  psi <- metadata$samplePsis[index, ]
  cnplotfun(dat1, 2, 'dat1')
  cnplotfun(dat2, 2, 'dat1')
  cnplotfun(mix.dat, 2, 'mixture')
  mixplotfun(mix.dat, dat1, dat2, 2)
}

###
plotSetTraits <- function(setDir, seq=TRUE, cn=TRUE, merge=FALSE, kmax=5) {
  simfiles <- list.files(setDir)
  metadata <- simfiles[1]
  simfiles <- simfiles[2:length(simfiles)]
  sims <- lapply(1:length(simfiles), function(j) {get(load(paste(setDir, '\\', simfiles[j], sep='')))})
  if(length(sims[[1]])==1) {
    sims <- lapply(1:length(sims), function(x) {sims[[x]]$tumor})
  }
  ks <- sapply(1:length(sims), function(j) {length(which(sims[[j]]$psi > 0))})
  psis <- t(sapply(1:length(sims), function(j) {
    psi <- sims[[j]]$psi
    if(length(psi)<kmax) {
      psi <- c(psi, rep(0, kmax-length(psi)))
    }
    psi
  }))
  frac.altered <- sapply(1:length(sims), function(j) {
    sim <- sims[[j]]
    lengths <- sim$clones[[1]]$cn$end + 1 - sim$clones[[1]]$cn$start
    sum(lengths[unique(unlist(lapply(1:length(sim$clones), function(x) {
      which(sim$clones[[x]]$cn$A != 1 | sim$clones[[x]]$cn$B != 1)})))])/sum(lengths)
  })
  if(merge) {
    sim <- sims[[j]]
    lengths <- sim$clones[[1]]$cn$end + 1 - sim$clones[[1]]$cn$start
    altered <- sapply(1:length(sim$clones), function(x) {sim$clones[[x]]$cn$A != 1 | sim$clones[[x]]$cn$B != 1})
    altered.indices <- which(altered)
    diffs <- diff(altered.indices)
    #To be continued?
  } else {
    length.altered <- lapply(1:length(sims), function(j) {
      sim <- sims[[j]]
      lengths <- sim$clones[[1]]$cn$end + 1 - sim$clones[[1]]$cn$start
      lengths[unique(unlist(lapply(1:length(sim$clones), function(x) {
        which(sim$clones[[x]]$cn$A != 1 | sim$clones[[x]]$cn$B != 1)})))]
    })
    nCNVs <- sapply(1:length(length.altered), function(z) {length(length.altered[[z]])}) 
  }
  length.altered <- unlist(length.altered)
  mutations <- sapply(1:length(sims), function(j) {
    sim <- sims[[j]]
    if(length(sim)==1) {
      sim <- sim$tumor
    }
    length(unique(unlist(lapply(1:length(sim$clones), function(x) {sim$clones[[x]]$seq$mut.id}))))
  })
  if(cn==TRUE & seq==FALSE) {
    #par(mfrow=c(1, 2))
    #hist(ks, breaks=c(.5, 1.5, 2.5, 3.5, 4.5, 5.5), xlab='Number of Clones', main='')
    plot(density(unlist(psis), bw=.05), xlab='Psi', xlim=c(0, 1), main='F) Density of Psi Values across Simulations')
    plot(density(frac.altered, bw=.01), xlab='Genome Altered Fraction', xlim=c(0, 1), main='G) Genome AF Density among Simulations')
    #plot(density(length.altered, bw=.01), xlab='CNV Lengths', xlim=c(0, max(length.altered)), main='')
    #hist(nCNVs, xlab='CNVs per Sample', main='')
  }else if(cn==FALSE & seq==TRUE) {
    #par(mfrow=c(1, 2))
    #hist(ks, breaks=c(.5, 1.5, 2.5, 3.5, 4.5, 5.5), xlab='Number of Clones', main='')
    plot(density(unlist(psis), bw=.05), xlab='Psi', xlim=c(0, 1), main='A) Density of Psi Values across Simulations')
    plot(density(mutations, bw=5), xlab='Mutations per Sample', xlim=c(0, max(mutations)), main='B) Density of Mutations per Simulation')
  }else if(cn & seq) {
    #par(mfrow=c(1, 3))
    #hist(ks, breaks=c(.5, 1.5, 2.5, 3.5, 4.5, 5.5), xlab='Number of Clones', main='')
    plot(density(unlist(psis), bw=.05), xlab='Psi', xlim=c(0, 1), main='C) Density of Psi Values across Simulations')
    plot(density(frac.altered, bw=.01), xlab='Genome Altered Fraction', xlim=c(0, 1), main='D) Genome AF Density among Simulations')
    #plot(density(length.altered, bw=.01), xlab='CNV Lengths', xlim=c(0, max(length.altered)), main='')
    #hist(nCNVs, xlab='CNVs per Sample', main='')
    plot(density(mutations, bw=5), xlab='Mutations per Sample', xlim=c(0, max(mutations)), main='E) Density of Mutations per Simulation')
  }
}

png('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/cf_images/fig_2.png', height=500, width=900, res=100)
layout(matrix(c(1, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 6, 7, 7, 7), 3, 6, byrow = TRUE))
plotSetTraits('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-1', cn=FALSE)
plotSetTraits('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-2')
plotSetTraits('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-3', seq=FALSE)
dev.off()

#png('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\images\\sims-1.png', width=10, height=3.5, units="in", res=300)
#plotSetTraits('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-1', cn=FALSE)
#dev.off()
#png('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\images\\sims-2.png', width=15, height=3.5, units="in", res=300)
#plotSetTraits('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-2')
#dev.off()
#png('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\images\\sims-3.png', width=10, height=3.5, units="in", res=300)
#plotSetTraits('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-3', seq=FALSE)
#dev.off()
#png('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\images\\sims-mix.png', width=10, height=3.5, units="in", res=300)
#plotSetTraits('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-mix', seq=FALSE, merge=FALSE)
#dev.off()

###
imgpath <- 'C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\images'
metadata <- get(load('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-3\\metadata-3.rda'))

plotSNP <- function(chr.sim, chr.dat, j, sam, homFrac=.68, mu.0=0, sigma.0=.03) {
  dat <- get(load(paste('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\dat\\data-3', '\\dat-3-', j, '.rda', sep='')))
  sim <- get(load(paste('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\sim\\sims-3', '\\sim-3-', j, '.rda', sep='')))
  segdata <- dat$dat$cn.data
  #sigma0.lrr <- metadata$data.params$sigma0.lrr
  sigma0.lrr <- .12
  sigma0.baf <- metadata$data.params$sigma0.baf
  segs <- segdata[segdata$chr==chr.sim, ]
  x.mu <- unlist(sapply(1:nrow(segs), function(k) {rep(segs$X[k], segs$markers[k])}))
  y.mu <- unlist(sapply(1:nrow(segs), function(k) {rep(segs$Y[k], segs$markers[k])}))
  index <- 1:length(x.mu)
  hom <- sample(1:length(index), round(length(index)*homFrac), replace=FALSE)
  inverted <- sample(1:length(index), round(length(index)*.5), replace=FALSE)
  x.mu[hom] <- x.mu[hom] + y.mu[hom]
  y.mu[hom] <- 0
  x.inverted <- x.mu[inverted]
  x.mu[inverted] <- y.mu[inverted]
  y.mu[inverted] <- x.inverted
  lrr.mu <- log10((x.mu + y.mu)/2)
  baf.mu <- x.mu/(x.mu + y.mu)
  lrr <- rnorm(length(lrr.mu), lrr.mu, sigma0.lrr)
  baf <- rnorm(length(baf.mu), baf.mu, sigma0.baf)
  baf[which(baf<0)] <- -baf[which(baf<0)]
  baf[which(baf > 1)] <- 1/baf[which(baf > 1)]
  path <- paste('E:\\SNP-HapMap\\MergeByChrom\\Full Data Table_Chr_', chr.dat, '.txt', sep='')
  cndata <- read.table(path, header=T)
  columns <- unique(sapply(4:ncol(cndata), function(x) {strsplit(colnames(cndata)[x], split='[.]')[[1]][1]}))
  samn <- which(columns==sam)
  lrr.dat <- cndata[, samn*3+2]
  baf.dat <- cndata[, samn*3+3]
  par(mfrow=c(2, 2))
  #maintab <- paste('Chr ', chr, sep='')
  plot(1:length(lrr), lrr, xlab='SNP Index', ylab='LRR', ylim=c(-1.5, 1), pch='.', col='gray', 
       main='A) SNP Array Data (simulated data)')
  plot(1:length(lrr.dat), lrr.dat, xlab='SNP Index', ylab='LRR', ylim=c(-1.5, 1), pch='.', col='gray', 
       main='B) SNP Array Data (real data)')
  abline(h=c(log10((1:4)/2)), col=c('gray', 'black', 'gray', 'gray'))
  abline(h=c(log10((1:4)/2)), col=c('gray', 'black', 'gray', 'gray'))
  plot(1:length(baf), baf, main='', xlab='SNP Index', ylab='BAF', ylim=c(0, 1), pch='.', col='gray')
  abline(h=.5, col='black')
  plot(1:length(baf.dat), baf.dat, main='', xlab='SNP Index', ylab='BAF', ylim=c(0, 1), pch='.', col='gray')
  abline(h=.5, col='black')
  #plot(density(baf), xlim=c(-1.5, 1.5), ylim=c(0, 5), main=paste('SD=', round(sd(baf), digits=3), sep=''))
}

plotSEQ <- function(seqdata, seqsim) {
  som.dat <- seqdata[seqdata$somatic_status=='SOMATIC', ]
  vaf.dat <- na.omit(som.dat$alt_read_count/(som.dat$alt_read_count + som.dat$ref_read_count))
  tc.dat <- som.dat$alt_read_count + som.dat$ref_read_count
  seq.sim <- seqsim$dat$seq.data
  som.sim <- seq.sim[seq.sim$status=='somatic', ]
  tc.sim <- som.sim$totalCounts
  vaf.sim <- som.sim$VAF
  par(mfrow=c(2, 2))
  plot(tc.sim[1:225], ylim=c(0, 250), xlab='Index', ylab='Total Read Counts', main='A) Somatic Variants (Simulation)', pch=16)
  plot(tc.dat[1:225], ylim=c(0, 250), xlab='Index', ylab='Total Read Counts', main='B) Somatic Variants (Data)', pch=16)
  plot(vaf.sim[1:225], ylim=c(0, 1), xlab='Index', ylab='VAF', main='', pch=16)
  plot(vaf.dat[1:225], ylim=c(0, 1), xlab='Index', ylab='VAF', main='', pch=16)
}

plotSNP(chr.sim=2, j=68, homFrac=.68, mu.0=0, sigma.0=.03, chr.dat=2, sam='NA18855')
plotSEQ(seqdata=get(load('C:\\Users\\Mark\\OneDrive - The Ohio State University\\deepcnv\\GS1115.rda')), 
    seqsim=get(load('C:\\Users\\Mark\\OneDrive - The Ohio State University\\clonetools\\dat\\data-1\\dat-1-10.rda')))

########################################################################
### Part 1: Simplices
###
### The d-dimensional simplex is the set of nonnegative points in R^d
### that satisfy  x_1 + .. + x_d = 1. These points represent partitions
### of data into subsets, such as taking the fraction of cells that are
### part of each clone.

### Sample n points from the d-simplex
sampleSimplex <- function(n, d = 5) {
    result <- matrix(NA, nrow=n, ncol = d)
    for (i in 1:n) {
        result[i,] <- diff(sort( c(0, 1, runif(d - 1, 0, 1)) ))
    }
    result
}

### Generate a regularly spaced lattice of points in the d-simplex,
### sapnned by k points along each edge.
### Use symmetry to reduce the number we need to try.
generateSimplex <- function(k, d, reps=1){
  simplex <- t(xsimplex(d, k))
  for(i in 1:nrow(simplex)){
    simplex[i,] <- sort(simplex[i,], decreasing=TRUE)
  }
  simplex <- unique(simplex)
  psis <- t(sapply(rep(1:nrow(simplex), reps), function(i){simplex[i,]/k}))
  psis
}

########################################################################
### Part 2: WeightVectors
###
### We also refer to points in a simplex as "weight vectors" (since
### "vector of clonal fractions" is both too long and too specialized).
### We define these as an S4 class so we can build validitiy checking
### into the constructor, and so we can more reliably use them when we
### generate "Tumor" objects later.

setClass("WeightVector", slots=c(psi = "numeric"))
setValidity("WeightVector", function(object) {
  all( object@psi >= 0 ) & sum(object@psi) == 1
})
setMethod("initialize", "WeightVector", function(.Object, psi = 1, ...) {
  .Object <- callNextMethod(.Object) # in case this gets inherited
  if (any(is.na(psi))) {
    stop("Psi should not contain missing values.")
  }
  if (any(psi < 0)) {
    stop("Psi should not contain negative values.")
  }
  if (all(psi == 0)) {
    stop("At least one psi component must be larger than zero.")
  }
  .Object@psi = psi/sum(psi)
  .Object
})
WeightVector <- function(phi) {
  new("WeightVector", psi = phi)
}
setAs("WeightVector", "numeric", function(from) from@psi)


########################################################################
# Part 3: Simulating Data

############ CLONE ############
# A clone is a list of segments, posibly with abnormalities.
setClass("Clone", representation=list(
                    segments = "integer",
                    weights = "numeric"
                    ))

# We should be able to simulate a clone if we know the number of segments,
#  the relative frequency/prevalence of each pure compartment.
Clone <- function(nSegments, weights=rep(1/5, 5), segnames=NULL) {
  # nSegments = integer, the number of segments
  # weights   = vector, the prevalence of each compartment

  if (nSegments < 1) stop("Number of segments must be positive.")
  
  # start with sanity checks on the weights
  weights <- as(WeightVector(weights), "numeric")

  # now sample the compartments to generate a clone
  segs <- sample(length(weights), nSegments, replace=TRUE, prob=weights)
  if (is.null(segnames)) segnames <- paste("Segment", 1:nSegments, sep='')
  if (length(segnames) != length(segs)) stop("Wrong number of segment names.")
  names(segs) <- segnames
  new("Clone", segments=segs, weights=weights)
}

############ TUMOR ############
# A tumor is a set of clones, each of which is associated with a
# fraction, subject to the constraint that the sum of the fractions
# equals one.

setClass("Tumor",
         slots = c(psi = "WeightVector",
                   clones = "list"))
setAs("Tumor", "list", function(from) {
  list(psi = as(from@psi, "numeric"),
       clones = from@clones)
})
setAs("list", "Tumor", function(from) {
  new("Tumor",
      psi = WeightVector(from$psi),
      clones = from$clones)
})
setMethod("summary", signature("Tumor"), function(object, ...) {
    tdata <- object@data
    table(A=tdata[,1], B=tdata[,2])
})

getClone <- function(tumor, i) {
  if (inherits(tumor, "Tumor")) {
    tumor <- as(tumor, "list")
  }
  tumor$clones[[i]] # should we do error checking on 'i'?
}

### TODO: Wrap everything except psi into some sort of parameter class.
Tumor <- function(psi, rounds, nu=100, pcnv=0.5, norm.contam=FALSE, cnmax=4) {
  K <- length(which(psi > 0)) # number of clones
  ## Choose the number of copy number (CN) segments.
  total.segs <- round(runif(1, 250, 500))
  ## Distribute segments across the chromosomes.
  segsperchr <- as.vector(rdirichlet(1, chlens/1000000)) # per megabase?
  segsperchr <- round(segsperchr*total.segs)
  segsperchr[segsperchr < 1] <- 1
  ## Start building a fake DNA copy data frame. Each segment
  ## needs a chromosome with start and end positions.
  chr <- unlist(lapply(1:24, function(i) {
    rep(i, segsperchr[i])
  }))
  ends <- lapply(1:24, function(i) {
    lens <- c(as.vector(rdirichlet(1, rep(1, segsperchr[i]))))
    sapply(1:length(lens), function(j) {sum(lens[1:j])})
  })
  ends <- lapply(1:length(ends), function(i) {
    round(chlens[chr[i]]*ends[[i]])
  })
  starts <- lapply(1:length(ends), function(i) {
    c(1, ends[[i]][1:(length(ends[[i]])-1)]+1)
  })
  ends <- unlist(ends)
  starts <- unlist(starts)
  ## Make the data frame.
  cnmat <- cbind(chr, unlist(starts), unlist(ends), rep(1, length(starts)), 
                 rep(1, length(starts)), 1:length(starts), rep(NA, length(starts)))
  colnames(cnmat) <- c('chr', 'start', 'end', 'A', 'B', 'seg', 'parent.index')

  ## Evolve some clones, starting with the currently normal one.
  startclone <- list('cn'=cnmat, 'seq'=NA)
  id.end <- 0
  clones <- list(startclone)
  while(length(clones) <= rounds) {
    parent.index <- sample(1:length(clones), 1)
    parent <- clones[[parent.index]]
    if(nu > 0 ) { # Add some mutations. Possibly CNV as well
      cnv <- sample(c(TRUE, FALSE), 1, prob = c(pcnv, 1 - pcnv))
    } else {      # Must be a CNV
      cnv <- TRUE
    }
    newclone.cn <- parent$cn
    newclone.cn[, which(colnames(newclone.cn)=='parent.index')] <- parent.index
    newclone.seq <- parent$seq

    ## handle copy number variants
    if(cnv) {
      allele <- sample(c('A', 'B'), 1)
      pool <- which(parent$cn[, which(colnames(cnmat)==allele)] > 0 &
                      parent$cn[, which(colnames(cnmat)==allele)]<cnmax)
      tochange <- sample(pool, 1)
      other.clones <- sapply(1:length(clones), function(j) {
        clones[[j]]$cn[tochange, which(colnames(cnmat)==allele)]
      })
      if(max(other.clones) > 1) {
        delta <- 1
      } else if(min(other.clones) < 1) {
        delta <- -1
      } else {
        delta <- sample(c(-1, 1), 1)
      }
      if(parent.index>1 & length(which(!is.na(unlist(newclone.seq)))) > 0) {
        muts.tochange <- intersect(which(newclone.seq[, i.seg] == tochange),
                                   which(newclone.seq[, i.allele] == allele))
        if(length(muts.tochange) > 0) {
          newclone.seq[muts.tochange, i.cn] <-
            as.numeric(newclone.seq[muts.tochange, i.cn]) + delta
        }
      }
      newclone.cn[tochange, which(colnames(cnmat)==allele)] <-
         as.numeric(newclone.cn[tochange, which(colnames(cnmat)==allele)]) + delta
    } #END: if(cnv)

    ## handle sequence mutations
    nmuts <- round(runif(1, nu/2, 1.5*nu))
    if(nmuts > 0){
      mutsperchr <- as.vector(rmultinom(1, nmuts, chlens/1000000))
      starts <- lapply(1:length(mutsperchr), function(i) {
        unique(round(sort(runif(mutsperchr[i], 1, chlens[i]))))
      })
      starts <- unlist(starts)
      mut.chrs <- unlist(lapply(1:length(mutsperchr), function(i) {
        rep(i, mutsperchr[i])
      }))
      mut.segs <- unlist(sapply(1:length(starts), function(i) {
        col.index <- which(colnames(cnmat)=='chr')
        tab <- cnmat[which(cnmat[, col.index]==mut.chrs[i]), ]
        bin1 <- which(tab[, which(colnames(tab)=='start')]<=starts[[i]])
        index <- bin1[which(tab[bin1, which(colnames(tab)=='end')]>=starts[[i]])]
        seg <- unname(tab[index, which(colnames(tab)=='seg')])
        seg
      }))
      alleles <- sample(c('A', 'B'), length(starts), replace=TRUE)
      mut.ids <- (id.end+1):(id.end+length(starts))
      seqmat <- cbind(mut.chrs, starts, mut.segs, mut.ids, rep(1, length(starts)), alleles)
      id.end <- max(mut.ids)
      colnames(seqmat) <- c('chr', 'start', 'seg', 'mut.id', 'mutated.copies', 'allele')
      newclone.seq <- rbind(newclone.seq, seqmat)
      rownames(newclone.seq) <- NULL
    } else {
      newclone.seq <- NA # No new mutations
    } #END: if(nmuts > 0)
    
    ## Make the new clone, remembering what changed.
    if (length(clones) == 1 & nmuts > 0) {
      i.seg <- which(colnames(seqmat) == 'seg')
      i.cn <- which(colnames(seqmat) == 'mutated.copies')
      i.allele <- which(colnames(seqmat) == 'allele')
    }
    newclone <- list('cn'=newclone.cn, 'seq'=newclone.seq)
    clones <- c(clones, list(newclone))
  } #END: while(length(clones) <= rounds)

  ## This is still safe when K == 1.
  if(norm.contam == TRUE) { # first "clone" consists of the normal cells.
    sampled <- c(1, sample(2:length(clones), K-1, replace=FALSE))
  }else{
    sampled <- sample(2:length(clones), K, replace=FALSE)
  }
  
  clones.final <- lapply(1:length(sampled), function(I) {
    cndf <- as.data.frame(clones[[sampled[I]]]$cn,
                          stringsAsFactors = TRUE)
    if(length(which(!is.na(unlist(clones[[sampled[I]]]$seq))))){
      seqdf <- as.data.frame(clones[[sampled[I]]]$seq,
                             stringsAsFactors = TRUE)
      for(J in which(colnames(seqdf)!='allele')) {
        seqdf[, J] <- as.numeric(as.character(seqdf[, J]))
        seqdf <- na.omit(seqdf[with(seqdf, order(seg, start)), ])
        rownames(seqdf) <- NULL
        seqdf
      }
      seqdf$normal.copies <- na.omit(unlist(lapply(1:nrow(cndf), function(J) {
        total.cn <- cndf$A[J] + cndf$B[J]
        if(length(which(seqdf$seg == J)) > 0){
          normal.cns <- total.cn - seqdf$mutated.copies[which(seqdf$seg == J)]
        } else {
          normal.cns <- NA
        }
        normal.cns
      })))
    } else {
      seqdf <- NA
    }
    if(nu > 0){
      output <- list('cn'=cndf, 'seq'=seqdf)
    }else{
      output <- list('cn'=cndf)
    }
    output
  })
  new("Tumor", clones = clones.final, psi = WeightVector(psi))
}

library(survival)
#library(prodlim)
library(quantmod)
library(ClassComparison)
cll.path <- 'C:/Users/Mark/OneDrive - The Ohio State University/clonetools/dat/data-CLL'
pos <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/pos.rda'))
cll.files <- list.files(cll.path)
samples <- sapply(1:length(cll.files),function(i){strsplit(strsplit(cll.files[i],split='dat-')[[1]][2],split='.rda')[[1]][1]})
cf.results <- lapply(1:length(cll.files), function(j){get(load(paste(cll.path, '/', cll.files[j], sep='')))$res})
cll.ids <- sapply(1:length(cll.files),function(j){strsplit(strsplit(cll.files[j],split='-')[[1]][4],split='.rda')[[1]][1]})
pars <- cf.results[[1]]$pars
rewind <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clonetools/SNP_array_data/rewind_both.rda'))
total.markers <- sum(rewind[rewind$SamID==samples[1],]$num.mark)
clinical <- get(load('C:/Users/Mark/Documents/Lab/clinok.rda'))
#clinical <- get(load('D:/CLL/ClinicalOK.rda'))
eligible <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clinical/eligible.rda'))
clinical$ttp.eligible <- eligible
cids <- rownames(clinical)
common.cids <- intersect(c(cids),samples)
clinical <- clinical[sapply(1:length(common.cids),function(j){which(rownames(clinical)==common.cids[j])}),]
cf.results <- lapply(1:length(common.cids),function(j){cf.results[[which(samples==common.cids[j])]]})
ttp.eligible <- which(clinical$ttp.eligible)

###
ks <- sapply(1:length(cf.results),function(j){length(which(cf.results[[j]]$psi>0))})
columns <- colnames(clinical)

###Treatment (significant):
#vars; note: time variables are in months (365.25/12).
White.blood.count <- clinical$White.blood.count
Rai.Stage <- clinical$Rai.Stage
TimeSample2SigTreat <- clinical$TimeSample2SigTreat
TimeDiag2SigTreat <- clinical$TimeDiagnosis2SigTreat
PFS.time <- clinical$PFS.time
EFS.time <- clinical$EFS.time
mutation.status <- clinical$mutation.status
OSAD <- clinical$OSAfterDiagnosis
OSAS <- clinical$OSAfterSample
TTP <- clinical$TTP.time
TxDSS <- clinical$TxDSS.time

#status (numeric)
NumericSigTreatment <- clinical$NumericSigTreatment
PFS.numericStatus <- clinical$PFS.numericStatus
EFS.numericStatus <- clinical$EFS.numericStatus
NumericVitalStatus <- clinical$NumericVitalStatus
NumericTTPStatus <- clinical$TTP.numericStatus
NumericTxDSSStatus <- clinical$TxDSS.numericStatus

########Cox Proportional Hazards:
colors <- c('red', 'blue', 'purple', 'darkorange')

###Unified segmentation of SNP data
segmentation <- function(dat, bandwidth=10){
  Reduce(rbind,lapply(1:22,function(chrom){
    data <- dat[dat$chrom==chrom,]
    breaks <- c(data$loc.start,data$loc.end)
    #plot(density(breaks,from=0,to=max(na.omit(data$loc.end)),
    #             bw=100),xlim=c(0,max(na.omit(data$loc.end))),main='Segment Breaks',xlab='Position')
    dens <- density(breaks,from=0,to=max(na.omit(data$loc.end)), bw=bandwidth)
    y <- dens$y
    peaks <- findPeaks(y)
    breaks <- round(unique(c(min(dens$x[peaks]),dens$x[peaks])))
    data.frame(chr=chrom,start=breaks[1:(length(breaks)-1)]+1,end=breaks[2:length(breaks)])
  }))
}
segments <- segmentation(rewind)
##

segInfo <- lapply(1:length(common.cids),function(i){
  resdat <- cf.results[[i]]$data$snpdata
  resdat.filt <- cf.results[[i]]$filtered.data$cndata.filt
  A.filt <- cf.results[[i]]$A
  B.filt <- cf.results[[i]]$B
  A <- B <- matrix(1,nrow=nrow(resdat),ncol=length(cf.results[[i]]$psi))
  for(j in 1:length(resdat.filt$seg)){
    A[resdat.filt$seg[j],] <- A.filt[j,]
    B[resdat.filt$seg[j],] <- B.filt[j,]
  }
  psi <- cf.results[[i]]$psi
  dat <- rewind[rewind$SamID==common.cids[i],]
  combined <- Reduce(rbind,lapply(1:22,function(j){
    resdat.chr <- resdat[resdat$chr==j,]
    A.chr <- A[resdat$chr==j,]
    B.chr <- B[resdat$chr==j,]
    CN.chr <- A.chr + B.chr
    if(length(which(resdat$chr==j))==1){
      CN.chr <- t(as.matrix(CN.chr))
    }
    dat.chr <- dat[dat$chrom==j,]
    df <- segments[segments$chr==j,]
    markers.chr <- pos[pos$Chr==j,]
    chrdf <- data.frame(t(sapply(1:nrow(df),function(k){
      overlapping <- dat.chr[which(dat.chr$loc.start<=df$end[k] & dat.chr$loc.end>=df$start[k]),]
      indices <- which(resdat.chr$LRR %in% overlapping$seg.median & resdat.chr$BAF %in% overlapping$AvgBAF)
      indices <- intersect(indices, which(!is.na(resdat.chr$BAF) & which(!is.na(resdat.chr$LRR))))
      if(length(indices)>0){
        index <- indices[which.max(resdat.chr$markers[indices])]
        up <- which(CN.chr[index,]>2)
        down <- which(CN.chr[index,]<2)
        AF <- max(sum(psi[up]), sum(psi[down]))
        if(sum(psi[down]) > sum(psi[up])){
          AF <- -1*AF
        }
        TCN <- resdat.chr$X[index] + resdat.chr$Y[index]
        markers <- length(which(markers.chr$Position <= df$end[k] & markers.chr$Position >= df$start[k]))
        row <- data.frame('chr'=j,'start'=df$start[k],'end'=df$end[k],'AF'=AF,'TCN'=TCN,'markers'=markers)
      }else{
        row <- data.frame('chr'=j,'start'=df$start[k],'end'=df$end[k], 'AF'=NA,'TCN'=NA,'markers'=NA)
      }
      row
    })))
    colnames(chrdf) <- c('chr','start','end','AF','TCN','markers')
    chrdf$chr <- unlist(chrdf$chr)
    chrdf$start <- unlist(chrdf$start)
    chrdf$end <- unlist(chrdf$end)
    chrdf$AF <- unlist(chrdf$AF)
    chrdf$TCN <- unlist(chrdf$TCN)
    chrdf$markers <- unlist(chrdf$markers)
    chrdf
  }))
  combined
})
names(segInfo) <- common.cids
save(segInfo, file='C:/Users/Mark/OneDrive - The Ohio State University/clinical/segInfo.rda')

###Response data:
response.status <- rep(0,length(ttp.eligible))
response.status[which(clinical$Response.1st.sig.treat[ttp.eligible]=='CR')] <- 1


###
load('C:/Users/Mark/OneDrive - The Ohio State University/clinical/segInfo.rda')

#####Clinical vars as a function of allele fraction (or total CN): 
suffixes <- c('OSAS','T2ST','PFS','EFS','TTP','OSAT')
nameset <- c('Overall Survival (from sample)','Time to Treatment','Progression Free Survival',
             'Event Free Survival','Time to Progression','Overall Survival (from treatment)')
filtered <- c(FALSE,FALSE,TRUE,TRUE,TRUE,TRUE)
survtimes <- list(OSAS,TimeDiag2SigTreat,PFS.time,EFS.time,TTP,OSAS)
statuses <- list(NumericVitalStatus,NumericSigTreatment,PFS.numericStatus,EFS.numericStatus,NumericTTPStatus,NumericVitalStatus)
pmat <- pmat.bin <- pmat.cut <- matrix(NA,nrow(segInfo[[1]]),ncol=length(suffixes))
response.pval <- response.oddsRatio <- rep(NA, nrow(segInfo[[1]]))
colnames(pmat) <- colnames(pmat.bin) <- colnames(pmat.cut) <- suffixes
loessfits <- array(NA,dim=c(nrow(segInfo[[1]]),length(suffixes),2,50))

#nMin is the minimum number of non-na alteration fractions for analysis to continue:
nMin <- 20
#kMin is the minimum number of entries for each level necessary to proceed; i.e.,
#if all or almost all samples are unaltered, there's no point in trying to compute
#a p-value
kMin <- 4

#Cox PH Regression (0 vs. nonzero) and Loess fits:
for(seg in 1:nrow(segInfo[[1]])){
  afs <- unlist(sapply(1:length(segInfo),function(j){segInfo[[j]]$AF[seg]}))
  tcns <- unlist(sapply(1:length(segInfo),function(j){segInfo[[j]]$TCN[seg]}))
  afs[which(is.na(tcns))] <- NA
  af.status <- rep(NA, length(afs))
  if(length(na.omit(afs))>=nMin){
    notna <- which(!is.na(afs))
    if(mean(na.omit(afs))>0){
      af.status[notna] <- 0
      af.status[notna][which(afs[notna]>0)] <- 1
    }else{
      af.status[notna] <- 0
      af.status[notna][which(afs[notna]<0)] <- 1
    } 
  }
  L <- length(which(sapply(0:1,function(i){length(which(af.status==i))>kMin})))
  if(L==2 & length(which(!is.na(afs)))>=nMin){
    tbl <- table(af.status[ttp.eligible], response.status)
    test <- fisher.test(tbl)
    response.pval[seg] <- test$p.value
    response.oddsRatio[seg] <- test$estimate
    for(h in 1:length(suffixes)){
      segname <- paste('chr',segInfo[[1]]$chr[seg][[1]],'_',round(segInfo[[1]]$start[seg][[1]]),sep='')
      filename.reg <- paste('C:/Users/Mark/OneDrive - The Ohio State University/clinical/AF_reg/',segname,'-',suffixes[h],'.png',sep='')
      filename.loess <- paste('C:/Users/Mark/OneDrive - The Ohio State University/clinical/AF_loess/',segname,'-',suffixes[h],'.png',sep='')
      if(filtered[h]){
        af.var <- afs[ttp.eligible]
        af.status.var <- af.status[ttp.eligible]
        cph <- coxph(Surv(survtimes[[h]][ttp.eligible],statuses[[h]][ttp.eligible])~af.var)
        cph.bin <- coxph(Surv(survtimes[[h]][ttp.eligible],statuses[[h]][ttp.eligible])~af.status.var)
        fit <- survfit(Surv(survtimes[[h]][ttp.eligible],statuses[[h]][ttp.eligible])~af.status.var)
      }else{
        af.var <- afs
        af.status.var <- af.status
        cph <- coxph(Surv(survtimes[[h]],statuses[[h]])~af.var)
        cph.bin <- coxph(Surv(survtimes[[h]],statuses[[h]])~af.status.var)
        fit <- survfit(Surv(survtimes[[h]],statuses[[h]])~af.status.var) 
      }
      ns <- fit$n
      p <- round(summary(cph)$logtest[3], digits=4)
      p.bin <- round(summary(cph.bin)$logtest[3], digits=4)
      pmat[seg,h] <- p
      pmat.bin[seg,h] <- p.bin
      if(p<.05){
        mainlab <- paste(nameset[h],' - Chrom ',round(segInfo[[1]]$chr[seg][[1]]),':',round(segInfo[[1]]$start[seg][[1]]),
                         '-',round(segInfo[[1]]$end[seg][[1]]),sep='')
        martingales <- residuals(cph,type="martingale")
        loessfit <- loess.smooth(cph$linear.predictors, martingales)
        loessfits[seg,h,1,] <- loessfit$x
        loessfits[seg,h,2,] <- loessfit$y
        legendlabs <- c(paste('AF = 0; n = ',ns[1],sep=''),paste('AF > 0; n = ',ns[2],sep=''))
        xmax <- max(na.omit(survtimes[[h]]))
        png(filename.reg,height=800,width=1000)
        plot(fit,col=colors[1:length(ns)],main=paste(mainlab,'; p-value = ',p,sep=''))
        legend(xmax-.35*xmax,1,legendlabs,col=colors[1:length(ns)],lty=rep(1,3),cex=.9)
        dev.off()
        png(filename.loess,height=800,width=1000)
        scatter.smooth(na.omit(af.var), martingales,xlab='Allele Fraction', ylab='Martingale Residual',main=mainlab)
        dev.off()
      }
    }
  }
}
save(pmat,file='C:/Users/Mark/OneDrive - The Ohio State University/clinical/seg_pvals.rda')
save(pmat.bin,file='C:/Users/Mark/OneDrive - The Ohio State University/clinical/seg_pvals_bin.rda')
save(response.pval,file='C:/Users/Mark/OneDrive - The Ohio State University/clinical/response.pval.rda')
save(response.oddsRatio,file='C:/Users/Mark/OneDrive - The Ohio State University/clinical/response.oddsRatio.rda')
save(loessfits,file='C:/Users/Mark/OneDrive - The Ohio State University/clinical/loess_fits.rda')


###
load('C:/Users/Mark/OneDrive - The Ohio State University/clinical/segInfo.rda')
markers <- segInfo[[1]]$markers
pvals <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clinical/seg_pvals.rda'))
pvals.bin <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clinical/seg_pvals_bin.rda'))
response.pval <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clinical/response.pval.rda'))
response.oddsRatio <- get(load('C:/Users/Mark/OneDrive - The Ohio State University/clinical/response.oddsRatio.rda'))
thresh <- 100
hist(pvals[which(markers>thresh),1],breaks=30,xlab='P-Value',main='Overall Survival')
hist(pvals[which(markers>thresh),2],breaks=30,xlab='P-Value',main='Time to Sig. Treat.')
hist(pvals[which(markers>thresh),3],breaks=30,xlab='P-Value',main='Progression Free Survival')
hist(pvals[which(markers>thresh),4],breaks=30,xlab='P-Value',main='Event Free Survival')
hist(pvals[which(markers>thresh),5],breaks=30,xlab='P-Value',main='Time to Progression')
hist(pvals[which(markers>thresh),6],breaks=30,xlab='P-Value',main='Overall Survival (after treatment)')
hist(response.pval[which(markers>thresh)],breaks=50,xlab='P-Value',main='Response to Treatment')
hist(response.oddsRatio[which(markers>thresh)],breaks=50,xlab='Coefficient',main='Response to Treatment')

#Multiple test correction:
cutoff.t2st <- cutoffSignificant(bt2t <- Bum(pvals[which(markers>thresh),2]), alpha=0.1, by="FDR")
cutoff.ttp <- cutoffSignificant(Bum(pvals[which(markers>thresh),5]), alpha=0.1, by="FDR")

#Looking at 'significant' segs:
segs.sig_t2st <- segInfo[[1]][which(pvals[,2]<cutoff.t2st & markers>thresh),1:3]
segs.sig_ttp <- segInfo[[1]][which(pvals[,5]<cutoff.ttp & markers>thresh),1:3]

#Figure 1: seg 29 (chr 2); seg 346 (chr 12); 476 (chr 13); seg 558 (chr 17); seg 617 (chr 19); seg 627 (chr 21). 
segs <- c(29,346,558,617,627)
ttp.elig <- TTP[ttp.eligible]
numericStatus <- NumericTTPStatus[ttp.eligible]

###Generating prodlim input:
f <- function(seg){
  afs <- unlist(sapply(1:length(segInfo),function(j){segInfo[[j]]$AF[seg]}))
  tcns <- unlist(sapply(1:length(segInfo),function(j){segInfo[[j]]$TCN[seg]}))
  afs[which(is.na(tcns))] <- NA
  af.status <- rep(NA, length(afs))
  notna <- which(!is.na(afs))
  if(mean(na.omit(afs))>0){
    af.status[notna] <- 0
    af.status[notna][which(afs[notna]>0)] <- 1
  }else{
    af.status[notna] <- 0
    af.status[notna][which(afs[notna]<0)] <- 1
  }
  af.status.var <- af.status[ttp.eligible]
  Altered <- af.status.var
  df <- data.frame('TTP'=ttp.elig,'numericStatus'=numericStatus,'Altered'=Altered)
  df
}

dfs <- lapply(c(346,558,29,617,627),f)
prod346 <- prodlim(Surv(TTP,numericStatus)~Altered,data=dfs[[1]])
prod558 <- prodlim(Surv(TTP,numericStatus)~Altered,data=dfs[[2]])
prod29 <- prodlim(Surv(TTP,numericStatus)~Altered,data=dfs[[3]])
prod617 <- prodlim(Surv(TTP,numericStatus)~Altered,data=dfs[[4]])
prod627 <- prodlim(Surv(TTP,numericStatus)~Altered,data=dfs[[5]])

#Figure 1: Known CNVs
par(mfrow=c(1,2))
plot(prod346,logrank=TRUE)
title('A) Trisomy 12',cex=1)
plot(prod558,logrank=TRUE)
title('B) 17p deletion',cex=1)

#Figure 2: Novel CNVs
par(mfrow=c(1,3))
plot(prod29,logrank=TRUE)
title('B) 2p gain',cex=1)
plot(prod617,logrank=TRUE)
title('B) 19p deletion',cex=1)
plot(prod627,logrank=TRUE)
title('B) 21p gain',cex=1)

#Figure 3: Alteration fractions (known and novel)
afmat <- t(sapply(1:length(segInfo),function(i){segInfo[[i]]$AF[segs]}))
par(mfrow=c(2,3))
plot(afmat[,1],ylim=c(0,1),xlab='Patient Index',ylab='Alteration Fraction',main='Alteration: 2P Gain')
plot(afmat[,2],ylim=c(0,1),xlab='Patient Index',ylab='Alteration Fraction',main='Alteration: Trisomy 12')
plot(afmat[,3],ylim=c(0,1),xlab='Patient Index',ylab='Alteration Fraction',main='Alteration: 17P Deletion')
plot(afmat[,4],ylim=c(0,1),xlab='Patient Index',ylab='Alteration Fraction',main='Alteration: 19P Deletion')
plot(afmat[,5],ylim=c(0,1),xlab='Patient Index',ylab='Alteration Fraction',main='Alteration: 21P Gain')

#Figure 4: Is clinical association due to presence vs. absence of CNV, or low vs. high alteration fraction?
#Not sure what to put here yet.

#Figure 5: Response to treatment.
#Not sure why, but R thinks some pvals given by 'fisher.test' that look like 1 are really >1.
response.pval[response.pval>1] <- 1
cutoff.rtt <- cutoffSignificant(Bum(response.pval[which(markers>thresh)]), alpha=0.1, by="FDR")
#Nothing passes?



###I know longer need this for figures 1 and 2, but here's prodlim being used within a function
#and the ensuing issues.
pfun <- function(seg){
  afs <- unlist(sapply(1:length(segInfo),function(j){segInfo[[j]]$AF[seg]}))
  tcns <- unlist(sapply(1:length(segInfo),function(j){segInfo[[j]]$TCN[seg]}))
  afs[which(is.na(tcns))] <- NA
  af.status <- rep(NA, length(afs))
  notna <- which(!is.na(afs))
  if(mean(na.omit(afs))>0){
    af.status[notna] <- 0
    af.status[notna][which(afs[notna]>0)] <- 1
  }else{
    af.status[notna] <- 0
    af.status[notna][which(afs[notna]<0)] <- 1
  }
  af.status.var <- af.status[ttp.eligible]
  Altered <- af.status.var
  df1 <- data.frame('TTP'=ttp.elig,'numericStatus'=numericStatus,'Altered'=Altered)
  prod <- prodlim(Surv(TTP,numericStatus)~Altered,data=df1)
  #plot(prod,logrank=TRUE,xlab='Time to Progression')
  #title(mainlab,cex=1)
  prod
}

par(mfrow=c(1,2))
pfun(346,mainlab='A) Trisomy 12')
pfun(558,mainlab='B) 17p deletion')

par(mfrow=c(1,3))
pfun(29,mainlab='A) 2p gain')
pfun(617,mainlab='B) 19p deletion')
pfun(627,mainlab='C) 21p gain')

temp29 <- pfun(29)
temp617 <- pfun(617)
temp627 <- pfun(627)

temp100 <- pfun(100)
plot(temp100,logrank=TRUE)

par(mfrow=c(1,3))
plot(temp617,logrank=TRUE)
plot(temp29,logrank=TRUE)
plot(temp627,logrank=TRUE)

library(CloneSeeker)

suppressWarnings( RNGversion("3.5.0") )
set.seed(461283)
sampleSimplex(4, 3)
all( apply(sampleSimplex(4, 3), 1, sum) == 1) # assertion

generateSimplex(5, 3)

# These should all fail, for obvious reasons.
try( sampleSimplex(0) )
try( sampleSimplex(-1) )
try( sampleSimplex(4, 0) )
try( sampleSimplex(4, -1) )

#KRC: source('parOpt/clonefinder/functions.R')
library(CloneFinder)
library(gtools)
library(sciClone)
library(phylobase)
library(expands)
library(irr)

format.sims <- function(sim, data, kmax){
  mutids <- unique(unlist(lapply(1:length(sim$clones), function(x){sim$clones[[x]]$seq$mut.id})))
  if(is.null(mutids)){
    mutids <- numeric(0)
  }
  markers <- data$cn.data$markers
  indices <- which(sim$clones[[1]]$cn$seg %in% data$cn.data$seg)
  clones <- lapply(1:length(sim$clones),function(x){
    mutated <- rep(0, length(mutids))
    if(length(mutids)>0){
      if(length(mutids)>0){
        for(j in 1:length(mutids)){
          if(mutids[j] %in% sim$clones[[x]]$seq$mut.id){
            index <- which(sim$clones[[x]]$seq$mut.id==mutids[j])
            mutated[j] <- sim$clones[[x]]$seq$mutated.copies[index]
          }else{
            mutated[j] <- 0
          }
        }
      } 
    }
    mutated <- data.frame('mutid'=mutids, 'mutated.copies'=mutated)
    segs <- data.frame('seg'=sim$clones[[x]]$cn$seg[indices], 'markers'=markers,
              'CN'=sim$clones[[x]]$cn$A[indices] + sim$clones[[x]]$cn$B[indices])
    list('psi'=sim$psi[x], 'cn'=segs, 'mutated'=mutated)
  })
  ord <- sort(sapply(1:length(clones),function(x){clones[[x]]$psi}), index.return=TRUE, decreasing=TRUE)$ix
  clones <- clones[ord]
  if(length(clones)<kmax){
    nullclone <- list('psi'=0, 'cn'=data.frame('seg'=sim$clones[[1]]$cn$seg[indices], 'markers'=markers, 
        'CN'=rep(0, length(markers))), 'mutated'=data.frame('mutid'=mutids, 'mutated.copies'=rep(0, length(mutids))))
    clones <- c(clones, lapply(1:(kmax-length(clones)),function(x){nullclone}))
  }
  clones
}

standardize <- function(res, alg, mutdata, cndata, kmax){
  if(alg=='exp'){
    res <- res$res
    psi <- res$finalSPs[,1]
    ancestors <- res$finalSPs[,5]
    ancestors[which(ancestors<psi)] <- NA
    NAs <- which(is.na(ancestors))
    nMuts <- res$finalSPs[NAs,5]
    maxna <- which.max(NAs)
    exclude <- c(NAs[-maxna])
    exclude <- c(exclude, which(ancestors %in% psi[NAs]))
    psi <- psi[-exclude]
    ancestors <- ancestors[-exclude]
    mutids <- as.vector(res$dm[,which(colnames(res$dm)=='mutid')])
    segs <- unlist(sapply(1:length(mutids),function(k){mutdata$seg[which(mutdata$mut.id==mutids[k])]}))
    segs.unique <- unique(segs)
    markers <- cndata$markers[segs.unique]
    if(is.null(res$dm)){
      clones <- list(list('psi'=c(1),'cn'=data.frame('seg'=numeric(0),'markers'=numeric(0),'CN'=numeric(0)),
                          'mutated'=data.frame('mutid'=numeric(0),'mutated.copies'=numeric(0))))
    }else if(ncol(res$dm)>15){
      clones <- lapply(1:(length(psi)+1),function(x){
        if(x<=length(psi)){
          cnvec <- res$dm[,12]
          cnvec[which(res$dm[,15+x]==1)] <- res$dm[which(res$dm[,15+x]==1),13]
          cnvec <- sapply(1:length(segs.unique),function(y){round(mean(cnvec[which(segs==segs.unique[y])]))})
          if(length(segs.unique)==0){
            cnvec <- numeric(0)
          }
          mutcopies <- res$dm[,15+x]
          if(length(mutids)==0){
            mutcopies <- numeric(0)
          }
          output <- list('psi'=unname(psi[x]-sum(psi[which(ancestors==psi[x])])), 'cn'=data.frame('seg'=segs.unique, 'markers'=markers,                                                                     
                                                                                                  'CN'=cnvec), 'mutated'= data.frame('mutid'=mutids,'mutated.copies'=mutcopies))
        }else{
          cnvec <- sapply(1:length(segs.unique), function(y){round(mean(res$dm[which(segs==segs.unique[y]),12]))})
          if(length(segs.unique)==0){
            cnvec <- numeric(0)
          }
          output <- list('psi'=unname(1-sum(psi[which(is.na(ancestors))])), 'cn'=data.frame('seg'=segs.unique,'markers'=markers,
                                                                                            'CN'=cnvec), 
                         'mutated'=data.frame('mutid'=mutids, 'mutated.copies'=rep(0, length(mutids))))
        }
        output
      })
    }else{
      psi <- psi[which.max(sapply(1:length(psi),function(x){length(which(res$dm[,10]==psi[x]))}))]
      clones <- list(list('psi'=psi,'cn'=data.frame('seg'=segs.unique,'markers'=markers,
                                                    'CN'=sapply(segs.unique, function(x){round(mean(res$dm[which(segs==x),13]))})),
                          'mutated'=data.frame('mutid'=mutids, 'mutated.copies'=rep(1, length(mutids)))), list('psi'=unname(1-psi), 
                                                                                                               'cn'=data.frame('seg'=segs.unique,'markers'=markers,
                                                                                                                               'CN'=sapply(segs.unique, function(x){round(mean(res$dm[which(segs==x),12]))})), 
                                                                                                               'mutated'=data.frame('mutid'=mutids, 'mutated.copies'=rep(0, length(mutids)))))
    }
  }else if(alg=='sc'){
    sc <- res$res
    psi <- rep(0, kmax)
    psi.sc <- sort(sc@clust$pi, decreasing=TRUE)
    psi[1:length(psi.sc)] <- psi.sc
    merged <- sc@vafs.merged
    mutids <- merged[,6]
    segs <- unlist(sapply(1:length(mutids),function(k){mutdata$seg[which(mutdata$mut.id==mutids[k])]}))
    segs.unique <- unique(segs)
    markers <- cndata$markers[segs.unique]
    clones <- lapply(1:length(psi.sc),function(x){
      muts <- rep(0, nrow(merged))
      muts[which(merged$cluster==x)] <- 1
      mutated <- data.frame('mutid'=mutids, 'mutated.copies'=muts)
      CN <- sapply(segs.unique, function(y){
        round(mean(merged[which(segs==y),8]))
      })
      cn <- data.frame('seg'=segs.unique, 'markers'=markers, 'CN'=CN)
      list('psi'=psi[x], 'cn'=cn, 'mutated'=mutated)
    })
  }else if(alg=='cf'){
    if(!is.null(res$res)){
      res <- res$res
    }
    clones <- lapply(1:length(res$psi), function(x){
      if(res$psi[x]==0){
        cnvec <- rep(0, nrow(cndata))
      }else{
        cnvec <- rep(2, nrow(cndata))
      }
      if(class(res)=='matrix'){
        cnvec[as.numeric(rownames(res$A))] <- res$A[,x] + res$B[,x]
      }
      if(class(res$mutated)=='matrix'){
        out <- list('psi'=res$psi[x], 'cn'=data.frame('seg'=cndata$seg,'markers'=cndata$markers,'CN'=cnvec), 
                    'mutated'=data.frame('mutid'=res$filtered.data$mutdata.filt$mut.id, 'mutated.copies'=res$mutated[,x]))
      }else{
        out <- list('psi'=res$psi[x], 'cn'=data.frame('seg'=cndata$seg,'markers'=cndata$markers,'CN'=cnvec), 
                    'mutated'=data.frame('mutid'=numeric(0), 'mutated.copies'=numeric(0)))
      }
      out
    })
  }
  ord <- sort(sapply(1:length(clones),function(x){clones[[x]]$psi}), index.return=TRUE, decreasing=TRUE)$ix
  clones <- clones[ord]
  if(length(clones)<kmax){
    nullclone <- list('psi'=0, 'cn'=data.frame('seg'=segs.unique, 'markers'=markers, 'CN'=rep(0,length(markers))),
                      'mutated'=data.frame('mutid'=mutids, 'mutated.copies'=rep(0, length(mutids))))
    clones <- c(clones, lapply(1:(kmax-length(clones)),function(x){nullclone}))
  }
  psis <- sapply(1:length(clones),function(j){clones[[j]]$psi})
  for(j in which(psis==0)){
    clones[[j]]$cn$CN <- rep(0,length(clones[[j]]$cn$CN))
    clones[[j]]$mutated$mutated.copies <- rep(0,length(clones[[j]]$mutated$mutated.copies))
  }
  clones
}

concordance <- function(x, y){
  if(class(x)=='matrix'){
    x <- as.vector(x)
  }
  if(class(y)=='matrix'){
    y <- as.vector(y)
  }
  if(sd(x)==0 | sd(y)==0){
    concord <- length(which(x==y))/length(x)
  }else{
    rho <- cor(x,y)
    mu_x <- mean(x)
    mu_y <- mean(y)
    sigma_x <- sd(x)
    sigma_y <- sd(y)
    concord <- 2*rho*sigma_x*sigma_y/(sigma_x^2 + sigma_y^2 + (mu_x - mu_y)^2)
  }
  concord
}

assessAlgs <- function(resultpaths, algs, simpath, datapath){
  sim <- get(load(simpath))
  dat <- get(load(datapath))
  if(!is.null(sim$tumor)){
    sim <- sim$tumor
  }
  if(!is.null(dat$dat)){
    dat <-dat$dat
  }
  cndata <- dat$cn.data
  tr <- format.sims(sim, dat, kmax=5)
  temp <- get(load(resultpaths[[which(unlist(algs)=='cf')]]))$res
  mutdata.filt <- temp$filtered.data$mutdata.filt
  standard <- lapply(1:length(resultpaths),function(j){
    standardize(get(load(resultpaths[[j]])), algs[[j]], mutdata.filt, cndata, kmax=5)
  })
  common.seg <- standard[[1]][[1]]$cn$seg
  common.mut <- standard[[1]][[1]]$mutated$mutid
  if(length(standard)>1){
    for(j in 2:length(standard)){
      common.seg <- intersect(common.seg, standard[[j]][[1]]$cn$seg)
      common.mut <- intersect(common.mut, standard[[j]][[1]]$mutated$mutid)
    } 
  }
  absent.in.true <- common.mut[!common.mut %in% tr[[1]]$mutated$mutid]
  absent.in.res <- tr[[1]]$mutated$mutid[!tr[[1]]$mutated$mutid %in% common.mut]
  false.pos.muts <- data.frame('mutid'=absent.in.true,'mutated.copies'=rep(0,length(absent.in.true)))
  false.neg.muts <- data.frame('mutid'=absent.in.res,'mutated.copies'=rep(0,length(absent.in.res)))
  lapply(1:length(standard),function(j){
    true <- tr
    res <- standard[[j]]
    for(k in 1:5){
      res[[k]]$cn <- res[[k]]$cn[which(res[[k]]$cn$seg %in% common.seg),]
      res[[k]]$cn$CN[is.na(res[[k]]$cn$CN)] <- 2
      res[[k]]$mutated <- res[[k]]$mutated[which(res[[k]]$mutated$mutid %in% common.mut),]
      res[[k]]$mutated <- rbind(res[[k]]$mutated,false.neg.muts)
      true[[k]]$cn <- true[[k]]$cn[which(true[[k]]$cn$seg %in% common.seg),]
      true[[k]]$mutated <- rbind(true[[k]]$mutated,false.pos.muts)
      res[[k]]$cn <- res[[k]]$cn[with(res[[k]]$cn,order(res[[k]]$cn$seg)),]
      res[[k]]$mutated <- res[[k]]$mutated[with(res[[k]]$mutated,order(res[[k]]$mutated$mutid)),]
      true[[k]]$cn <- true[[k]]$cn[with(true[[k]]$cn,order(true[[k]]$cn$seg)),]
      true[[k]]$mutated <- true[[k]]$mutated[with(true[[k]]$mutated,order(true[[k]]$mutated$mutid)),]
    }
    tr.psis <- sapply(1:5,function(k){tr[[k]]$psi})
    psi.diffs <- sapply(1:5,function(k){res[[k]]$psi - true[[k]]$psi})
    cndiffs <- t(sapply(1:5,function(k){res[[k]]$cn$CN - true[[k]]$cn$CN}))
    mutdiffs <- t(sapply(1:5,function(k){res[[k]]$mutated$mutated.copies - true[[k]]$mutated$mutated.copies}))
    if(nrow(cndiffs)!=5){
      cndiffs <- t(cndiffs)
    }
    if(length(common.seg)==0){
      weighted.cndiffs <- NA
      concord.cn <- NA
    }else{
      weighted.cndiffs <- t(tr.psis*cndiffs)
      concord.cn <- concordance(sapply(1:5,function(j){true[[j]]$cn$CN}),sapply(1:5,function(j){res[[j]]$cn$CN}))
    }
    if(nrow(mutdiffs)!=5){
      mutdiffs <- t(mutdiffs)
    }
    if(length(common.mut)==0){
      weighted.mutdiffs <- NA
      concord.mut <- NA
    }else{
      weighted.mutdiffs <- t(tr.psis*mutdiffs)
      concord.mut <- concordance(sapply(1:5,function(j){true[[j]]$mutated$mutated.copies}),sapply(1:5,function(j){res[[j]]$mutated$mutated.copies}))
    }
    kdiff <- length(which(sapply(1:5,function(k){standard[[j]][[k]]$psi})>0)) - length(which(sapply(1:5,function(k){true[[k]]$psi})>0))
    concord.psi <- concordance(sapply(1:5,function(k){true[[k]]$psi}),sapply(1:5,function(k){standard[[j]][[k]]$psi}))
    list('k.dist'=(kdiff^2)^.5,'psi.dist'=sum(psi.diffs^2)^.5,'cn.dist'=sum(weighted.cndiffs^2)^.5,'mut.dist'=sum(weighted.mutdiffs^2)^.5,
         'kdiff'=kdiff,'avg.psidiff'=mean(psi.diffs),'avg.cndiff'=mean(weighted.cndiffs),'avg.mutdiff'=mean(weighted.mutdiffs),
         'concord.psi'=concord.psi,'concord.cn'=concord.cn,'concord.mut'=concord.mut,'true'=true,'res'=res)
  })
}

#####################################################################
algs <- list('cf')
simpaths <- sapply(1:300,function(j){paste('parOpt/sim/sims-mix/mixsim-',j,'.rda',sep='')})
datapaths <- sapply(1:300,function(j){paste('parOpt/dat/data-mix/mixdat-', j, '.rda', sep='')})
resultpaths <- lapply(1:300,function(j){paste('parOpt/res/res-mix/res-cf-mix/res-cf-mix-', j, '.rda', sep='')})
assessments.mix <- lapply(1:300, function(i){assessAlgs(resultpaths[[i]],algs,simpaths[i],datapaths[i])})
save(assessments.mix, file='parOpt/assess/assess-mix/assessments-mix-26.rda')

algs <- list('cf','sc','exp')
simpaths <- sapply(1:300,function(j){paste('parOpt/sim/sims-1/sim-1-',j,'.rda',sep='')})
datapaths <- sapply(1:300,function(j){paste('parOpt/dat/data-1/dat-1-', j, '.rda', sep='')})
resultpaths <- lapply(1:300,function(j){list(paste('parOpt/res/res-1/res-cf-1/res-cf-1-', j, '.rda', sep=''),
                                             paste('parOpt/res/res-1/res-sc-1/res-sc-1-', j, '.rda', sep=''),
                                             paste('parOpt/res/res-1/res-exp-1/res-exp-1-', j, '.rda', sep=''))})
assessments.1 <- lapply(1:300, function(i){assessAlgs(resultpaths[[i]],algs,simpaths[i],datapaths[i])})
save(assessments.1, file='parOpt/assess/assess-1/assessments-1-4.rda')

algs <- list('cf','sc','exp')
simpaths <- sapply(1:300,function(j){paste('parOpt/sim/sims-2/sim-2-',j,'.rda',sep='')})
datapaths <- sapply(1:300,function(j){paste('parOpt/dat/data-2/dat-2-', j, '.rda', sep='')})
resultpaths <- lapply(1:300,function(j){list(paste('parOpt/res/res-2/res-cf-2/res-cf-2-', j, '.rda', sep=''),
                                             paste('parOpt/res/res-2/res-sc-2/res-sc-2-', j, '.rda', sep=''),
                                             paste('parOpt/res/res-2/res-exp-2/res-exp-2-', j, '.rda', sep=''))})
assessments.2 <- lapply(1:300, function(j){assessAlgs(resultpaths[[j]],algs,simpaths[j],datapaths[j])})
save(assessments.2, file='parOpt/assess/assess-2/assessments-2-5.rda')

algs <- list('cf')
simpaths <- sapply(1:300,function(j){paste('parOpt/sim/sims-3/sim-3-',j,'.rda',sep='')})
datapaths <- sapply(1:300,function(j){paste('parOpt/dat/data-3/dat-3-', j, '.rda', sep='')})
resultpaths <- lapply(1:300,function(j){paste('parOpt/res/res-3/res-cf-3/res-cf-3-', j, '.rda', sep='')})
assessments.3 <- lapply(1:300, function(i){assessAlgs(resultpaths[[i]],algs,simpaths[i],datapaths[i])})
save(assessments.3, file='parOpt/assess/assess-3/assessments-3-3.rda')

library(CloneSeeker)

fixSmry <- function(X, D = NULL, ...) {
  if (!is.null(D)) {
    S <- summary(X, digits = D, ...)
  } else {
    S <- summary(X, ...)
  }
  W <- grep("NA's", S)
  if (any(W)) S[W] <- gsub("NA's", "NAs", S[W])
  names(S) <- gsub("NA's", "NAs", names(S))
  S
}

dataPars <- list(snps.seq = 1000000,
                 snps.cgh = 600000,
                 mu = 70,
                 sigma.reads = 25,
                 sigma0.lrr = 0.15, 
                 sigma0.baf = 0.03,
                 density.sigma = 0.1)
psis <- c(0.6, 0.3, 0.1)

suppressWarnings( RNGversion("3.5.0") )
set.seed(412634)
### Mutations only? without contamination
tumor <- Tumor(psis, rounds = 400, nu = 100, pcnv = 0, norm.contam = FALSE)
clone <- getClone(tumor, 1)
summary(clone$cn)  # why is the parent index constant?
summary(clone$seq)

### Mutations only? with normal contamination
tumor <- Tumor(psis, rounds = 400, nu = 100, pcnv = 0, norm.contam = TRUE)
clone <- getClone(tumor, 1)
fixSmry(clone$cn)  # why is the parent index missing?
fixSmry(clone$seq) # this looks like a bug to me.

### CNV and Mutations, without normal contamination
tumor <- Tumor(psis, rounds = 400, nu = 100, pcnv = 0.5, norm.contam = FALSE)
clone <- getClone(tumor, 1)
summary(clone$cn)
summary(clone$seq)

### CNV and Mutations, with normal contamination
tumor <- Tumor(psis, rounds = 400, nu = 100, pcnv = 0.5, norm.contam = TRUE)
clone <- getClone(tumor, 1)
fixSmry(clone$cn)
fixSmry(clone$seq) # this looks like a bug to me.

### CNV-only, with normal contamination
tumor <- Tumor(psis, rounds = 400, nu = 0, pcnv = 1, norm.contam = TRUE) 
clone <- getClone(tumor, 1)
fixSmry(clone$cn)
summary(clone$seq) # this is correct

### CNV-only, without normal contamination
tumor <- Tumor(psis, rounds = 400, nu = 0, pcnv = 1, norm.contam = FALSE)
clone <- getClone(tumor, 1)
summary(clone$cn)
summary(clone$seq) # this is correct

### Test coercion routines
tumor <- as(tumor, "list")
object <- as(tumor, "Tumor")

### Test data generation (i.e., simulation)
dataset <- generateTumorData(object,
                dataPars$snps.seq, dataPars$snps.cgh, dataPars$mu,
                dataPars$sigma.reads, dataPars$sigma0.lrr,
                dataPars$sigma0.baf, dataPars$density.sigma)
class(dataset)
length(dataset)
names(dataset)
lapply(dataset, class)
lapply(dataset, dim)
summary(as.matrix(dataset$cn.data), digits = 3)
fixSmry(dataset$seq.data[,-(6:7)], character.method = NULL)
## note that all mut.id are NA's since there were no mutations

plotTumorData(object, dataset)

if (packageVersion("CloneSeeker") < "0.9.0") {
  stop("You need to update 'CloneSeeker'.")
}
library("CloneSeeker")

fixSmry <- function(X, D = NULL, ...) {
  if (!is.null(D)) {
    S <- summary(X, digits = D, ...)
  } else {
    S <- summary(X, ...)
  }
  W <- grep("NA's", S)
  if (any(W)) S[W] <- gsub("NA's", "NAs", S[W])
  names(S) <- gsub("NA's", "NAs", names(S))
  S
}


psiSets <- list(c(1),                     # only one clone
                c(0.7, 0.3),              # two clones
                c(0.6, 0.25, 0.15),       # three clones
                c(0.5, 0.25, 0.15, 0.10)) # four clones

paramSets <- data.frame(nu = rep(c(0, 100, 100), each = 2),
                        pcnv = rep(c(1, 0.5, 0), each = 2),
                        norm.contam = rep(c(TRUE, FALSE), times = 3))

paramSets <- data.frame(nu = rep(c(0, 100, 100)),
                        pcnv = rep(c(1, 0.5, 0)))

#########################################
### Simulate the data sets.
suppressWarnings( RNGversion("3.5.0") )
set.seed(20248)
simData <- list()
for (psi in psiSets) {
  cat("Working on psi =", psi, "\n", file=stderr())
  for (J in 1:nrow(paramSets)) {
    cat("Working on set J =", J, "\n", file=stderr())
    tumor <- Tumor(psi,
                   rounds = 100,
                   cnmax = 4,
                   nu = paramSets$nu[J],
                   pcnv = paramSets$pcnv[J],
                   norm.contam = FALSE)
    dset <- generateTumorData(tumor,
                              snps.seq = 10000,
                              snps.cgh = 600000,
                              mu = 70,
                              sigma.reads = 25,
                              sigma0.lrr = 0.15, 
                              sigma0.baf = 0.03,
                              density.sigma = 0.1)
    simData[[1 + length(simData)]] <- list(tumor = tumor,
                                           dset = dset,
                                           psi = psi,
                                           params = paramSets[J,])
  }
}
length(simData)

#########################################
### Set up algorithm parameters
psis.20 <- generateSimplex(20,5)
cnmodels <- as.matrix(expand.grid(lapply(1:5, function(i){ 0:5 })))
pars <- list(sigma0=5, theta = 0.9, ktheta = 0.3, mtheta = 0.9,
             alpha = 0.5, thresh = 0.04, cutoff = 100, Q = 100, iters = 4)

### Don't run full tests at CRAN.
if (Sys.getenv("KRC_DO_ALL_TESTS") == "TRUE") {
  testset <- 1:length(simData)
} else {
  testset <- c(1, 5)
}

for (J in testset) {
  cat("\n\nDataset", J, "\n", file=stdout())
  dset <- simData[[J]]$dset
  ## Cheat and limit the number of mutations to speed up the algorithm for CRAN
  ## This code was copied out of the "seekClones" algorithm.
  seqdata <- dset$seq.data
  if(nrow(seqdata) > 0) {
    read.den <- density(seqdata$refCounts)
    peak <- read.den$x[which.max(read.den$y)]
  }
  mutdata <- seqdata[seqdata$status == 'somatic',]
  if (nrow(mutdata) > 1) {
    mut.filt <- CloneSeeker:::filterMutations(mutdata, mu=peak, threshold=3)
    seqdata <- mut.filt$mat
    if (nrow(seqdata) > 10) seqdata <- seqdata[1:10,]
  }
  ## "Try" was placed here during debugging. These should succeed.
  ra <- try( runAlg(dset$cn.data, seqdata,
                    cnmodels, psis.20,
                    pars = pars, imputedCN = NULL) )
  if (inherits(ra, "try-error")) {
    cat(ra, "\n", stdout())
  } else {
    cat("psi:\n")
    print(ra$psi)
    cat("A,B:\n")
    print(cbind(A = ra$A, B = ra$B))
    cat("eta:\n")
    print(data.frame(etaA = ra$etaA, etaB = ra$etaB))
    cat("indices:\n")
    print(lapply(ra$indices, function(X) {
      if (length(X) == 0) {
        cat("Empty.\n")
      } else {
        summary(X)
      }
    }))
    cat("filtered data:\nMutations:\n")
    print(summary(ra$filtered.data$mutdata.filt, character.method = NULL))
    if (J != 5) {
      cat("CNVs:\n")
      print(summary(as.matrix(ra$filtered.data$cndata.filt),
                    digits = 2, character.method = NULL))
    }
    print(fixSmry(ra$mutated))
    cat("posteriors\n")
    print(fixSmry(ra$psiPosts))
  }
}

if (FALSE) { # save the profiling code for later
  Rprof("profile3.txt")
  for (J in c(1, 5, 9)) {
    dset <- simData[[J]]$dset
    ra <- seekClones(dset$cn.data, dset$seq.data,
                     cnmodels, psis.20,
                     pars = pars, imputedCN = NULL)
  }
  Rprof(NULL)
  summaryRprof("profile3.txt")
}

# Method of moments to reparametrize beta distribution.
estBetaParams <- function(mu, var) {
  alpha <- ((1 - mu) / var - 1 / mu) * mu ^ 2
  beta <- alpha * (1 / mu - 1)
  c('alpha' = abs(alpha), 'beta' = abs(beta)) # shouldn't negatives be an error ?! 
}

rbeta2 <- function(n, mu, sigma){
  params <- estBetaParams(mu, sigma^2)
  rbeta(n, params[1], params[2])
}

dbeta2 <- function(x, mu, sigma, log=FALSE){
  params <- estBetaParams(mu, sigma^2)
  dbeta(x, params[1], params[2], log=log)
}

psiPrior <- function(psi, alpha=.5, kmax=5, minim=.001){
  psivec <- psi
  psivec[which(psivec<minim)] <- 0
  psivec <- sort(psivec/sum(psivec), decreasing=TRUE)
  if(length(which(psivec>0))==1){
    psivec[1] <- 1 - minim
    psivec[2] <- minim
  }
  p <- log(ddirichlet(psivec[c(which(psivec>0))], rep(alpha, length(which(psivec>0)))))
  p
}


filterCN <- function(data, threshold){
  indices <- which(abs(data$X - 1) >= threshold | abs(data$Y - 1) >= threshold)
  if (length(indices) < 1) { # always return at least one segment, even if it's normal
    indices <- 1
  }
  list('mat'=data[indices,], 'indices'=indices)
}

filterMutations <- function(mutdata, mu, threshold){
  indices <- which(abs(mutdata$refCounts - mu) >= threshold | abs(mutdata$varCounts - mu) >= threshold)
  ids <- mutdata$mut.id[indices]
  list(mat = mutdata[indices,], ids = ids)
}

seqSeg <- function(seqsnps, len, thresh){
  sigma0 <- sd(c(seqsnps$refCounts, seqsnps$varCounts))
  dens <- density(seqsnps$refCounts)
  mu <- dens$x[which.max(dens$y)]
  chrs <- unique(seqsnps$chr)
  dflist <- lapply(1:length(chrs),function(i){
    seqdata.chr <- seqsnps[seqsnps$chr == chrs[i],]
    meandiffs <- t(sapply(1:(nrow(seqdata.chr) - len), function(j) {
      mu1.ref <- mean(seqdata.chr$refCounts[((j - 1)*len+1):(j*len)])
      mu2.ref <- mean(seqdata.chr$refCounts[(j*len+1):((j+1)*len)])
      mu1.var <- mean(seqdata.chr$varCounts[((j - 1)*len+1):(j*len)])
      mu2.var <- mean(seqdata.chr$varCounts[(j*len + 1):((j + 1)*len)])
      c(abs(mu2.ref - mu1.ref), abs(mu2.var - mu1.var))
    }))
    peaks.ref <- findPeaks(meandiffs[,1], thresh=.25)
    peaks.var <- findPeaks(meandiffs[,2], thresh=.25)
    peaks <- sort(unique(c(peaks.ref, peaks.var)), decreasing=FALSE)
    if(length(peaks) > 0){
      start <- c(1, 1 + len*sapply(1:length(peaks), function(j) {peaks[j]} ))
      end <- c(start[-1] - 1, nrow(seqdata.chr))
    }else{
      start <- 1
      end <- nrow(seqdata.chr)
    }
    markers <- end - start + 1
    X <- sapply(1:length(start), function(j) {median(seqdata.chr$refCounts[start[j]:end[j]])})/mu
    Y <- sapply(1:length(start), function(j) {median(seqdata.chr$varCounts[start[j]:end[j]])})/mu
    LRR <- log10((X + Y)/2)
    BAF <- X/(X + Y)
    data.frame('chr'=rep(chrs[i],length(start)), 'start'=start, 'end'=end, 'LRR'=LRR, 
               'BAF'=BAF, 'X'=X, 'Y'=Y, 'markers'=markers)
  })
  df <- Reduce(rbind, dflist)
  df$seg <- 1:nrow(df)
  df
}

if (!isGeneric("hist"))
  setGeneric("hist",
             function(x, ...) standardGeneric("hist"))

if (!isGeneric("plot"))
  setGeneric("plot",
             function(x, y, ...) standardGeneric("plot"))

if (!isGeneric("summary"))
  setGeneric("summary",
             function(object, ...) standardGeneric("summary"))

### The first three functions are here for backwards compatibility.
### They are officially "deprecated", which means (a) you should not
### use them in new code because (b) at some point they will simply
### go away.
tumorGen <- function(...) {
  tumor <- Tumor(...)
  as(tumor, "list")
}

dataGen <- function(tumor, ...) {
  generateTumorData(as(tumor, "Tumor"), ...)
}

runAlg <- function(...) seekClones(...)

#debugMe <- FALSE

psiOptim <- function(cndata.filt, mutdata.filt, psis, cnmodels, pars, cnmax=5, kPriors=NULL, kmax = 5){
  if(is.null(kPriors)){
    kPriors <- dgeom((1:5)-1, prob=pars$ktheta, log=TRUE)
  }
  kPriorset <- sapply(1:nrow(psis), function(k){kPriors[length(which(psis[k,]>pars$thresh))]})
  dirichletPriors <- sapply(1:nrow(psis), function(k){psiPrior(psis[k,])})
  ## Outer loop, over psi vectors
  res <- lapply(1:nrow(psis), function(Ipsi) {
#    if(debugMe) cat("Ipsi =", Ipsi, "\n", file=stderr())
    nonzero <- which(psis[Ipsi,] > 0)
    models <- cnmodels[,nonzero]
    if(length(nonzero) == 1){
      models <- as.matrix(models)
    }
    models <- as.matrix(unique(as.data.frame(models[,nonzero])))
    cnPriors <- sapply(1:nrow(models), function(j) {sum(dgeom(abs(1-models[j,]), prob=pars$theta, log=TRUE))})
    index.upper <- which(sapply(1:nrow(models), function(m) {length(which(models[m,] < 1)) == 0}))
    index.lower <- which(sapply(1:nrow(models), function(m) {length(which(models[m,] > 1)) == 0}))
    mods.upper <- as.matrix(models[index.upper,])
    mods.lower <- as.matrix(models[index.lower,])
    if(length(index.upper)==1) {
      mods.upper <- t(mods.upper)
    }
    if(length(index.lower)==1) {
      mods.lower <- t(mods.lower)
    }
    upper.priors <- cnPriors[index.upper]
    lower.priors <- cnPriors[index.lower]
    etas.upper <- as.vector(psis[Ipsi, nonzero] %*% t(mods.upper))
    etas.lower <- as.vector(psis[Ipsi, nonzero] %*% t(mods.lower))
    etaList <- list(etas.lower, etas.upper)
    priorList <- list(lower.priors, upper.priors)
    modList <- list(mods.lower, mods.upper)
    x.numbers <- y.numbers <- rep(1, nrow(cndata.filt))
    x.numbers[which(cndata.filt$X > 1)] <- 2
    y.numbers[which(cndata.filt$Y > 1)] <- 2
    xmat <- t(sapply(1:nrow(cndata.filt), function(L) {
      post <- dnorm(cndata.filt$X[L]-etaList[[x.numbers[L]]], 0, pars$sigma0/(cndata.filt$markers[L]^.5), log=TRUE) + 
        priorList[[x.numbers[L]]]
      avec <- c(modList[[x.numbers[L]]][which.max(post),], rep(0, kmax-length(nonzero)))
      c(avec, etaList[[x.numbers[L]]][which.max(post)], max(post))
    }))
    ymat <- t(sapply(1:nrow(cndata.filt), function(L) {
      post <- dnorm(cndata.filt$Y[L]-etaList[[y.numbers[L]]], 0, pars$sigma0/(cndata.filt$markers[L]^.5), log=TRUE) + 
        priorList[[y.numbers[L]]]
      bvec <- c(modList[[y.numbers[L]]][which.max(post),], rep(0, kmax-length(nonzero)))
      c(bvec, etaList[[y.numbers[L]]][which.max(post)], max(post))
    }))
    if(nrow(cndata.filt)==0 | length(na.omit(cndata.filt$seg))==0){
      xmat <- ymat <- matrix(numeric(),nrow=0,ncol=kmax+2)
    }
    psiPost <- sum(xmat[, kmax + 2]) + sum(ymat[, kmax + 2])
    cnRes <- list(A = xmat[, 1:kmax, drop=FALSE],
                  B = ymat[, 1:kmax, drop=FALSE],
                  etaA = xmat[, kmax+1],
                  etaB = ymat[, kmax+1],
                  post = psiPost)
    if(nrow(mutdata.filt) > 0) {
      ## Nested loop, over mutation data
      mutmat <- t(sapply(1:nrow(mutdata.filt) ,function(Imut) {
        cn.index <- which(cndata.filt$seg==mutdata.filt$seg[Imut])
        if(length(cn.index) > 0) {
          A <- cnRes$A[cn.index,]
          B <- cnRes$B[cn.index,]
        } else {
          A <- B <- rep(1,kmax)
        }
        mut.models <- as.matrix(expand.grid(lapply(1:kmax,function(m) {0:max(c(A[m], B[m]))})))
        mll <- sapply(1:nrow(mut.models), function(Imm) {
          if(sum((A+B)*psis[Ipsi,]) > 0) {
            p <- sum(mut.models[Imm,]*psis[Ipsi,])/sum(psis[Ipsi,]*(A+B))
            if(p<.0001) {
              p <- .0001
            }
          } else {
            p <- .0001
          }
          log(sum(dbinom(mutdata.filt$varCounts[Imut],
                         mutdata.filt$totalCounts[Imut],
                         p))) #PROF: one-third of total time
        })
        ## Inner loop, over models for mutation data
        mprior <- sapply(1:nrow(mut.models), function(Imm) {
          dgeom(length(unique(mut.models[Imm, which(psis[Ipsi,] > pars$thresh)])) - 1,
                prob=pars$mtheta, log=TRUE) # PROF:about one-fourth of total time
        })
        ## End: Inner loop, over models for mutation data
        mposts <- mll + mprior
        maxind <- which.max(mposts)
        etaM <- sum(psis[Ipsi,]*mut.models[maxind,])/sum(psis[Ipsi,]*(A+B))
        if(is.na(etaM)) {
          etaM <- 0
        }
        unname(c(mut.models[which.max(mposts),],mposts[maxind],etaM))
      }))
      ## End: Nested loop, over mutation data
      mutPosts <- sum(mutmat[,ncol(mutmat)-1])
      psiPost <- psiPost + sum(mutPosts)
      etaM <- mutmat[,ncol(mutmat)]
      mutated <- mutmat[,1:(ncol(mutmat)-2)]
      rownames(mutated) <- rownames(mutdata.filt)
    } else {
      mutated <- etaM <- NA
    }
    list('A'=cnRes$A,'B'=cnRes$B,'mutated'=mutated,'psiPost'=psiPost,'etaA'=cnRes$etaA,'etaB'=cnRes$etaB,'etaM'=etaM)
  })
  ## End: Outer loop, over psi vectors
  psiPosts <- sapply(1:length(res), function(j) {res[[j]]$psiPost})
  psiPosts <- psiPosts + kPriorset + dirichletPriors
  pick <- which.max(psiPosts)
  A <- res[[pick]]$A
  B <- res[[pick]]$B
  if(is.null(dim(A))) {
    A <- matrix(A, nrow=1)
    B <- matrix(B, nrow=1)
  }
  if(nrow(A)>0){
    rownames(A) <- rownames(B) <- rownames(cndata.filt)
  }
  list(A = A, B = B,
       mutated = res[[pick]]$mutated,
       etaA = res[[pick]]$etaA,
       etaB = res[[pick]]$etaB,
       etaM = res[[pick]]$etaM,
       psiPosts = psiPosts)
}

###Other functions
psiFilter <- function(psis, psi.center, range) {
  diff <- t(t(psis) - psi.center)
  bin <- abs(diff) <= range
  pass <- which(sapply(1:nrow(bin),function(k) {length(which(bin[k,]))==ncol(bin)}))
  list('psis'=psis[pass,], 'indices'=pass)
}

#'snp data' means cgh data unless there is no cgh data, in which case we substitute in snp info from seq data
seekClones <- function(cndata, vardata, cnmodels, psiset, pars, imputedCN=NULL) {
  if(is.null(cndata)) {
    if(is.null(imputedCN)) {
      seqsnps <- vardata[vardata$status=='germline',]
      if(nrow(seqsnps)>0){
        snpdata <- seqSeg(seqsnps, len=100, thresh=.25)
      }else{
        snpdata <- data.frame('chr'=numeric(),'seg'=numeric(),'LRR'=numeric(),'BAF'=numeric(),
                              'X'=numeric(),'Y'=numeric(),'markers'=numeric())
      }
    } else {
      snpdata <- imputedCN
    }
  } else {
    snpdata <- cndata
    if(is.null(snpdata$X)){
      tcn <- 2*10^snpdata$LRR
      snpdata$X <- tcn*snpdata$BAF
      snpdata$Y <- tcn*(1-snpdata$BAF)
    }
  }
  #There's no reason vardata should have class 'logical' unless it's NA
  if(is.null(vardata) | inherits(vardata, 'logical')) {
    seqdata <- data.frame("chr"=numeric(),"seg"=numeric(),"mut.id"=numeric(),"refCounts"=numeric(),
                          "varCounts"=numeric(),"VAF"=numeric(),"totalCounts"=numeric(),'status'=character())
    mu <- NA
  } else {
    seqdata <- vardata
    if(nrow(vardata)>0) {
      read.den <- density(vardata$refCounts)
      peak <- read.den$x[which.max(read.den$y)]
      mu <- peak
      pars$mu <- mu
      pars$sigma.counts <- sd(vardata$refCounts)
    }
  }
#was:  cn.filt <- filter(snpdata, threshold=pars$thresh, cutoff=pars$cutoff)
  cn.filt <- filterCN(snpdata, threshold=pars$thresh)
  cndata.filt <- cn.filt$mat
  indices.cn <- cn.filt$indices
  mutdata <- seqdata[seqdata$status=='somatic',]
  mut.filt <- filterMutations(mutdata, mu=mu, threshold=3)
  mutdata.filt <- mut.filt$mat
  mutids.filt<- mut.filt$ids
  kPriors <- dgeom((1:5)-1, prob=pars$ktheta, log=TRUE)
  psis <- psiset
  kmax <- ncol(psis)
  temp <- psiOptim(cndata.filt, mutdata.filt, psis, cnmodels, pars,
                   cnmax = 5, kmax = kmax, kPriors = kPriors)
  A <- temp$A
  B <- temp$B
  mutated <- temp$mutated
  etaA <- temp$etaA
  etaB <- temp$etaB
  etaM <- temp$etaM
  posts <- temp$psiPosts
  maxPost <- max(posts)
  iters <- pars$iters
  Q <- pars$Q
  iter <- 1
  logPosts <-  rep(NA, (Q)*(iters-1)+nrow(psis))
  psibank <- matrix(NA, ncol=ncol(psis), nrow=length(logPosts))
  while(iter <= iters) {
    if(iter == 1) {
      bank.indices <- 1:nrow(psis)
    } else {
      bank.indices <- min(which(is.na(psibank[,1]))):(min(which(is.na(psibank[,1]))) + Q - 1)
    }
    if(max(posts) > maxPost) {
      A <- temp$A
      B <- temp$B
      mutated <- temp$mutated
      etaA <- temp$etaA
      etaB <- temp$etaB
      etaM <- temp$etaM
      maxPost <- max(posts)
    }
    logPosts[bank.indices] <- posts
    psibank[bank.indices,] <- psis
    max.index <- max(bank.indices)
    postset <- logPosts[1:max.index]
    probs <- exp(postset)
    if(length(which(is.infinite(probs)))==length(probs)) {
      probs <- rep(1/length(probs), length(probs))
    }
    probs[which(is.infinite(probs))] <- max(c(1,max(probs[which(is.finite(probs))])))
    if(is.infinite(sum(probs))){
      power <- log10(length(probs)) + log10(mean(probs))
      divideBy <- 10^(power-308)
      probs <- probs/divideBy
    }
    probs <- probs/sum(probs)
    if(length(which(is.na(probs)))==length(probs)) {
      min.post <- min(postset)
      adjusted <- postset - min.post
      adjusted <- adjusted + min(adjusted)
      probs <- adjusted/sum(adjusted)
      if(length(which(is.na(probs)))==length(probs)) {
        probs <- rep(1/length(probs),length(probs))
      }
    }
    indices <- sample(1:max.index, Q, replace=TRUE, prob=probs)
    alphas <- psibank[indices,]
    psis <- t(sapply(1:nrow(alphas),function(j) {
      temp <- as.vector(rdirichlet(1,alpha=alphas[j,]))
      if(length(temp)<kmax) {
        temp <- c(temp, rep(0, kmax-length(temp)))
      }
      sort(temp, decreasing=TRUE)
    }))
    psis <- t(sapply(1:nrow(psis),function(x) {
      foo <- psis[x,]
      foo[foo<pars$thresh] <- 0
      foo/sum(foo)
    }))
    temp <- psiOptim(cndata.filt, mutdata.filt, psis, cnmodels, pars,
                     cnmax = 5, kmax = kmax, kPriors = kPriors)
    posts <- temp$psiPosts
    iter <- iter + 1
  }
  max.index <- which.max(logPosts)
  psi <- psibank[max.index,]
  psi[which(psi<pars$thresh)] <- 0
  psi <- psi/sum(psi)
  res <- list('indices'=list('indices.cn' = indices.cn, 'mutids.filt' = mutids.filt),
      'data'=list('seqdata'=seqdata, 'snpdata'=snpdata),
      'filtered.data'=list('mutdata.filt'=mutdata.filt,'cndata.filt'=cndata.filt),
      'psibank' = psibank,'etaA'=etaA,'etaB'=etaB,'etaM'=etaM,'psi'=psi,'A'=A,'B'=B,'mutated'=mutated,'psiPosts'=logPosts,
      'pars'=pars,'max.index'=max.index)
  res
}

library(CloneSeeker)

new("WeightVector")
new("WeightVector", 1:3)
try( new("WeightVector", 0) )
try( new("WeightVector", -1:3) )

WeightVector(1:4)
try( WeightVector(0) )
try( WeightVector(-1:3) )
try( WeightVector(LETTERS[1:3]) )
try( WeightVector(c(1, 2, 'a')) )

wv <- WeightVector(4:1)
as(wv, "numeric")

try( as.numeric(wv) ) # doesn't play well in the methods sandbox

canCoerce(wv, "numeric")
# bug in R core fixed in going from 3.5 to 3.6
ifelse(getRversion() < "3.6.0",
  !canCoerce(wv, "double"),   # fails in older versions
  canCoerce(wv, "double"))    # works in newer versions

### want to use Tumor objects as input
generateTumorData <- function(tumor, snps.seq, snps.cgh, mu, sigma.reads, sigma0.lrr, sigma0.baf, density.sigma){
  if(snps.cgh > 0){
    cndat <- snpDataGen(tumor, snps.cgh, sigma0.lrr, sigma0.baf, density.sigma)
  }else{
    cndat <- NA
  }
  if(snps.seq){
    seqdat <- seqDataGen(tumor, snps.seq, density.sigma, mu, sigma.reads)
  }else{
    seqdat <- NA
  }
  list('cn.data'=cndat, 'seq.data'=seqdat)
}

### TODO: Document the algorithm
snpDataGen <- function(tumor, snps.cgh=600000, sigma0.lrr=.01, sigma0.baf=.01, density.sigma=.1){
  psi <- as(tumor@psi, "numeric")
  cn.clones <- lapply(1:length(tumor@clones), function(i){tumor@clones[[i]]$cn})
  chr <- cn.clones[[1]]$chr
  eta <- data.frame('A'=Reduce('+', lapply(1:length(which(psi>0)), function(j){psi[j]*cn.clones[[j]]$A})), 
                    'B'=Reduce('+', lapply(1:length(which(psi>0)), function(j){psi[j]*cn.clones[[j]]$B})))
  eta.total <- eta$A + eta$B
  mu.baf <- eta$B/eta.total
  mu.lrr <- log10(eta.total/2)
  snp.dens <- rbeta2(1:nrow(eta), .5, density.sigma)
  markers <- as.vector(rmultinom(1, size=snps.cgh, prob=snp.dens))
  sigmas.lrr <- sigma0.lrr/(markers)^.5
  sigmas.baf <- sigma0.baf/(markers)^.5
  lrr <- rnorm(length(markers), mean=mu.lrr, sd=sigmas.lrr)
  baf <- rnorm(length(markers), mean=mu.baf, sd=sigmas.baf)
  baf[baf<0] <- -baf[baf<0]
  baf[baf>1] <- 1/baf[baf>1]
  total.backcomp <- 2*(10^(lrr))
  X <- total.backcomp*(baf)
  Y <- total.backcomp*(1 - baf)
  segdf <- data.frame('chr'=chr, seg=1:nrow(eta), 'LRR'=lrr, 'BAF'=baf, 'X'=X, 'Y'=Y, 'markers'=markers)
  segdf
}

#snp.info is an option for an already made snp df (like from snpDataGen) to generate
#read counts from those in addition to generating new ones.
seqDataGen <- function(tumor, snps.seq=1000000, density.sigma, mu, sigma.reads){
  cn.clones <- lapply(1:length(tumor@clones), function(i){tumor@clones[[i]]$cn})
  chrs <- cn.clones[[1]]$chr
  psi <- as(tumor@psi, "numeric")
  eta <- data.frame('A'=Reduce('+', lapply(1:length(which(psi>0)), function(j){psi[j]*cn.clones[[j]]$A})), 
                    'B'=Reduce('+', lapply(1:length(which(psi>0)), function(j){psi[j]*cn.clones[[j]]$B})))
  eta.total <- eta$A + eta$B
  snp.dens <- rbeta2(1:nrow(eta), .5, density.sigma)
  markers <- as.vector(rmultinom(1, size=snps.seq, prob=snp.dens))
  starts <- cn.clones[[1]]$start
  ends <- cn.clones[[1]]$ends
  snpdf <- matrix(NA, nrow=sum(markers), ncol=7)
  colnames(snpdf) <- c('chr', 'seg', 'mut.id', 'refCounts', 'varCounts', 'VAF', 'totalCounts')
  for(j in 1:length(markers)){
    z <- sample(1:2, 2, replace=FALSE)
    pR <- eta[j, z[1]]/(eta[j, z[1]]+eta[j, z[2]])
    totalCounts <- round(rnorm(markers[j], 2*mu, sigma.reads))
    totalCounts[totalCounts<0] <- -totalCounts[totalCounts<0]
    refCounts <- rbinom(markers[j], totalCounts, pR)
    varCounts <- totalCounts - refCounts
    chr <- rep(chrs[j], markers[j])
    VAFs <- varCounts/(totalCounts)
    seg <- rep(j, markers[j])
    for(k in 1:length(refCounts)){
      snpdf[sum(markers[1:j]) - markers[j] +k, ] <- c(chr[k], seg[k], NA, refCounts[k], varCounts[k], VAFs[k], totalCounts[k])
    }
  }
  snpdf <- as.data.frame(snpdf)
  snpdf$status <- rep('germline', nrow(snpdf))
  seq.clones <- lapply(1:length(tumor@clones), function(i){tumor@clones[[i]]$seq})
  cn.clones <- lapply(1:length(tumor@clones), function(i){tumor@clones[[i]]$cn})
  ## Here be monsters. Next part fails if norm.contam was set to TRUE earlier.
  mutids <- unique(unlist(lapply(1:length(which(psi > 0)), function(j) {
    seq.clones[[j]]$mut.id
  })))
  mutdf <- matrix(NA, nrow=length(mutids), ncol=7)
  colnames(mutdf) <- c('chr', 'seg', 'mut.id', 'refCounts', 'varCounts', 'VAF', 'totalCounts')
  if(length(mutids)>0){
    for(j in 1:length(mutids)){
      indices <- which(sapply(1:length(which(psi>0)), function(k){mutids[j] %in% seq.clones[[k]]$mut.id}))
      indices.unmutated <- which(sapply(1:length(which(psi>0)), function(k){!mutids[j] %in% seq.clones[[k]]$mut.id}))
      coefs <- psi[indices]
      mutated <- sapply(indices, function(k){seq.clones[[k]]$mutated.copies[which(seq.clones[[k]]$mut.id==mutids[j])]})
      normal <- sapply(indices, function(k){seq.clones[[k]]$normal.copies[which(seq.clones[[k]]$mut.id==mutids[j])]})
      seg <- seq.clones[[indices[1]]]$seg[which(seq.clones[[indices[1]]]$mut.id==mutids[j])]
      if(length(indices.unmutated)>0){
        coefs.unmutated <- psi[indices.unmutated]
        eta.unmutated <- sum(psi[indices.unmutated]*sapply(indices.unmutated, function(i){
          cn.clones[[i]]$A[seg]+cn.clones[[i]]$B[seg]}))
      }else{
        eta.unmutated <- 0
      }
      eta <- c(sum(coefs*mutated), sum(coefs*normal) + eta.unmutated)
      pR <- eta[2]/(eta[2]+eta[1])
      totalCount <- round(rnorm(1, 2*mu, sigma.reads))
      totalCount[totalCount<0] <- -totalCount[totalCount<0]
      refCount <- rbinom(1, totalCount, pR)
      varCount <- totalCount - refCount
      VAF <- varCount/(varCount+refCount)
      chr <- seq.clones[[indices[1]]]$chr[which(seq.clones[[indices[1]]]$mut.id==mutids[j])]
      mutdf[j, ] <- c(chr, seg, mutids[j], refCount, varCount, VAF, totalCount)
    }
  }
  mutdf <- as.data.frame(mutdf)
  mutdf$status <- rep('somatic', nrow(mutdf))
  vardf <- rbind(snpdf, mutdf)
  vardf <- vardf[with(vardf, order(seg)), ]
  vardf
}


#A plot function to visualize data and verify that the simulation is working.
plotTumorData <- function(tumor, data){
  snpdata <- data$cn.data
  seqdata <- data$seq.data
  cn.clones <- lapply(1:length(tumor@clones), function(i){tumor@clones[[i]]$cn})
  psi <- as(tumor@psi, "numeric")
  markers <- snpdata$markers
  starts <- sapply(1:length(markers), function(j){sum(markers[1:j])-markers[j]})
  ends <- sapply(1:length(markers), function(j){sum(markers[1:j])})
  eta <- data.frame('A'=Reduce('+', lapply(1:length(psi), function(j){psi[j]*cn.clones[[j]]$A})), 
                    'B'=Reduce('+', lapply(1:length(psi), function(j){psi[j]*cn.clones[[j]]$B})))
  lesser <- sapply(1:nrow(eta), function(j){min(eta$A[j], eta$B[j])})
  lesser.data <- sapply(1:nrow(eta), function(j){min(snpdata$X[j], snpdata$Y[j])})
  greater <- sapply(1:nrow(eta), function(j){max(eta$A[j], eta$B[j])})
  greater.data <- sapply(1:nrow(eta), function(j){max(snpdata$X[j], snpdata$Y[j])})
  errors <- c(lesser.data - lesser, greater.data - greater)
  opar <- par(mfrow=c(3, 1))
  on.exit(par(opar))
  plot(0, type='n', xlim=c(0, sum(markers)), ylim=c(0, 5), main='Number of Lesser Allele Copies', 
       xlab='Marker Index', ylab='Expected Copy Number')
  sapply(1:nrow(eta), function(j){
    segments(x0=starts[j], x1=ends[j], y0=lesser[j], y1=lesser[j], col='black')
    segments(x0=starts[j], x1=ends[j], y0=lesser.data[j], y1=lesser.data[j], col='red')
  })
  plot(0, type='n', xlim=c(0, sum(markers)), ylim=c(0, 5), main='Number of Greater Allele Copies', 
       xlab='Marker Index', ylab='Expected Copy Number')
  sapply(1:nrow(eta), function(j){
    segments(x0=starts[j], x1=ends[j], y0=greater[j], y1=greater[j], col='black')
    segments(x0=starts[j], x1=ends[j], y0=greater.data[j], y1=greater.data[j], col='blue')
  })
  plot(density(errors), xlab='Error Index', main='Density of Segment Errors')
  invisible(data)
}

%\VignetteIndexEntry{CloneSeeker}
%\VignetteKeywords{SNP,Copy Number,clone,subclone}
%\VignetteDepends{stats}
%\VignettePackage{CloneSeeker}
\documentclass{article}
\usepackage{Sweave}
\addtolength{\textwidth}{1in}
\addtolength{\oddsidemargin}{-.5in}
\setlength{\evensidemargin}{\oddsidemargin}
\newcommand{\code}[1]{\texttt{#1}}
\title{CloneSeeker}
\author{Mark Zucker \and Kevin R. Coombes}
\date{November, 2018} 

\begin{document}
\SweaveOpts{concordance=TRUE}
\maketitle
\tableofcontents
\section{Introduction}
Tumors often consist of multiple distinct subpopulations or
clones. Information about the number of clones present in a tumor can
be inferred using either mutation allele frequency data, from
sequencing studies, or from copy number varants (CNVs), derived either
from sequencing or from SNP array data. The \code{CloneSeeker} package
can be applied to SNP array data, sequencing data, or both, from tumor
cells from a cancer patient. \code{CloneSeeker} can determine the
number of clones, the distribution of cells among clones, and the copy
number variations and mutations (depending on the available data
sources) that occur in each clone. The presence of multiple detectable
clones is called ``clonal heterogeneity'' in the literature.

Clonal heterogeneity likely plays an important role in the clinical
course of a cancer. It is possible, for example, that the tumor cells
that will eventually become the refractory cancer after treatment are
present as a minor subclone in the tumor early on.

First, we load the \code{CloneSeeker} package:
<<lib>>=
library(CloneSeeker)
@ 

\section{Simulated Tumor Containing Multiple Clones}
In order to illustrate the algorithms, we are going to simulate data
where we know the true structure. Specifically, we will simulate copy
number and mutation data for a tumor with three clones.  We start with
an object that represents the Tumor at a somewhat abstract level.
<<simTumor>>=
set.seed(21303) # for reproducibility
simTumor <- Tumor(c(5, 3, 2), rounds = 100, 
                  nu = 10, pcnv = 0.8, norm.contam = FALSE)
@ 
The first argument to the \code{Tumor} constructor is a vector that
specifies the relative proportions of cells belonging to each
clone; the length of the vector determines the number of clones. These
values are automatically converted to fractions that add up to one:
<<truePsi>>=
simTumor@psi
@ 

The second argument, \code{rounds}, specifies the number of
generations through which the tumor clones are evolved. The idea is
that new abnormalities, either in the form of mutations or copy number
variants (CNVs), are acquired at each evolutionary step from some parent
cell. The parameter \code{nu} is the expected number of new mutations
and the parameter \code{pcnv} is the probability of a new CNV at each
step. The final parameter, \code{norm.contam}, is a logical indicator
of whether the tumor sample is assumed to include a subset of cells
that represent non-cancerous ``normal contamination''.

The resulting simulated tumor contains descriptions of each
individual clone. In the current implementation, these are stored as a
list of clones.
<<cloneMeta>>=
class(simTumor@clones)
length(simTumor@clones)
@ 
Individual clones contain descriptions of both CNVs and mutations.
<<oneClone>>=
oneClone <- simTumor@clones[[1]]
class(oneClone)
length(oneClone)
names(oneClone)
@ 
The copy number data includes the chromosome, with start and end
positions, the number of copies of the \code{A} and \code{B} alleles,
an arbitrary ``segment'' identifier, and (as a residual from the
simulated evolutionary history), a ``parent'' identifier.
<<oneCN>>=
dim(oneClone$cn)
summary(oneClone$cn)
@ 
The mutation data has a chromosomal location, arbitrary segment and
mutation identifiers, the number of mutated and wild type copies
for each mutation, and the affected allele.
<<oneMut>>=
dim(oneClone$seq)
oneClone$seq
@ 

\subsection{Simulating Tumor Data}
Now that we have the tumor in place, we can simulate data arising
from a sudy of that tumor.
<<simData>>=
simData <- generateTumorData(simTumor,
                             snps.seq = 10000,
                             snps.cgh = 600000,
                             mu = 70,
                             sigma.reads = 25,
                             sigma0.lrr = 0.15,
                             sigma0.baf = 0.03,
                             density.sigma = 0.1)
@ 

For a description of the many parameters to the
\code{generateTumorData} function, see the man page.  The first two
arguments are size parameters. The first, \code{snp.seq}, determines
the number of \textit{germline} variants to simulate; in the absence
of separate copy number data, these are used to provide a crude
estimate. The second, \code{snps.cgh}, represents the number of SNP
locations on the simulated SNP chip from which copy number segments
are derived.  The remaining parameters control the simulated read depth
and variabilty.

As with individual clones, the simulated data is structured as a list
with separate data frames for the CNVs and mutations.
<<peekData>>=
class(simData)
length(simData)
names(simData)
@ 
The simulated copy number data includes chromosomal locations along
with estimated log R ratios (\code{LRR}), B allele frequencies
(\code{BAF}), separate intensity values for the two parental alleles
(\code{X} and \code{Y}), and the number of SNPs in each segment
(\code{markers}).
<<simCNData>>=
cnDat <- simData$cn.data
dim(cnDat)
summary(cnDat)
@ 

The simulated sequencing data, in addition to chromosomal locations,
has read counts for the number of reference alleles, alternate
(meaning varianmt or mutated) alleles, total counts, the variant
allele frequency (\code{VAF}), and a status indicator of whether the
variant is believed to be germline or somatic.
<<simMutData>>=
dim(simData$seq.data)
seqDat <- simData$seq.data
somatic <- seqDat[seqDat$status=='somatic',]
dim(seqDat)
summary(seqDat)
table(seqDat$status)
@ 


\section{Seeking Clones}

To run \code{CloneSeeker}, we will need a starting set of $\psi$
vectors as inputs, where $\psi$ records the fraction of cells
belonging to each clone. For each $\psi$ vector, the algorithm will
compute the most probable copy number state for each clone at each
segment. The maximum posterior probability is computed for each input
$\psi$ vector, and these probabilities are used to resample new potential
$\psi$ vectors. We usually start by considering every possible
decomposition of the tumor into five clones, where the fraction
assigned to each clone is a multiple of $1/20 = 0.05$.  We can
generate this initial matrix of $\psi$ vectors as follows:
<<psis>>=
psis <- generateSimplex(20, 5)
dim(psis)
head(psis)
tail(psis)
@

For SNP array data, we also need, as input, a set of possible clonal
segment copy number states. If none exists the function will
automatically generate one. The version used here considers all
possible copy number states from 0 to 5 copies, but it imposes a
strong prior belief that two different clones cannot both gain and
lose the same segment.
<<cnmodels>>=
cnmodels <- expand.grid(rep(list(0:5),5))
include <- sapply(1:nrow(cnmodels), function(i) {
  length(which(cnmodels[i,] >= 1))==5 | length(which(cnmodels[i,] <= 1)) == 5
})
cnmodels <- cnmodels[include,]
@

Now we will define the other algorithm parameters:
<<pars>>=
pars <- list(sigma0 = 1,    # SNP-wise standard deviation
             ktheta = 0.3,  # geometric prior parameter on number of clones
             theta = 0.9,   # geometric prior parameter on copy number changes
             mtheta = 0.9,  # gemoetric prior parameter on point mutations
             alpha = 0.5,   # parameter for a symmetric Dirichlet prior on psi
             thresh = 0.04, # smallest possible detectble clone
             cutoff = 100,  # filter out copy number segments supported by fewer SNPs
             Q = 100,       # number of new psi vectors resamples at each iteration
             iters = 4)     # number of iterations
@

<<realWork,echo=FALSE,eval=TRUE>>=
f <- system.file("auxiliary/stash.Rda", package="CloneSeeker")
if (file.exists(f)) {
  load(f)
} else {
  resCN <- seekClones(cndata = cnDat, vardata = NULL, cnmodels = cnmodels, psis, pars = pars)
  resMut <- seekClones(cndata = NULL, vardata = seqDat, cnmodels = cnmodels, psiset = psis, pars = pars)
  resBoth <- seekClones(cndata = cnDat, vardata = somatic, cnmodels = cnmodels, psis, pars = pars)
  save(resCN, resMut, resBoth, file = "../inst/auxiliary/stash.Rda")
}
rm(f)
@ 

\subsection{Seeking Clones from Copy Number Data}
The \code{seekClones} function can estimate the clonal architecture
from copy number data, or from mutation and variant data, or jointly
from both kinds of data.  In this section, we will run the algorithm
using \textbf{only the copy number data}. To do that, we set the
\code{varData} argument to \code{NULL}.
<<resCN,eval=FALSE>>=
resCN <- seekClones(cndata = cnDat, vardata = NULL, 
                    cnmodels = cnmodels, psiset = psis, pars = pars)
@
Here are the results of the ``CNV only'' analysis of this sample:
<<compare.psis>>=
resCN$psi
simTumor@psi
@
In this case, \code{CloneSeeker} accurately estimates not only the
number of clones but also the clonal fractions. Let's look at the
clonal copy number assignments as well:
<<CNassign>>=
trueCN_Assignments <- t(sapply(1:nrow(resCN$filtered.data$cndata.filt),
function(i) {
  index <- rownames(simTumor@clones[[1]]$cn) == 
           rownames(resCN$filtered.data$cndata.filt)[i]
  sapply(1:length(simTumor@clones),function(j){
    simTumor@clones[[j]]$cn$A[index] + simTumor@clones[[j]]$cn$B[index]
  })
}))

inferredCN_Assignments <- (resCN$A+resCN$B)[,1:length(simTumor@clones)]
colnames(inferredCN_Assignments) <- colnames(trueCN_Assignments) <- 
  paste("C", 1:3)

data.frame(Truth = trueCN_Assignments,
           Infer = inferredCN_Assignments)
@
Although not perfect, the algorithm managed to correctly estimate  most of the
segment-wise allelic copy numbers of different clones.

\subsection{Sequencing Data}
Now, let's illustrate the use of \code{CloneSeeker} in analyzing mutation
data (by which we mean variant data such as one would find in a \code{.vcf}
file) to seek clones.  This time, we run the \code{CloneSeeker}
algorithm with the \code{cndata} argument set to \code{NULL}.
<<resMut, eval=FALSE>>=
resMut <- seekClones(cndata = NULL, vardata = seqDat, 
                     cnmodels = cnmodels, psiset = psis, pars = pars)
@
Here the results aren't as good; at least one of the actual clones
has been split into separate pieces.
<<resMutResults>>=
resMut$psi
simTumor@psi
@
%Let's also look at the mutation assignment compared to the truth:
<<trueMut,echo=FALSE,eval=FALSE>>=
trueMutAssignments <- sapply(1:length(simTumor@clones),function(i){
  sapply(1:length(resMut$filtered.data$mutdata.filt$mut.id),function(j){
    index <- which(simTumor@clones[[i]]$seq$mut.id==resMut$filtered.data$mutdata.filt$mut.id[j])
    if(length(index)==0){
      val <- 0
    }else{
      val <- simTumor@clones[[i]]$seq$mutated.copies[index]
    }
    val
  })
})
inferredMutAssignments <- resMut$mutated[,1:length(simTumor@clones)]
colnames(inferredMutAssignments) <- colnames(trueMutAssignments) <- c('C1','C2','C3')

data.frame(Truth = trueMutAssignments,
           Infer = inferredMutAssignments)
@

\subsection{Both Sequencing and SNP Array Data}
Finally, we illustrate running \code{CloneSeeker} on a sample for
which there is both SNP array and mutation data. 
<<resBoth,eval=FALSE, echo=TRUE>>=
resBoth <- seekClones(cndata = cnDat, vardata = somatic, 
                      cnmodels = cnmodels, psiset = psis, pars = pars)
@

And we can look at the inferred allocation of tumor fraction to clones:
<<reBothResults>>=
resBoth$psi
simTumor@psi
@
Surprisingly, the results here are similar to the overaggressive
results obtained using just the sequencing data rather than the simpler
and correct results obtained when using just the copy number data.

In conclusion, CloneSeeker can be applied effectively to cases where
one has SNP array data, (processed) sequencing data, or both. 

\end{document}

\name{Simplices}
\alias{Simplices}
\alias{sampleSimplex}
\alias{generateSimplex}

\title{ Simplices and Clonal Fractions }
\description{
  Utility functions for working with vectors of clonal fractions.
}
\usage{
sampleSimplex(n, d = 5)
generateSimplex(k, d, reps = 1)
}
\arguments{
  \item{d}{ an integer, the dimension of the simplex, or the number of
    clones. }
  \item{n}{ an integer, the number of vectors to sample randomly.  }
  \item{k}{ an integer, the number of equally spaced points to select along
  each side of the simplex while constructing a lattice. }
  \item{reps}{ an integer, the number of times to repeat the lattice. }
}
\details{
  When studying the clonal subpopulations of a tumor sample, we
  frequently need access to vectors that contain the fraction of cells
  belonging to each clone. These vectors are characterized by the fact
  that each entry is nonzero and they must add up to 1. The set of such
  vectors/points in d-dimensional space defines the "d-simplex". The
  functions defined here allow us to work with d-simplices, either by
  randomly sampling vectors (\code{sampleSimplex}) or by systematically
  filling the space with a regular lattice (\code{generateSimplex}).
}
\value{
  Both functions return a matrix with \code{d} columns. Each row
  contains nonzeo real numbers that sum to 1. The \code{generateSimplex}
  function ensures that (a) each row is unique and (b) the entries in
  each row appear in decreasing order. 
}
\author{Kevin R. Coombes \email{krc@silicovore.com},
  Mark Zucker \email{zucker.64@buckeyemail.osu.edu}
}
\examples{
sampleSimplex(5, 3)
generateSimplex(5, 3)
}
\keyword{array}

\name{Tumor-class}
\alias{Tumor-class}
\alias{Tumor}
\alias{getClone}
\alias{summary,Tumor-method}
\alias{coerce,Tumor,list-method}
\docType{class}
\title{Class "Tumor"}
\description{
  A class that represents tumors, thought of as a collection of
  (sub)clones each with an associated measure as a fraction of all
  tumor cells.
}
\usage{
Tumor(psi, rounds, nu = 100, pcnv = 0.5, norm.contam = FALSE, cnmax = 4)
getClone(tumor, i)
}
\arguments{
  \item{psi}{ a numeric vector containing non-negative values.}
  \item{rounds}{an integer; the number of generations through which to
    evolve the potential clones.}
  \item{nu}{an integer; the expected number of mutations in each clonal
    generation.}
  \item{pcnv}{a real number between 0 and 1; the probability of a CNV
    occuring per generation.}
  \item{norm.contam}{a logical value; should we treat one of the cell
    populations as normal cells that are "contaminating" the tumor
    specimen?}
  \item{cnmax}{an integer, the maximum copy number allowed in the
    simulated data.}
  \item{tumor}{ an object of the \code{Tumor} class.}
  \item{i}{a integer; which clone to extract.}
}
\details{
  The \code{Tumor} class is used to represent compex tumors, each of
  which consists a set of subclones representing different fractional
  parts of the tumor. Each clone is characterized by a set of copy
  number variants (modeled by the output produced by something like the
  \code{DNAcopy} package) and, optionally, a set of sequence
  mutations. Each of these genetic events is mapped to a specific
  interval or point in the human genome.
  
  In the current implementation, a \code{Tumor} consists of a weight
  vector that specifies the fractions of cells for each clone and a list
  of clones. At present, each clone is itself a list containing one (if
  there are no mutations) or two (if there are both copy number
  variants and mutations) data frames. This structure is likely to
  change in later versions of the package, since we expect to implement a
  full-fledged S4 class to represent clones. So, one should not rely on
  the current implementation. 
}
\value{
  The constructor returns a valid object of the \code{Tumor}
  class.
}
\section{Objects from the Class}{
  Although objects can be created using \code{new}, the preferred method
  is to use the constructor function, \code{Tumor}.
}
\section{Slots}{
  \describe{
    \item{\code{psi}}{ a \code{\link{WeightVector}} containing
      non-negative values whose sum  equals one. }
    \item{\code{clones}}{ a list, each of whose elements represents a
      clone.}
  }
}
\section{Methods}{
  \describe{
    \item{coerce(from, to, strict = TRUE)}{Convert the
      \code{Tumor} object into a simple numeric vector. Never
      actually used in this form, since the preferred method is to write
      \code{as(WV, "list")}.}
  }
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com},
  Mark Zucker \email{zucker.64@buckeyemail.osu.edu}
}
\seealso{
  \code{\link{WeightVector}}
}
\examples{
showClass("Tumor")

tumor <- Tumor(c(0.5, 0.3, 0.2), 100)
simpleTumor <- as(tumor, "list")
reformed <- as(simpleTumor, "Tumor")
}
\keyword{classes}

\name{Simulating Clones}
\alias{generateTumorData}
\alias{plotTumorData}
\alias{tumorGen}
\alias{dataGen}
\title{ Simulating Tumor Clones }
\description{
  Simulating copy number segmentation data and sequencing mutation data
  for tumors composed of multiple clones.
}
\usage{
generateTumorData(tumor, snps.seq, snps.cgh, mu, sigma.reads,
                  sigma0.lrr, sigma0.baf, density.sigma)
plotTumorData(tumor, data)
tumorGen(...)
dataGen(tumor, ...)
}
\arguments{
  \item{tumor}{ an object of the \code{Tumor} class. }
  \item{snps.seq}{an integer; the total number of germline variants and
  somatic mutations to simulate in the tumor genome.}
  \item{snps.cgh}{an integer; the number of single nucleotide
    polymorphisms (SNPs) to simulate as meaurements made to estimate
    copy number.}
  \item{mu}{an integer; the average read depth of a simulated sequencing
    study giving rise to mutations.}
  \item{sigma.reads}{a real number; the standard deviation of the number
    of simulated sequencing reads per base.}
  \item{sigma0.lrr}{a real number; the standard deviation of the
    simulated per-SNP log R ratio (LRR) for assessing copy number.}
  \item{sigma0.baf}{a real number; the standard deviation of the
    simulated B allele frequency (BAF) for assessing copy number.}
  \item{density.sigma}{a real number; the standard deviation of a beta
    distribution used to simulate the number of SNP markers per copy
    number segment.}
  \item{data}{ a list containing two data frames, \code{cn.data} and
    \code{seq.data}, as produced by \code{generateTumorData}.}
  \item{...}{additional variables}
}
\details{
  Copy number and mutation data are simulated essentially
  independently. Each simulation starts with a single "normal" genome, 
  and CNVs and/or mutations are randomly generated for each new "branch"
  or subclone. (The number of subclones depends on the input
  parameters.) Each successive branch is randomly determined to descend
  from one of the existing clones, and therefore contains both the
  aberrations belonging to its parent clone and the novel aberrations
  assigned to it. Depending on input parameters, the algorithm can also
  randomly select some clones for extinction in the process of
  generating the heterogeneous tumor, to yield a more realistic
  population structure. 

  Note that \code{tumorGen} (an alias for \code{Tumor} that returns a
  list instead of a Tumor object) and \code{dataGen} (an alias for
  \code{generateTumorData}) are DEPRECATED.
}
\value{
  The \code{generateTumorData} function returns a list with two
  components, \code{cn.data} and \code{seq.data}. Each component is
  itself a data frame. Note that in some cases, one of these data frames
  may have zero rows or may be returned as an \code{NA}.

  The \code{cn.data} component contains seven columns:
  \describe{
    \item{\code{chr}}{the chromosome number;}
    \item{\code{seq}}{a unique segment identifier;}
    \item{\code{LRR}}{simulated segment-wise log ratios;}
    \item{\code{BAF}}{simulated segment-wise B allele frequencies;}
    \item{\code{X} and \code{Y}}{simulated intensities for two separate
      alleles/haplotypes per segment; and} 
    \item{\code{markers}}{the simulated number of SNPS per segment.}
  }

  The \code{seq.data} component contains eight columns:
  \describe{
    \item{\code{chr}}{the chromosome number;}
    \item{\code{seq}}{a unique "segment" identifier;}
    \item{\code{mut.id}}{a unique mutation identifier;}
    \item{\code{refCounts} and \code{varCounts}}{the simulated numbers
      of reference and variant counts per mutation;}
    \item{\code{VAF}}{the simulated variant allele frequency;}
    \item{\code{totalCounts}}{the simulated total number of read counts;
      and}
    \item{\code{status}}{a character (that should probably be a factor)
      indicating whether a variant should be viewed as somatic or
      germline.}
  }

  The \code{plotTumorData} function invisibly returns its \code{data}
  argument.
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com},
  Mark Zucker \email{zucker.64@buckeyemail.osu.edu}
}
\examples{
psis <- c(0.6, 0.3, 0.1) # three clones
# create tumor with copy number but no mutation data
tumor <- Tumor(psis, rounds = 400, nu = 0, pcnv = 1, norm.contam = FALSE)
# simulate the dataset
dataset <- generateTumorData(tumor, 10000, 600000, 70, 25, 0.15, 0.03, 0.1)
#plot it
plotTumorData(tumor, dataset)
}
\keyword{array}

\name{WeightVector-class}
\alias{WeightVector-class}
\alias{WeightVector}
\alias{coerce,WeightVector,numeric-method}
\docType{class}
\title{Class "WeightVector"}
\description{
  A class, with validity checking, to contain vectors of non-negative
  real numbers whose sum equals one.
}
\usage{
WeightVector(phi)
}
\arguments{
  \item{phi}{ a numeric vector containing non-negative values.}
}
\details{
  When trying to simulate or model biological data from (sub)clonal
  populations of cells, we need vectors that keep track of the fraction
  of cells belonging to each clone. These vectors can only contain
  non-negative entries, and the entries must add up to one. (We
  thought about calling these things "ClonalFractions", but that seems
  overly specialized for a notion that is likely to prove useful in
  other contexts.) Such vectors of length \code{d} can also be viewed as
  points of a d-dimensional simplex.

  We have implemented \code{WeightVector}s as an S4 class, primarily so
  we can enforce the defining properties. We also expect this design to
  make it easier to use them as slots in other classes.
}
\section{Objects from the Class}{
  Although objects can be created using \code{new}, the preferred method
  is to use the constructor function, \code{WeightVector}.
}
\section{Slots}{
  \describe{
    \item{\code{psi}}{ a vector containing non-negative values whose sum
      equals one. }
  }
}
\section{Methods}{
  \describe{
    \item{coerce(from, to, strict = TRUE)}{Convert the
      \code{WeightVector} object into a simple numeric vector. Never
      actually used in this form, since the preferred method is to write
      \code{as(WV, "numeric")}.}
  }
}
\value{
  The constructor returns a valid object of the \code{WeightVector}
  class.
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com},
  Mark Zucker \email{zucker.64@buckeyemail.osu.edu}
}
\seealso{
  \code{\link{Simplices}}
}
\examples{
showClass("WeightVector")

WeightVector(1:4)
as(WeightVector(c(2, 3, 5)), "numeric")
}
\keyword{classes}

\name{Seeking Clones}
\alias{seekClones}
\alias{runAlg}
\title{Seeking Tumor Clones From Data}
\description{
  Starting with copy number segmentation data and/or sequencing mutation
  data for a tumor, seek the number of clones, the fraction of cells
  belonging to each clone, and the likely set of abnormalities in each
  clone.
}
\usage{
seekClones(cndata, vardata, cnmodels, psiset, pars, imputedCN = NULL)
runAlg(...)
}
\arguments{
  \item{cndata}{ A data frame with seven columns; can also be NULL. The
    names of the required columns are enumerated in the  man page for
    \code{generateTumorData}. These are the same as the output typically
    produced by the \code{DNAcopy} algorithm, where each row represents a
    segment (a contiguous region of a chromosome) where the copy number is
    believed to be constant.}
  \item{vardata}{ A data frame with eight columns; can also be NULL. The
    names of the required columns are enumerated in the man page for
    \code{generateTumorData}. These are typical of the output of a DNA
    sequencing experiment that has been processed to identify variants,
    which may be either germline or somatic.}
  \item{cnmodels}{ A matrix. Each row represents a model to be considered;
    each column represents a clone.  The entries are integers specifying
    the number of DNA copies present in that clone. See details.}
  \item{psiset}{ A matrix. Each column represents a clone, and each row
    represents a different possible model of the fraction of cells per
    clone. See details. }
  \item{pars}{ A list of algorithm parameters; see details.}
  \item{imputedCN}{a logical value; if missing, should the copy number be
    imputed from the mutation data.}
  \item{...}{additional variables}
}
\details{
  The algorithm starts with an initial set of 'psi' parameters
  (representing the fraction of tumor belonging to each clone). It
  computes the best (maximum a posteriori) clonal copy number and/or
  number of mutated alleles for each clone for each segment/mutation,
  conditional on the data and each of the initial psi vectors. It then
  computes the posterior probability for each psi-vector and its
  computed copy number and mutation parameters. It uses these posterior
  probabilities to resample new possible psi-vectors. The process
  repeats iteratively, and with each iterations obtains a better
  estimate of psi and the clonal segment copy number and mutation
  assignments until it terminates.
   
  The set of copy number models that we use is typically
  generated using the following command:
  \code{as.matrix(expand.grid(lapply(1:5, function(i){0:5})))}
  This setup considers all (7776) possible models with up to five clones,
  where the copy number for each clone ranges from 0 to 5. (In the future,
  we are likely to make this the default; right now, you have to generate
  these models yourself.)

  The set of possible psi-vectors (that is, the fraction of cells allocated
  to each clone) that we use is typically generated using the following
  command:
  \code{psis.20 <- generateSimplex(20,5)}
  This setup considers all (192) possible divisions of the tumor into up to
  five clones, where the fraction of cells per clone is any possible
  multiple of 0.05. Each row is sorted to put the most abundant clones first,
  which makes it easier to identify specific clones, except in the rare case
  when two clones contain exactly the same fraction of cells. (In the future,
  we are likely to make this the default; right now, you have to generate
  these models yourself.)

  The object \code{pars} is a list of numerical algorithm parameters. The
  elements are:

  \describe{
    \item{sigma0}{The standard deviation of measured allelic copy
      number at the SNP level.}
    \item{ktheta}{The probability parameter of the geometric prior
      distribution on \code{K}, the number of clones.} 
    \item{theta}{The probability parameter of the geometric prior
      distribution on genomic copy number.}
    \item{mtheta}{The probability parameter of the geometric prior
      distribution on the occurence of point mutations.}
    \item{alpha}{The (repeated) alpha parameter of a symmetric Dirichlet
      distributed prior on the  fractions of cells belong to each clone;
      default value is 0.5, giving a Jeffreys Prior.}
    \item{thresh}{The threshold determining the smallest possible
      detectable clone.}
    \item{cutoff}{SNP array segments with fewer markers than this are
      excluded.} 
    \item{Q}{Determines the number of new \code{psi} vectors resampled
      from the estimated posterior probability distribution at each
      iteration of the algorithm } 
    \item{iters}{The number of iterations in the algorithm.}
  }
  The default settings we used are from
  commonly used unfinformative priors (e.g., alpha=0.5 for the Dirichlet
  distribution is the Jeffreys Prior) or based on empirical assessments
  of the variation in data (sigma0, for example, which describes
  variation in SNP array data).
  
  Note that \code{runAlg} (an alias for \code{seekClones}) is DEPRECATED.
}
\value{
  The \code{seekClones} function returns a (rather long) list containing:

  \item{psi}{The most likely posterior psi-vector, given the data. The
    number of non-zero entries is the number of clones found, and the non-zero
    entries are the fraction of cells per clone}
  \item{A}{The most likely copy numbers for the A allele in each segment in each clone.}
  \item{B}{The most likely copy numbers for the B allele in each segment in each clone.}
  \item{psibank}{A matrix, where each row is one of the psi-vectors
    considered during the analysis.}
  \item{psiPosts}{A numeric vector, the (marginal) posterior probability of
    each psi-vector considered during the analysis.}
  \item{indices}{???}
  \item{data}{a list with two data-frame components containing the data
    used during the analysis.}
  \item{filtered.data}{a list with two data-frame components containing the
    filtered data used during the analysis. Filtering removes
    non-informative segments that have normal copy number or contain only
    germline mutations.}
  \item{etaA}{A vector of the weighted average allelic copy number for the 'A-Allele' at each segment (that is, the sum of the clonal A-allelic copy number values multiplied by the fraction of the tumor made up by each clone)}
  \item{etaB}{A vector of the weighted average allelic copy number for the 'B-Allele' at each segment}
  \item{etaM}{A vector of the weighted average number of copies of the mutated allele at each mutation}
  \item{mutated}{A matrix of the number of mutated alleles at each locus in each clone, where the number of rows is the number of somatic mutations in the data and the number of columns is the number of clones}
}
\references{
  Zucker MR, Abruzzo LV, Herling CD, Barron LL, Keating MJ, Abrams ZB,
  Heerema N, Coombes KR. Inferring Clonal Heterogeneity in Cancer using
  SNP Arrays and Whole Genome Sequencing. Bioinformatics. To
  appear. doi: 10.1093/bioinformatics/btz057.
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com},
  Mark Zucker \email{zucker.64@buckeyemail.osu.edu}
}
\examples{
# set up models
psis.20 <- generateSimplex(20,5)
cnmodels <- as.matrix(expand.grid(lapply(1:5, function(i){ 0:5 })))
# set up algortihm parameters
pars <- list(sigma0=5, theta = 0.9, ktheta = 0.3, mtheta = 0.9,
             alpha = 0.5, thresh = 0.04, cutoff = 100, Q = 100, iters = 4)
# create a tumor
psis <- c(0.6, 0.3, 0.1) # three clones
tumor <- Tumor(psis, rounds = 100, nu = 0, pcnv = 1, norm.contam = FALSE)
# simulate a dataset
dataset <- generateTumorData(tumor, 10000, 600000, 70, 25, 0.15, 0.03, 0.1)
\donttest{result <- seekClones(dataset$cn.data, dataset$seq.data,
             cnmodels, psis.20, pars = pars, imputedCN = NULL) }
}
\keyword{array}
